/*LICENSE_BEGIN
 * ******************************************************************
 * Copyright 2006 by the Dainamite Robocup-Team at the
 * DAI-Labor, TU-Berlin (www.dai-labor.de)
 * 
 * Members of Dainamite are:
 * Holger Endert
 * Robert Wetzker
 * Thomas Karbe
 * Axel Heßler
 * Philippe Bütter
 * Felix Brossmann
 * 
 * Licensed under the GNU GPL (General Public License) Free Software 
 * License, Version 2. You may use this file only in compliance with 
 * the License. You may obtain a copy of the License at:
 * 
 * http://www.fsf.org/licensing/licenses/gpl.txt
 * ******************************************************************
LICENSE_END*/
package robocup.component.actions;

import robocup.component.ControllerNumbers.VIEW_ANGLE;
import robocup.component.ControllerNumbers.VIEW_QUALITY;

/**
 * This action changes the view-mode of the agent, which is defined
 * by a view-quality and a view-angle. This also defines, how often visual
 * information arrives.
 */
public class ChangeViewModeAction extends Action {

	/**
	 * The view-quality is one of LOW or HIGH
	 */
	private VIEW_QUALITY _quality;

	/**
	 * The view-angle is one of NARROW, NORMAL or WIDE.
	 */
	private VIEW_ANGLE _angle;

	/**
	 * Constructor - action sets the quality and the view-angle
	 * 
	 * @param quality
	 *            HIGH or LOW
	 * @param angle
	 *            NARROW, MEDIUM or WIDE
	 */
	public ChangeViewModeAction(VIEW_QUALITY quality, VIEW_ANGLE angle) {
		this._quality = quality;
		this._angle = angle;
		this._type = TYPE.CHANGE_VIEW_MODE;
	}

	/**
	 * Returns the actions string representation:
	 * @return (change_view &lt;view-angle&gt; &lt;view-quality&gt;)
	 */
	@Override
	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append("(change_view ");
		buf.append(this._angle.getString());
		buf.append(" ");
		buf.append(this._quality.getString());
		buf.append(")");
		return buf.toString();
	}

	/**
	 * @return Returns the angle.
	 */
	public VIEW_ANGLE getAngle() {
		return this._angle;
	}

	/**
	 * @return Returns the quality.
	 */
	public VIEW_QUALITY getQuality() {
		return this._quality;
	}

	/**
	 * The view cone width that results by the action.
	 * 
	 * @return view cone width in degree (45,90 or 180)
	 */
	public double getViewWidth() {
		return this._angle.getFactor() * 90;
	}
}
