/*LICENSE_BEGIN
 * ******************************************************************
 * Copyright 2006 by the Dainamite Robocup-Team at the
 * DAI-Labor, TU-Berlin (www.dai-labor.de)
 * 
 * Members of Dainamite are:
 * Holger Endert
 * Robert Wetzker
 * Thomas Karbe
 * Axel Heßler
 * Philippe Bütter
 * Felix Brossmann
 * 
 * Licensed under the GNU GPL (General Public License) Free Software 
 * License, Version 2. You may use this file only in compliance with 
 * the License. You may obtain a copy of the License at:
 * 
 * http://www.fsf.org/licensing/licenses/gpl.txt
 * ******************************************************************
LICENSE_END*/
package robocup.component.infotypes;

/**
 * A visual info for the ball. Depending on the distance, a different
 * precision of the seen values are given.
 */
public class BallInfo extends DynamicObjectInfo {

	/**
	 * Constructs an info of a seen ball
	 * 
	 * @param cycle -
	 *            when was it seen
	 * @param iDistance -
	 *            the distance to the seen ball
	 * @param iDirection -
	 *            the direction to the seen ball
	 * @param iDistchange -
	 *            the distance change (used for speed) of the seen ball
	 * @param iDirchange -
	 *            the direction change (used for speed) of the seen ball
	 */
	public BallInfo(int cycle, double iDistance, double iDirection,
			double iDistchange, double iDirchange) {
		// der Ball hat keine Id, deshalb -1
		super(cycle, -1, iDistance, iDirection, iDistchange, iDirchange);
	}

	/**
	 * @return String - a debug-string for the seen ball.
	 */
	@Override
	public String toString() {
		return ("BallInfo: Dist:" + this.distance + " Dir: " + this.direction
				+ " dDist: " + this.getDistChange() + " dDir: " + this
				.getDirChange());
	}
}
