/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope;

import java.awt.Event;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.zip.DeflaterOutputStream;
import java.util.zip.InflaterInputStream;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFileChooser;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;

import soccerscope.daistatistic.ActionDialog;
import soccerscope.daistatistic.PlayerDataDialog;
import soccerscope.daistatistic.Register;
import soccerscope.daistatistic.StatisticDialog;
import soccerscope.file.GameLogFileFilter;
import soccerscope.file.PlanLogFileFilter;
import soccerscope.file.ServerLogFileFilter;
import soccerscope.file.SoccerScopeFilter;
import soccerscope.model.ClientScene;
import soccerscope.model.ScenePlayModeList;
import soccerscope.model.SceneSetMaker;
import soccerscope.model.Team;
import soccerscope.model.WorldModel;
import soccerscope.net.SoccerServerConnection;
import soccerscope.util.ButtonManager;
import soccerscope.view.FieldPane;
import soccerscope.view.InformationWindow;
import soccerscope.view.OptionWindow;
import soccerscope.view.PreferenceDialog;
import soccerscope.view.SelectWindow;
import soccerscope.view.StatisticsWindow;
import soccerscope.view.layer.Layer;
import soccerscope.db.RobocupDatabase;
import soccerscope.db.DatabaseReader;
import soccerscope.view.LoadGameDialog;
import soccerscope.view.DatabaseConfigDialog;
import soccerscope.view.StatisticFrame;
import soccerscope.db.DatabaseConfig;

public class SoccerScopeMenuBar extends JMenuBar implements ActionListener,
		ItemListener {
	/**
	 *
	 */
	private static final long serialVersionUID = 3256726186435426101L;

	SoccerScope soccerscope = null;

	// XiRoT start
	// DAI Dialogs/Frames
	ActionDialog actionDialog;

	PlayerDataDialog playerDataDialog;

	StatisticDialog statisticDialog;

	// MenuBar Item
        private JCheckBoxMenuItem dbSaveScene;
        private JMenuItem dbNewGame;
        private JMenuItem dbLoadGame;
        private JMenuItem dbStatistics;
        private JMenuItem dbDBConfig;

    private JMenuItem showActions;

	private JMenuItem showPlayerData;

	private JMenuItem showDaiStatistic;

	private JCheckBoxMenuItem storeDaiWMOnOff;

	private JMenu storeSelection = new JMenu("Refine");

	private JMenuItem no = new JMenuItem("No");

	private JMenuItem all = new JMenuItem("All");

	public JCheckBoxMenuItem storePlayerPosOnOff;

	public JCheckBoxMenuItem storePlayerSpeedOnOff;

	public JCheckBoxMenuItem storeBallPosOnOff;

	public JCheckBoxMenuItem storeBallSpeedOnOff;

	public JCheckBoxMenuItem storeBallIPOnOff;

	public JCheckBoxMenuItem storeAnglesOnOff;

	public JCheckBoxMenuItem storeTeammatesAndOpponentsOnOff;

	public JCheckBoxMenuItem storePosStatisticsOnOff;

	public JCheckBoxMenuItem storeOppIPOnOff;

	// XiRoT end

	// MenuBar Item
	private JMenuItem save;

	private JMenuItem load;

	private JMenuItem openMI;

//	private JMenuItem loadMI;

	private JMenuItem loadplanMI;

	private JMenuItem closeMI;

	private JMenuItem preferenceMI;

	private JMenuItem loadlogMI;

	private JMenuItem quitMI; // menu item for system.exit(0)

	private JMenuItem exitSSMI; // menu item for dispose soccer scope

	private JMenuItem connectMI;

	private JMenuItem connectToHostMI;

	private JMenuItem disconnectMI;

	private JMenuItem kickoffMI;

	private JMenuItem dropballMI; // menu item for drop the ball to current
									// ball position

	private JMenuItem dropballCenterMI; // menu item for drop the ball to the
										// center position

	private JMenuItem freekickleftMI;

	private JMenuItem freekickrightMI;

	private JCheckBoxMenuItem choiceMI;

	private JCheckBoxMenuItem detailinfoMI;

	private JCheckBoxMenuItem statisticsMI;

	private JCheckBoxMenuItem optionMI;

	private JCheckBoxMenuItem antialiasMI;

	private JCheckBoxMenuItem statusBarMI;

	private JCheckBoxMenuItem toolBarMI;

	private JCheckBoxMenuItem scenePlayerMI;

	// private JCheckBoxMenuItem showClientPlayersMI;

	public SoccerScopeMenuBar(SoccerScope soccerscope) {
		this.soccerscope = soccerscope;
		Register.scopeMenu = this;

		JMenu fileMenu = new JMenu("File");
		fileMenu.setMnemonic('F');
		this.add(fileMenu);
		JMenu monitorMenu = new JMenu("Monitor");
		monitorMenu.setMnemonic('M');
		this.add(monitorMenu);
		JMenu windowMenu = new JMenu("Window");
		windowMenu.setMnemonic('W');
		this.add(windowMenu);
		JMenu viewMenu = new JMenu("View");
		viewMenu.setMnemonic('V');
		this.add(viewMenu);
		JMenu analyzeMenu = new JMenu("Analyze");
		analyzeMenu.setMnemonic('A');
		this.add(analyzeMenu);

		// XiRoT start
		// adding the Dai Menu Item to the main menu

		JMenu daiMenu = new JMenu("Dai");
		daiMenu.setMnemonic('D');

		// XiRoT end

		// File Menu
		this.save = new JMenuItem("Save");
		this.save.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,
				Event.CTRL_MASK));
		this.save.addActionListener(this);
		// remove if save&load work fine
		// save.setEnabled(false);
		this.load = new JMenuItem("Load");
		this.load.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L,
				Event.CTRL_MASK));
		this.load.addActionListener(this);
		// remove if save&load work fine
		// load.setEnabled(false);
		this.openMI = new JMenuItem("Open");
		this.openMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,
				Event.CTRL_MASK));
		this.openMI.addActionListener(this);
		this.loadplanMI = new JMenuItem("Load AgentLog");
		this.loadplanMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E,
				Event.CTRL_MASK));
		this.loadplanMI.addActionListener(this);
		this.loadlogMI = new JMenuItem("Load ServerLog");
		this.loadlogMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R,
				Event.CTRL_MASK));
		this.loadlogMI.addActionListener(this);
		this.closeMI = new JMenuItem("Close");
		this.closeMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W,
				Event.CTRL_MASK));
		this.closeMI.addActionListener(this);
		this.preferenceMI = new JMenuItem("Preference");
		this.preferenceMI.addActionListener(this);
		this.quitMI = new JMenuItem("Exit Platform");
		this.quitMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q,
				Event.CTRL_MASK));
		this.quitMI.addActionListener(this);
		this.exitSSMI = new JMenuItem("Exit SoccerScope");
		this.exitSSMI.addActionListener(this);
		fileMenu.add(this.save);
		fileMenu.add(this.load);
		fileMenu.add(this.openMI);
		// fileMenu.add(loadplanMI);
		// fileMenu.add(loadlogMI);
		// fileMenu.add(closeMI);
		fileMenu.addSeparator();
		fileMenu.add(this.preferenceMI);
		fileMenu.addSeparator();
		fileMenu.add(this.quitMI);
		fileMenu.add(this.exitSSMI);
		// ButtonManager.addButton("logfile", loadMI);
		// ButtonManager.addButton("logfile", loadplanMI);
		// ButtonManager.addButton("logfile", loadlogMI);
		// ButtonManager.addButton("logfile", closeMI);

		// Monitor Menu
		this.connectMI = new JMenuItem("Connect to Local");
		this.connectMI.addActionListener(this);
		this.connectToHostMI = new JMenuItem("Connect to Host");
		this.connectToHostMI.addActionListener(this);
		this.disconnectMI = new JMenuItem("Disconnect");
		this.disconnectMI.addActionListener(this);
		this.kickoffMI = new JMenuItem("KickOff");
		this.kickoffMI.addActionListener(this);
		this.dropballMI = new JMenuItem("DropBall");
		this.dropballMI.addActionListener(this);
		this.dropballCenterMI = new JMenuItem("DropBall to Center");
		this.dropballCenterMI.addActionListener(this);
		this.dropballCenterMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C,
				Event.CTRL_MASK));
		this.freekickleftMI = new JMenuItem("FreeKickLeft");
		this.freekickleftMI.addActionListener(this);
		this.freekickrightMI = new JMenuItem("FreeKickRight");
		this.freekickrightMI.addActionListener(this);
		monitorMenu.add(this.connectMI);
		monitorMenu.add(this.connectToHostMI);
		monitorMenu.add(this.disconnectMI);
		monitorMenu.addSeparator();
		monitorMenu.add(this.kickoffMI);
		monitorMenu.add(this.dropballMI);
		monitorMenu.add(this.dropballCenterMI);
		monitorMenu.add(this.freekickleftMI);
		monitorMenu.add(this.freekickrightMI);

		ButtonManager.addButton("network", this.disconnectMI);
		ButtonManager.addButton("network", this.kickoffMI);
		ButtonManager.addButton("network", this.dropballMI);
		ButtonManager.addButton("network", this.freekickleftMI);
		ButtonManager.addButton("network", this.freekickrightMI);

		// Window Menu
		this.choiceMI = new JCheckBoxMenuItem("Select Player");
		this.choiceMI.addItemListener(this);
		this.choiceMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L,
				Event.CTRL_MASK));
		this.detailinfoMI = new JCheckBoxMenuItem("Detail Info");
		this.detailinfoMI.addItemListener(this);
		this.detailinfoMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_K,
				Event.CTRL_MASK));
		this.statisticsMI = new JCheckBoxMenuItem("Statistics");
		this.statisticsMI.addItemListener(this);
		this.statisticsMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_M,
				Event.CTRL_MASK));
		this.optionMI = new JCheckBoxMenuItem("Option");
		this.optionMI.addItemListener(this);
		this.optionMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T,
				Event.CTRL_MASK));
		windowMenu.add(this.choiceMI);
		windowMenu.add(this.detailinfoMI);
		windowMenu.add(this.statisticsMI);
		windowMenu.add(this.optionMI);
		ButtonManager.addButton(SelectWindow.title, this.choiceMI);
		ButtonManager.addButton(InformationWindow.title, this.detailinfoMI);
		ButtonManager.addButton(StatisticsWindow.title, this.statisticsMI);
		ButtonManager.addButton(OptionWindow.title, this.optionMI);

		// View Menu
		ArrayList showLayer = soccerscope.getScopePane().getShowLayer();
		Iterator showit = showLayer.iterator();
		while (showit.hasNext()) {
			viewMenu.add(((Layer) showit.next()).createJCheckBoxMenuItem());
		}
		viewMenu.addSeparator();
		this.antialiasMI = new JCheckBoxMenuItem("Antialias");
		this.antialiasMI.setSelected(soccerscope.getScopePane().getFieldPane()
				.getDrawOption(FieldPane.DRAW_ANTIALIAS));
		this.antialiasMI.addItemListener(this);
		viewMenu.add(this.antialiasMI);
		this.toolBarMI = new JCheckBoxMenuItem("ToolBar");
		this.toolBarMI.setSelected(true);
		this.toolBarMI.addItemListener(this);
		viewMenu.add(this.toolBarMI);

		this.scenePlayerMI = new JCheckBoxMenuItem("ClientScene Player");
		this.scenePlayerMI.setSelected(true);
		this.scenePlayerMI.addItemListener(this);
		viewMenu.add(this.scenePlayerMI);

		this.statusBarMI = new JCheckBoxMenuItem("Status Bar");
		this.statusBarMI.setSelected(true);
		this.statusBarMI.addItemListener(this);
		viewMenu.add(this.statusBarMI);

		/*
		 * showClientPlayersMI = new JCheckBoxMenuItem("Show Player");
		 * showClientPlayersMI.setSelected(true);
		 * showClientPlayersMI.addItemListener(this);
		 * viewMenu.add(showClientPlayersMI);
		 */

		// Analyze Menu
		ArrayList drawLayer = soccerscope.getScopePane().getDrawLayer();
		Iterator drawit = drawLayer.iterator();
		while (drawit.hasNext()) {
			analyzeMenu.add(((Layer) drawit.next()).createJCheckBoxMenuItem());
		}

		// XiRoT start
		// DaiMenu
                this.dbNewGame = new JMenuItem("New game");
                this.dbNewGame.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N,
                                Event.SHIFT_MASK));
                this.dbNewGame.addActionListener(this);
//                this.dbNewGame.setEnabled(RobocupDatabase.connection != null);
                daiMenu.add(this.dbNewGame);

                this.dbSaveScene = new JCheckBoxMenuItem("Record scenes");
                this.dbSaveScene.setState(RobocupDatabase.recorded);
                this.dbSaveScene.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C,
                                Event.SHIFT_MASK));
                this.dbSaveScene.addActionListener(this);
//                this.dbSaveScene.setEnabled(RobocupDatabase.connection != null);
                daiMenu.add(this.dbSaveScene);

                this.dbLoadGame = new JMenuItem("Load game...");
                this.dbLoadGame.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L,
                                Event.SHIFT_MASK));
                this.dbLoadGame.addActionListener(this);
                daiMenu.add(this.dbLoadGame);

                this.dbStatistics = new JMenuItem("Statistics...");
                this.dbStatistics.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T,
                                Event.SHIFT_MASK));
                this.dbStatistics.addActionListener(this);
                daiMenu.add(this.dbStatistics);

                this.dbDBConfig = new JMenuItem("Database settings...");
                this.dbDBConfig.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,
                                Event.SHIFT_MASK));
                this.dbDBConfig.addActionListener(this);
                daiMenu.add(this.dbDBConfig);
                daiMenu.addSeparator();

		this.showActions = new JMenuItem("Actions");
		this.showActions.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A,
				Event.SHIFT_MASK));
		this.showActions.addActionListener(this);
		daiMenu.add(this.showActions);

		this.showPlayerData = new JMenuItem("Player Data");
		this.showPlayerData.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D,
				Event.SHIFT_MASK));
		this.showPlayerData.addActionListener(this);
		daiMenu.add(this.showPlayerData);

		this.showDaiStatistic = new JMenuItem("Statistics");
		this.showDaiStatistic.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,
				Event.SHIFT_MASK));
		this.showDaiStatistic.addActionListener(this);
		daiMenu.add(this.showDaiStatistic);

		this.storeDaiWMOnOff = new JCheckBoxMenuItem("Record Dai-WM");
		this.storeDaiWMOnOff.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W,
				Event.SHIFT_MASK));
		this.storeDaiWMOnOff.addActionListener(this);
		this.storeDaiWMOnOff.setState(true);
		daiMenu.add(this.storeDaiWMOnOff);

		this.storePlayerPosOnOff = new JCheckBoxMenuItem("Player Position", true);
		this.storePlayerSpeedOnOff = new JCheckBoxMenuItem("Player Speed", true);
		this.storeBallPosOnOff = new JCheckBoxMenuItem("Ball Position", true);
		this.storeBallSpeedOnOff = new JCheckBoxMenuItem("Ball Speed", true);
		this.storeBallIPOnOff = new JCheckBoxMenuItem("Ball IP", true);
		this.storeOppIPOnOff = new JCheckBoxMenuItem("Opp IP", true);
		this.storeAnglesOnOff = new JCheckBoxMenuItem("Angles", true);
		this.storeTeammatesAndOpponentsOnOff = new JCheckBoxMenuItem(
				"Teammates and Opponents", true);
		this.storePosStatisticsOnOff = new JCheckBoxMenuItem("Position Statistics",
				false);

		this.no.addActionListener(this);
		this.all.addActionListener(this);
		this.storeSelection.add(this.no);
		this.storeSelection.add(this.all);
		this.storeSelection.addSeparator();
		this.storeSelection.add(this.storePlayerPosOnOff);
		this.storeSelection.add(this.storePlayerSpeedOnOff);
		this.storeSelection.add(this.storeBallPosOnOff);
		this.storeSelection.add(this.storeBallSpeedOnOff);
		this.storeSelection.add(this.storeBallIPOnOff);
		this.storeSelection.add(this.storeOppIPOnOff);
		this.storeSelection.add(this.storeAnglesOnOff);
		this.storeSelection.add(this.storeTeammatesAndOpponentsOnOff);
		this.storeSelection.add(this.storePosStatisticsOnOff);
		// storeSelection.addActionListener(this);
		daiMenu.add(this.storeSelection);
		this.add(daiMenu);

		// XiRoT end

	}

	/**
	 * before closing the statistic dialog inform the soccerscope menu bar
	 */
	public void statisticDialogToNull() {
		this.statisticDialog = null;
	}

	// / ActionListener
	public void actionPerformed(ActionEvent ae) {
		Object o = ae.getSource();

		// XiRoT start

		if (ae.getActionCommand().equals("No")) {
			this.storePlayerPosOnOff.setState(false);
			this.storePlayerSpeedOnOff.setState(false);
			this.storeBallPosOnOff.setState(false);
			this.storeBallSpeedOnOff.setState(false);
			this.storeBallIPOnOff.setState(false);
			this.storeOppIPOnOff.setState(false);
			this.storeAnglesOnOff.setState(false);
			this.storeTeammatesAndOpponentsOnOff.setState(false);
			this.storePosStatisticsOnOff.setState(false);
		}
		if (ae.getActionCommand().equals("All")) {
			this.storePlayerPosOnOff.setState(true);
			this.storePlayerSpeedOnOff.setState(true);
			this.storeBallPosOnOff.setState(true);
			this.storeBallSpeedOnOff.setState(true);
			this.storeBallIPOnOff.setState(true);
			this.storeOppIPOnOff.setState(true);
			this.storeAnglesOnOff.setState(true);
			this.storeTeammatesAndOpponentsOnOff.setState(true);
			this.storePosStatisticsOnOff.setState(true);
		}

		if (o == this.showActions) {
			if (this.actionDialog == null)
				this.actionDialog = new ActionDialog(this.soccerscope);
			if (this.actionDialog.dialogVisible())
				ActionDialog.setVisible(false);
			else
				ActionDialog.setVisible(true);
		}

		if (o == this.showPlayerData) {
			if (this.playerDataDialog == null)
				this.playerDataDialog = new PlayerDataDialog(this, this.soccerscope, 192);
			if (this.playerDataDialog.dialogVisible())
				PlayerDataDialog.setVisible(false);
			else
				PlayerDataDialog.setVisible(true);
		}

		if (o == this.showDaiStatistic) {
			if (this.statisticDialog == null)
				this.statisticDialog = new StatisticDialog(this, this.soccerscope);
			else {
				this.statisticDialog.stop();
			}
		}

		if (o == this.storeDaiWMOnOff) {
			if (this.storeDaiWMOnOff.getState()) {
				Register.saveDaiWorldModel = true;
			}
			if (!this.storeDaiWMOnOff.getState()) {
				Register.saveDaiWorldModel = false;
			}
		}

                if (o == this.dbNewGame) {
                    RobocupDatabase rcDatabase = RobocupDatabase.getInstance();
                    if (rcDatabase == null)
                    {
                        rcDatabase = RobocupDatabase.getInstance(WorldModel.getSceneSet());
                        rcDatabase.saveGame();
                        rcDatabase.start();
                    }
                    else
                    {
                        rcDatabase.recordDisable();
                        this.dbSaveScene.setState(false);
                        try {
                            Thread.sleep(1000); // wartet bis die letzte Szene gespeichert wird
                        } catch (InterruptedException ex1) {
                        }
                        rcDatabase.finalizeGame();

                        rcDatabase.saveGame();
                    }
                }

                if (o == this.dbSaveScene) {
                    if (DatabaseConfig.getInstance().isDetectTeams() && (WorldModel.getSceneSet().lastScene().right.name.equals("") || WorldModel.getSceneSet().lastScene().left.name.equals("")))
                    {
                        JOptionPane.showMessageDialog(null, "Both teams must be loaded. Saving abort!", "Recording Error", JOptionPane.ERROR_MESSAGE);
                        this.dbSaveScene.setState(false);
                        return;
                    }

                    if (DatabaseConfig.getInstance().isDetectPlaying() && WorldModel.getSceneSet().lastScene().time == 65535)
                    {
                        JOptionPane.showMessageDialog(null, "Game not started. Saving abort!", "Recording Error", JOptionPane.ERROR_MESSAGE);
                        this.dbSaveScene.setState(false);
                        return;
                    }

                    if (RobocupDatabase.getInstance() == null)
                    {
                        RobocupDatabase rcDatabase = RobocupDatabase.getInstance(WorldModel.getSceneSet());
                        rcDatabase.saveGame();
                        rcDatabase.start();
                    }

                    if (RobocupDatabase.recorded)
                    {
                        RobocupDatabase.recordDisable();
                        RobocupDatabase.getInstance().finalizeGame();
                    }
                    else
                    {
                        RobocupDatabase.recordEnable();
//                        RobocupDatabase.recorded = !RobocupDatabase.recorded;
//                        RobocupDatabase.getInstance().start();
                    }
                }

                if (o == this.dbLoadGame) {
                    DatabaseReader dbReader = DatabaseReader.getInstance(WorldModel.getSceneSet());
                    LoadGameDialog lgDialog = new LoadGameDialog();
                    lgDialog.setVisible(true);

                    dbReader.gameID = lgDialog.getSelectedGame().getId();
                    if (dbReader.gameID != -1)
                    {
                        dbReader.progress = soccerscope.getStatusBar().getProgressBar();
                        dbReader.start();
                        this.soccerscope.getScenePlayer().stop();
                        WorldModel.getInstance().clear();
                        this.soccerscope.getScenePlayer().head();
                        this.soccerscope.setTitle("SoccerScope: " + lgDialog.getSelectedGame().getDescription());

                    }
                }

                if (o == this.dbDBConfig)
                {
                    DatabaseConfigDialog dcDialog = new DatabaseConfigDialog();
                    dcDialog.setVisible(true);
                }

                if (o == this.dbStatistics)
                {
                    StatisticFrame sFrame = new StatisticFrame();
                    sFrame.setVisible(true);
                }

		// XiRoT end

		if (o == this.save) {
			JFileChooser chooser = new JFileChooser();
			chooser.setFileFilter(new SoccerScopeFilter());
			chooser.setCurrentDirectory(new File(this.soccerscope.getChooseFile()
					.getPath()));
			int returnVal = chooser.showSaveDialog(this.soccerscope);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				File fileName = chooser.getSelectedFile();
				if (fileName == null || fileName.getName().equals("")) {
					JOptionPane.showMessageDialog(this, "Invalid File Name",
							"Invalid File Name", JOptionPane.ERROR_MESSAGE);
				} else {
					if (!fileName.getName().contains("."))
						fileName = new File(fileName.getAbsolutePath() + ".scs");
					try {
						ObjectOutputStream output = new ObjectOutputStream(
								new DeflaterOutputStream(new FileOutputStream(fileName)));
                        WorldModel.writeWorldModel(output);
                        output.close();
					} catch (Exception ex) {
						JOptionPane.showMessageDialog(this, ex.toString(),
								"Error", JOptionPane.ERROR_MESSAGE);
					}
				}
			}
		}
		if (o == this.load) {
			JFileChooser chooser = new JFileChooser(System
					.getProperty("user.dir"));
			chooser.setFileFilter(new SoccerScopeFilter());
			chooser.setCurrentDirectory(new File(this.soccerscope.getChooseFile()
					.getPath()));
			int returnVal = chooser.showOpenDialog(this.soccerscope);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				File fileName = chooser.getSelectedFile();
				if (fileName == null || fileName.getName().equals("")) {
					JOptionPane.showMessageDialog(this, "Invalid File Name",
							"Invalid File Name", JOptionPane.ERROR_MESSAGE);
				} else {
					try {
						ObjectInputStream input = new ObjectInputStream(
								new InflaterInputStream(new FileInputStream(fileName)));
                        WorldModel.loadWorldModel(input);
                        input.close();
                        soccerscope.getScenePlayer().stop();
                        ScenePlayModeList.allChanged();
					} catch (Exception ex) {
						JOptionPane.showMessageDialog(this, ex.toString(),
								"Error", JOptionPane.ERROR_MESSAGE);
					}
				}
			}
		}

		// not needed ?

		// Menu Item Open ...
		if (o == this.openMI) {
			JFileChooser chooser = new JFileChooser(this.soccerscope.getChooseFile());
			chooser.setFileFilter(new GameLogFileFilter());
			chooser.setSelectedFile(this.soccerscope.getChooseFile());
			int returnVal = chooser.showOpenDialog(this.soccerscope);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				this.soccerscope.getScenePlayer().stop();
				File chooseFile = chooser.getSelectedFile();
				if (chooseFile == null)
					return;
				this.soccerscope.setChooseFile(chooseFile);
				WorldModel wm = WorldModel.getInstance();
				this.soccerscope.getScenePlayer().head();
				wm.clear();
				this.soccerscope.setTitle("SoccerScope: " + chooseFile.getName());
				this.soccerscope.openLogFile(wm.getSceneSet(), chooseFile.getPath());
				ButtonManager.setEnabled("logfile", true);
			}
			// Menu Item Load Server Log ...
		} else if (o == this.loadlogMI) {
			JFileChooser chooser = new JFileChooser(this.soccerscope.getChooseFile());
			chooser.setFileFilter(new ServerLogFileFilter(this.soccerscope
					.getServerFile()));
			chooser.setSelectedFile(this.soccerscope.getChooseServerFile());
			int returnVal = chooser.showOpenDialog(this.soccerscope);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				this.soccerscope.getScenePlayer().stop();
				File chooseFile = chooser.getSelectedFile();
				if (chooseFile == null)
					return;
				this.soccerscope.setChooseServerFile(chooseFile);
				WorldModel wm = WorldModel.getInstance();
				this.soccerscope.getScenePlayer().head();
				this.soccerscope.openServerLogFile(wm.getSceneSet(), chooseFile
						.getPath());
			}

			// Menu Item Load Agent Log ...
		} else if (o == this.loadplanMI) {
			JFileChooser chooser = new JFileChooser(this.soccerscope.getChooseFile());
			chooser.setFileFilter(new PlanLogFileFilter());
			int returnVal = chooser.showOpenDialog(this.soccerscope);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				this.soccerscope.getScenePlayer().stop();
				File chooseFile = chooser.getSelectedFile();
				if (chooseFile == null)
					return;
				this.soccerscope.setChooseFile(chooseFile);
				WorldModel wm = WorldModel.getInstance();
				wm.clearAgentPlan();
				wm.makeAgentPlan(chooseFile.getPath());
				ButtonManager.setEnabled("AgentLog", true);
			}

			// եĤ
		} else if (o == this.closeMI) {
			this.soccerscope.getScenePlayer().stop();
			this.soccerscope.getScenePlayer().head();
			WorldModel wm = WorldModel.getInstance();
			wm.clear();
			this.soccerscope.setTitle("SoccerScope");
			ButtonManager.setEnabled("logfile", false);
			ButtonManager.setEnabled("AgentWorldModel", false);
			ButtonManager.setEnabled("AgentLog", false);

			// soccerscope
		} else if (o == this.quitMI) {
			System.exit(0);

		} else if (o == this.exitSSMI) {
			this.soccerscope.setVisible(false);
			this.soccerscope.dispose();

		} else if (o == this.preferenceMI) {
			PreferenceDialog pd = new PreferenceDialog(this.soccerscope);
			pd.show();
			this.soccerscope.repaint();
		} else if (o == this.connectMI) {
			try {
				WorldModel wm = WorldModel.getInstance();
				this.soccerscope
						.setSoccerServerConnection(new SoccerServerConnection());
				SceneSetMaker ssm = new SceneSetMaker(this.soccerscope
						.getSoccerServerConnection(), wm.getSceneSet());
				try {
					this.soccerscope.getSoccerServerConnection().dispinit();
					Thread.sleep(1000);
					this.soccerscope.getScenePlayer().sync();
					ssm.start();
					ButtonManager.setEnabled("network", true);
				} catch (InterruptedException iie) {
				} catch (IOException ie) {
					System.err.println(ie);
				}
			} catch (SocketException se) {
				System.err.println(se);
			} catch (UnknownHostException uhe) {
				System.err.println(uhe);
			}

			// connection SoccerScope to a server
		} else if (o == this.connectToHostMI) {
			String host = JOptionPane
					.showInputDialog("Input Host", "localhost");
			if (host != null) {
				try {
					WorldModel wm = WorldModel.getInstance();
					this.soccerscope
							.setSoccerServerConnection(new SoccerServerConnection(
									host));
					SceneSetMaker ssm = new SceneSetMaker(this.soccerscope
							.getSoccerServerConnection(), wm.getSceneSet());
					try {
						this.soccerscope.getSoccerServerConnection().dispinit();
						Thread.sleep(1000);
						this.soccerscope.getScenePlayer().sync();
						ssm.start();
						ButtonManager.setEnabled("network", true);
					} catch (InterruptedException iie) {
					} catch (IOException ie) {
						System.err.println(ie);
					}
				} catch (SocketException se) {
					System.err.println(se);
				} catch (UnknownHostException uhe) {
					System.err.println(uhe);
				}
			}

			// disconnect from server
		} else if (o == this.disconnectMI) {
			try {
				this.soccerscope.getSoccerServerConnection().dispbye();
				this.soccerscope.setSoccerServerConnection(null);
//                                soccerscope.rcDatabase.recordDisable();

				ButtonManager.setEnabled("network", false);
			} catch (IOException ie) {
				System.err.println(ie);
			}

			// kick off
		} else if (o == this.kickoffMI) {
			try {
				this.soccerscope.getSoccerServerConnection().dispstart();
//                                soccerscope.rcDatabase.recordEnable();
			} catch (IOException ie) {
				System.err.println(ie);
			}

			// drop ball
		} else if (o == this.dropballMI) {
			try {
				ClientScene clientScene = this.soccerscope.getScenePlayer()
						.getCurrentScene();
				this.soccerscope.getSoccerServerConnection().dispfoul(
						clientScene.ball.pos.x, clientScene.ball.pos.y,
						Team.NEUTRAL);
			} catch (IOException ie) {
				System.err.println(ie);
			}

		} else if (o == this.dropballCenterMI) {
			try {
//				ClientScene clientScene = this.soccerscope.getScenePlayer()
//						.getCurrentScene();
				this.soccerscope.getSoccerServerConnection().dispfoul(0, 0,
						Team.NEUTRAL);
			} catch (IOException ie) {
				System.err.println(ie);
			}

			// free kick left
		} else if (o == this.freekickleftMI) {
			try {
				ClientScene clientScene = this.soccerscope.getScenePlayer()
						.getCurrentScene();
				this.soccerscope.getSoccerServerConnection().dispfoul(
						clientScene.ball.pos.x, clientScene.ball.pos.y,
						Team.LEFT_SIDE);
			} catch (IOException ie) {
				System.err.println(ie);
			}

			// free kick right
		} else if (o == this.freekickrightMI) {
			try {
				ClientScene clientScene = this.soccerscope.getScenePlayer()
						.getCurrentScene();
				this.soccerscope.getSoccerServerConnection().dispfoul(
						clientScene.ball.pos.x, clientScene.ball.pos.y,
						Team.RIGHT_SIDE);
			} catch (IOException ie) {
				System.err.println(ie);
			}
		}
	}

	// / ItemListener
	public void itemStateChanged(ItemEvent ie) {
		Object o = ie.getSource();

		if (o == this.choiceMI) {
			this.soccerscope.getSelectWindow().setVisible(this.choiceMI.isSelected());
			soccerscope.getSelectWindow().setState(Frame.NORMAL);

		} else if (o == detailinfoMI) {
			soccerscope.getInformationWindow().setVisible(
					detailinfoMI.isSelected());
			soccerscope.getInformationWindow().setState(Frame.NORMAL);

		} else if (o == statisticsMI) {
			soccerscope.getStatisticsWindow().setVisible(
					statisticsMI.isSelected());
			soccerscope.getStatisticsWindow().setState(Frame.NORMAL);

		} else if (o == optionMI) {
			soccerscope.getOptionWindow().setVisible(optionMI.isSelected());
			soccerscope.getOptionWindow().setState(Frame.NORMAL);

			// ON/OFF
		} else if (o == antialiasMI) {
			soccerscope.getScopePane().getFieldPane().setDrawOption(
					FieldPane.DRAW_ANTIALIAS, antialiasMI.isSelected());
		} else if (o == toolBarMI) {
			soccerscope.initToolBar(toolBarMI.isSelected());
			soccerscope.repaint();
		} else if (o == scenePlayerMI) {
			soccerscope.initScenePlayer(scenePlayerMI.isSelected());
			soccerscope.repaint();
		} else if (o == statusBarMI) {
			soccerscope.initStatusBar(statusBarMI.isSelected());
			soccerscope.repaint();
		} /*
			 * else if (o == showClientPlayersMI) { FieldPane fieldPane =
			 * soccerscope.getScopePane().getFieldPane(); ArrayList foreGround =
			 * fieldPane.getForegroundList(); Iterator e =
			 * foreGround.iterator(); boolean selected = ((AbstractButton)
			 * o).isSelected(); while (e.hasNext()) { Layer layer = (Layer)
			 * e.next(); if (layer.getLayerName().equals("Player")) {
			 * layer.setEnabled(false); } } ArrayList backGround =
			 * fieldPane.getBackgroundList(); e = backGround.iterator(); while
			 * (e.hasNext()) { Layer layer = (Layer) e.next(); if
			 * (layer.getLayerName().equals("Player")) {
			 * layer.setEnabled(false); } } soccerscope.repaint(); }
			 */
	}
}
/**
 * $Log: SoccerScopeMenuBar.java,v $
 * Revision 1.3.24.5  2006/11/23 09:41:14  endert
 * Added License Note at top of each *.java file.
 *
 * Revision 1.3.24.4  2006/11/13 16:50:22  endert
 * Removed CVS-Logs and author tags
 *
 * Revision 1.3.24.3  2006/10/25 14:33:35  endert
 * Latest SoccerScope Version plus Database ability
 *
 * Revision 1.3.10.14  2006/10/11 17:08:55  lphle
 * 2 neue Einstellungen fr die Aufnahme
 *
 * Revision 1.3.10.13  2006/10/09 00:46:56  lphle
 * MenuItem fr DB-Statistik
 *
 * Revision 1.3.10.12  2006/10/05 21:48:03  lphle
 * Mene-Eintrag fr Datenbankeinstellung-Dialogbox eingefgt
 *
 * Revision 1.3.10.11  2006/08/14 13:43:48  endert
 * Replaced Jump-Editors to ScenePlayer and Removed Jump..Toolbar
 *
 * Revision 1.3.10.10  2006/07/29 13:32:42  benjaboe
 * Some parts of the AgentModelLayer can be activated/deactivated.
 * The drawing does not use the LocalScene any more - LocalScene is deleted.
 *
 * Revision 1.3.10.9  2006/07/24 16:24:47  benjaboe
 * Jump to scenes is possible
 *
 * Revision 1.3.10.8  2006/07/18 15:49:41  lphle
 * menu fr DB umstruktuiert
 *
 * Revision 1.3.10.7  2006/07/18 07:01:06  lphle
 * Menu zum Laden eines Spiels
 *
 * Revision 1.3.10.6  2006/07/17 10:24:50  lphle
 * Database wird nicht mehr hier gestartet
 *
 * Revision 1.3.10.5  2006/07/14 12:08:57  lphle
 * startet Datenbank-Thread
 *
 * Revision 1.3.10.4  2006/07/14 09:33:35  lphle
 * Menuitem funktioniert nur wenn DB-Verbindung hergestellt wurde
 *
 * Revision 1.3.10.4  2006/07/13 06:36:55  lphle
 * disable db menu items when no db connection
 *
 * Revision 1.3.10.3  2006/07/12 16:02:03  lphle
 * Datenbank eingebaut
 *
 * Revision 1.3.10.2  2006/07/08 07:40:01  benjaboe
 * Speichern und Laden scheint zu funktionieren
 *
 * Revision 1.3.10.1  2006/06/26 07:04:22  benjaboe
 * Datamodell umgebaut
 *
 * Revision 1.3  2006/04/05 22:48:01  wetzker
 * added prediction of other players state
 * each player now contains a stateEvaluation (see StateTactic)
 * used the prediction for calculation the players of the next cycle (can be seen in the gui)
 * + new State (OpponentBallInterception) for prediction of opponents
 * removed bug in delayed intercetion (cause stamina loss for nothing)
 * meta_playmodes can be used now (added always playmode)
 * improved attack in line
 *
 * Revision 1.2  2006/03/27 23:25:53  wetzker
 * simplified stateEvaluation
 * - moved most of SimpleState into Abstract
 * - moved most of StateEvaluation into StateTactic
 * adapted GUIUpdate
 * - deleted state tables
 * - only 1 state update each cycle
 * - gui should now be much faster (less holes)
 *
 * Revision 1.1  2006/03/13 07:38:27  axle
 * moved from ROOT/src for Maven 2 Buildsystem and Team/Soccerscope separation
 *
 * Revision 1.14  2006/03/07 12:46:39  endert
 * Merge after refactoring:
 *  - New State-Interface: One Method for each action type
 *  - Action Bean removed - ActionFactorys for helper methods, States calculate
 *    main-Action themselves
 *  - formatted code
 *  - added State-List in Monitor (Context-Menu of Player)
 *  - other smaller stuff
 *
 * Revision 1.13.2.2  2006/03/06 23:17:54  endert
 * Removed warnings, added State-Overview to Context-Menu!
 *
 * Revision 1.13.2.1  2006/02/25 22:07:29  wetzker
 * formated
 *
 * Revision 1.13  2006/02/22 16:05:12  endert
 * This is done:
 * - Rebound SoccerScope to Agents (can be added or removed in config-file)
 * - Situations now all extend abstract class Situation
 * - Refactorings: ActionBean _ removed from Attributes
 * - SoccerScope will automatically connect to host given in Start-Call
 * - Trainer uses the same host as soccerscope
 * - Removed unused stuff (old confs, bat, etc.)
 * - Code formatted (good luck when merging)
 * - organized imports
 * - all that i forgot to mention here
 * Revision 1.12 2005/09/01 17:30:33 thyrion added quick'n'dirty display
 * of opponentinterceptionpoint
 *
 * Revision 1.11 2005/06/20 17:42:46 andpaul remuved: unused worldmodeldialog
 * variable
 *
 * Revision 1.10 2005/06/15 17:03:38 wetzker removed JIAC from Head + changes in
 * framework please report bugs or shortcomings some scenarios might not work
 * because of wrong imports
 *
 * Revision 1.6.2.1 2005/06/15 16:44:56 wetzker final version
 *
 * Revision 1.9 2005/06/12 18:52:18 thyrion fixed saving&loading of our
 * worldmodel (to load first open server log)
 *
 * Revision 1.8 2005/06/06 23:36:07 andpaul added: a new entry at Dai -> Refine
 *
 * Revision 1.7 2005/06/06 22:59:31 thyrion changed a few things to hopefully
 * show worldmodel while playing
 *
 * Revision 1.6 2005/06/05 09:47:09 andpaul added: values to store are
 * selectable via Dai MenuItem
 *
 * Revision 1.5 2005/05/11 09:57:29 andpaul changed: window handling with soccer
 * scope added: layout manager to PlayerDataDialog and ActionDialog fix: some
 * "null pointers" when an unknown player was selected
 *
 * Revision 1.4 2005/05/10 08:46:09 thyrion added SoccerScopeFilter... our
 * logfile got the extension ".scs", deactivated save & load until they work
 * correctly
 *
 * Revision 1.3 2005/05/10 00:35:48 thyrion Added Save & Load
 *
 * Revision 1.2 2005/04/25 15:20:46 andpaul added: selector for dai wm recording
 * in DAI menu
 *
 * Revision 1.1 2005/04/20 10:39:42 endert Fresh Team Repository installation
 * from /home/e/endert/Robocup to here.
 *
 * Revision 1.10 2005/04/13 17:05:57 andpaul no need for a soccerscopemenubar
 * parameter in actiondialog constructor
 *
 * Revision 1.9 2005/04/02 11:08:19 andpaul documentation
 *
 * Revision 1.8 2005/03/31 19:41:12 andpaul changed statistic dialog handling
 *
 * Revision 1.7 2005/03/13 15:02:09 haeder * some modifications to reduce the
 * number of warnings ;-) - removed unreadable comments
 *
 * Revision 1.6 2005/03/12 14:19:39 andpaul added exit SoccerScope and exit
 * platform added dropBall (drops the ball to its current position) and dropBall
 * to Center (shortcut STRG-C, drops the ball to the center position)
 *
 *
 *
 */
