/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope;

import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.io.IOException;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.util.Properties;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;

import robocup.component.AbstractAgent;
import robocup.component.RobocupAgent;
import robocup.component.TrainerAgent;
import soccerscope.file.GameLogFileFilter;
import soccerscope.model.ClientScene;
import soccerscope.model.SceneSetMaker;
import soccerscope.model.Team;
import soccerscope.model.WorldModel;
import soccerscope.net.SoccerServerConnection;
import soccerscope.training.TrainingModel;
import soccerscope.util.ButtonManager;
import soccerscope.view.FieldPane;
import soccerscope.view.ScopePane;

public class SoccerScopeToolBar extends JToolBar implements ActionListener,
		ItemListener {

	private static final long serialVersionUID = 7388206577923466583L;

	private SoccerScope soccerscope;

	// private ScopePane scopePane;

	// ToolBar Item
	private JButton openButton;

	// private JButton printButton;
	private JButton connectButton;

	private JButton disconnectButton;

	private JButton fitButton;

	private JButton sxgaButton;

	private JButton zoominButton;

	private JButton zoomoutButton;

	private JButton zoomBallButton;

	private JButton trainingButton;

	private JButton trainEditButton;
	
	private JButton startGameButton;
	
	private JButton dropBallButton;

	private JToggleButton dogBallButton;

	private ButtonGroup mouseButtonGroup;

	private JToggleButton selectButton;

	private JToggleButton magnifyButton;

	private JToggleButton handButton;

	// private JToggleButton moveButton;

	public SoccerScopeToolBar(SoccerScope soccerscope) {
		this.soccerscope = soccerscope;
		// this.scopePane = soccerscope.getScopePane();
		// FieldPane fieldPane = this.scopePane.getFieldPane();
		
		this.setFloatable(false);

		this.openButton = new JButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/general/Open24.gif")));
		// printButton = new JButton(new
		// ImageIcon(ClassLoader.getSystemResource("soccerscope/image/general/Print24.gif")));
		this.connectButton = new JButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/connect_creating.gif")));
		this.disconnectButton = new JButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/connect_no.gif")));

		// ΨΥܥ
		// x6Button = new JButton("x6");
		// x10Button = new JButton("x10");
		// x24Button = new JButton("x24");
		// x46Button = new JButton("x46");
		this.fitButton = new JButton("fit");
		this.sxgaButton = new JButton("SXGA");
		this.zoominButton = new JButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/general/ZoomIn24.gif")));
		this.zoomoutButton = new JButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/general/ZoomOut24.gif")));
		this.zoomBallButton = new JButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/ZoomBall24.gif")));
		this.dogBallButton = new JToggleButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/WatchBall24.gif")));
		this.trainEditButton = new JButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/huetchen.gif")));
		this.trainingButton = new JButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/ball.gif")));

		this.startGameButton = new JButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/startgame.gif")));
		this.dropBallButton  = new JButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/dropball.gif")));
		
		// ޥ⡼ɤΥܥ
		this.selectButton = new JToggleButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/cursor.gif")), true);
		this.selectButton.setMargin(new Insets(4, 4, 4, 4));
		this.magnifyButton = new JToggleButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/general/Zoom24.gif")));
		this.magnifyButton.setMargin(new Insets(0, 0, 0, 0));
		this.handButton = new JToggleButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/palette_hand.gif")));
		this.handButton.setMargin(new Insets(4, 4, 4, 4));
		// moveButton = new JToggleButton(new
		// ImageIcon(ClassLoader.getSystemResource("soccerscope/offline/image/Move.gif")));
		// moveButton.setMargin(new Insets(0,0,0,0));
		this.mouseButtonGroup = new ButtonGroup();
		this.mouseButtonGroup.add(this.selectButton);
		this.mouseButtonGroup.add(this.magnifyButton);
		this.mouseButtonGroup.add(this.handButton);
		// mouseButtonGroup.add(moveButton);

		this.add(this.openButton);
		// this.add(printButton);
		// this.addSeparator();
		this.add(this.connectButton);
		this.add(this.disconnectButton);
		this.addSeparator();
		// this.add(x6Button);
		// this.add(x10Button);
		// this.add(x24Button);
		// this.add(x46Button);
		this.add(this.fitButton);
		this.add(this.sxgaButton);
		this.add(this.zoominButton);
		this.add(this.zoomoutButton);
		this.add(this.zoomBallButton);
		this.add(this.dogBallButton);
		this.addSeparator();
		this.add(this.selectButton);
		this.add(this.magnifyButton);
		this.add(this.handButton);

		// Training
		this.addSeparator();
		this.add(this.trainEditButton);
		this.trainEditButton.setToolTipText("This starts the Scenario-Editor");
		this.add(this.trainingButton);
		this.trainingButton.setToolTipText("This tries to start the "
				+ "trainer - be careful!");

		this.addSeparator();
		this.add(this.startGameButton);
		this.startGameButton.setToolTipText("Sets PlayMode to Kick-Off!");
		this.add(this.dropBallButton);
		this.dropBallButton.setToolTipText("Drops the Ball to Center!");
		this.startGameButton.addActionListener(this);
		this.dropBallButton.addActionListener(this);
		
		// this.add(moveButton);
		this.openButton.addActionListener(this);
		// printButton.addActionListener(this);
		this.connectButton.addActionListener(this);
		this.disconnectButton.addActionListener(this);
		// x6Button.addActionListener(this);
		// x10Button.addActionListener(this);
		// x24Button.addActionListener(this);
		// x46Button.addActionListener(this);
		this.fitButton.addActionListener(this);
		this.sxgaButton.addActionListener(this);
		this.zoominButton.addActionListener(this);
		this.zoomoutButton.addActionListener(this);
		this.zoomBallButton.addActionListener(this);
		this.trainEditButton.addActionListener(this);
		this.trainingButton.addActionListener(this);
		this.dogBallButton.addItemListener(this);
		this.selectButton.addItemListener(this);
		this.magnifyButton.addItemListener(this);
		this.handButton.addItemListener(this);
		// moveButton.addItemListener(this);

		ButtonManager.addButton("network", this.disconnectButton);
	}

	// / ActionListener
	public void actionPerformed(ActionEvent ae) {
		Object o = ae.getSource();
		ScopePane scopePane = this.soccerscope.getScopePane();
		FieldPane fieldPane = scopePane.getFieldPane();

		if (o == this.openButton) {
			// ե򳫤
			JFileChooser chooser = new JFileChooser(this.soccerscope.getChooseFile());
			chooser.setFileFilter(new GameLogFileFilter());
			chooser.setSelectedFile(this.soccerscope.getChooseFile());
			int returnVal = chooser.showOpenDialog(this.soccerscope);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				this.soccerscope.getScenePlayer().stop();
				File chooseFile = chooser.getSelectedFile();
				if (chooseFile == null)
					return;
				this.soccerscope.setChooseFile(chooseFile);
				WorldModel wm = WorldModel.getInstance();
				this.soccerscope.getScenePlayer().head();
				wm.clear();
				this.soccerscope.setTitle("SoccerScope: " + chooseFile.getName());
				this.soccerscope.openLogFile(WorldModel.getSceneSet(), chooseFile.getPath());
				ButtonManager.setEnabled("logfile", true);
			}

		} else if (o == this.connectButton) {
			// ꤵ줿ۥȤ³
			String host = JOptionPane.showInputDialog("Input Host");
			if (host != null) {
				try {
					this.soccerscope
							.setSoccerServerConnection(new SoccerServerConnection(
									host));
					SceneSetMaker ssm = new SceneSetMaker(this.soccerscope
							.getSoccerServerConnection(), WorldModel.getSceneSet());
					try {
						this.soccerscope.getSoccerServerConnection().dispinit();
						Thread.sleep(1000);
						this.soccerscope.getScenePlayer().sync();
						ssm.start();
						ButtonManager.setEnabled("network", true);
					} catch (InterruptedException iie) {
					} catch (IOException ie) {
						System.err.println(ie);
					}
				} catch (SocketException se) {
					System.err.println(se);
				} catch (UnknownHostException uhe) {
					System.err.println(uhe);
				}
			}

		} else if (o == this.disconnectButton) {
			// ФȤ³Ǥ
			try {
				this.soccerscope.getSoccerServerConnection().dispbye();
				this.soccerscope.setSoccerServerConnection(null);
				ButtonManager.setEnabled("network", false);
			} catch (IOException ie) {
				System.err.println(ie);
			}

			// } else if (o == printButton) {
			// ߤΰ֤ɸϤ˽Ϥ
			// scopePane.printOut();
			// TableModel tb = GameAnalyzer.getTableModel();
			// System.out.println();
			// for (int i = 0; i < tb.getRowCount(); i++) {
			// for (int j = 0; j < tb.getColumnCount(); j++) {
			// System.out.print(tb.getValueAt(i, j) + ", ");
			// }
			// System.out.println();
			// }

			// tb = GameAnalyzer.getLeftTableModel();
			// System.out.println();
			// for (int i = 0; i < tb.getRowCount(); i++) {
			// for (int j = 0; j < tb.getColumnCount(); j++) {
			// System.out.print(tb.getValueAt(i, j) + ", ");
			// }
			// System.out.println();
			// }

			// tb = GameAnalyzer.getRightTableModel();
			// System.out.println();
			// for (int i = 0; i < tb.getRowCount(); i++) {
			// for (int j = 0; j < tb.getColumnCount(); j++) {
			// System.out.print(tb.getValueAt(i, j) + ", ");
			// }
			// System.out.println();
			// }

			// } else if (o == x6Button) {
			// scopePane.setMagnify(6.0f, 7.2f, 3.6f);
			// } else if (o == x10Button) {
			// scopePane.setMagnify(10.0f, 5.0f, 2.0f);
			// } else if (o == x24Button) {
			// scopePane.setMagnify(24.0f, 1.8f, 1.3f);
			// } else if (o == x46Button) {
			// scopePane.setMagnify(46.0f, 1.0f, 1.0f);
		} else if (o == this.fitButton) {
			scopePane.fitMagnify();
		} else if (o == this.zoomBallButton) {
			ClientScene s = this.soccerscope.getScenePlayer().getCurrentScene();
			if (fieldPane.getWatchPoint().equals(s.ball.pos)
					&& fieldPane.getMagnify() == 46.0f) {
				scopePane.fitMagnify();
			} else {
				scopePane.getFieldPane().setWatchPoint(s.ball.pos);
				scopePane.setMagnify(46.0f, 1.0f, 1.0f);
			}
		} else if (o == this.sxgaButton) {
			scopePane.setMagnify(10.5f, 6.0f, 3.0f);
		} else if (o == this.zoominButton) {
			scopePane.setMagnify(scopePane.getMagnify() + 5);
		} else if (o == this.zoomoutButton) {
			scopePane.setMagnify(scopePane.getMagnify() - 5);
		} else if (o == this.trainingButton) {
			if (WorldModel.getInstance().trainer == null) {
				JFileChooser chooser = new JFileChooser();
				int returnVal = chooser.showOpenDialog(this.soccerscope);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					String filename = chooser.getSelectedFile()
							.getAbsolutePath();
					TrainingModel tm = TrainingModel
							.loadTrainingModel(filename);
					WorldModel.getInstance().trainModel = tm;
					Properties props = RobocupAgent
							.loadProperties("trainer.conf");
					props.put(AbstractAgent.HOST_NAME,
							SoccerServerConnection.DEFAULT_HOST);
					WorldModel.getInstance().trainer = new TrainerAgent(props);
					WorldModel.getInstance().trainer.initialize();
					WorldModel.getInstance().trainer.start();
					WorldModel.getInstance().setTrainingMode(true);
					if (scopePane != null) {
						scopePane.openTrainStatsDialog();
					}
				}
			} else {
				WorldModel.getInstance().trainer
						.sendActions("(change_mode play_on)");
			}
		} else if (o == this.trainEditButton) {
			if (scopePane != null) {
				scopePane.openTrainScenarioDialog();
			}
		} else if (o == this.startGameButton) {
			try {
				this.soccerscope.getSoccerServerConnection().dispstart();
			} catch (IOException ie) {
				System.err.println(ie);
			}
		} else if (o == this.dropBallButton) {
			try {
				ClientScene clientScene = this.soccerscope.getScenePlayer()
						.getCurrentScene();
				this.soccerscope.getSoccerServerConnection().dispfoul(
						clientScene.ball.pos.x, clientScene.ball.pos.y,
						Team.NEUTRAL);
			} catch (IOException ie) {
				System.err.println(ie);
			}
		}
	}

	// / ItemListener
	public void itemStateChanged(ItemEvent ie) {
		Object o = ie.getSource();
		ScopePane scopePane = this.soccerscope.getScopePane();

		if (o == this.selectButton) {
			scopePane.setMouseMode(ScopePane.SELECT_MODE);

		} else if (o == this.magnifyButton) {
			scopePane.setMouseMode(ScopePane.MAGNIFY_MODE);

		} else if (o == this.handButton) {
			scopePane.setMouseMode(ScopePane.MOVE_MODE);
			// scopePane.setMouseMode(ScopePane.HAND_MODE);

		} else if (o == this.dogBallButton) {
			scopePane.setBallDogMode(this.dogBallButton.isSelected());
			// } else if (o == moveButton) {
			// scopePane.setMouseMode(ScopePane.MOVE_MODE);
		}
	}

}
