/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope.db;

import java.sql.*;
import soccerscope.model.ClientScene;
import soccerscope.model.SimplePlayer;
import soccerscope.model.PlayerWorldModel;
import soccerscope.model.Player;
import soccerscope.model.Team;
import soccerscope.model.SceneSet;
import soccerscope.model.WorldModel;

/**
 * <p>Title: Robocup 2006</p>
 *
 * <p>Description: Datenbank Modul<br/>Skript zum Erstellen der Datenbank Robocup.sql im Verzeichnis Robocup/etc</p>
 *
 * <p>Copyright: Copyright (c) 2006</p>
 *
 * <p>Company: </p>
 * @version 1.0
 */
public class RobocupDatabase extends Thread {
    public RobocupDatabase() {
    }

    // Zugang zu Datenbank-Server
    public static Connection connection = null;
    private String sqlHost;
    private String sqlDB;
    private String sqlUser;
    private String sqlPassword;

    // Anfragen
    private String squery = "insert into client_scene (game_id, play_mode, ball_pos_x, ball_pos_y, ball_vel_x, ball_vel_y, time, ball_mode, ball_bestPosX, ball_bestPosY, ball_accX, ball_accY, id) values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
    private PreparedStatement sstm = null;

    private String tquery = "insert into team (scene_id, name, score, offside_line, offside) values (?, ?, ?, ?, ?)";
    private PreparedStatement tstm = null;

    private String pquery =
    "INSERT INTO player (scene_id, unum, pos_x, pos_y, vel_x, vel_y, acc_x, acc_y, side, stamina, angle, angle_neck, angle_visible, view_quality, say_message, catch_dir, offside, mode, type, player_type, interception_time, role_number, kick_count, dash_count, turn_count, say_count, turn_neck_count, catch_count, move_count, change_view_count, hear_message, state, reliable, id) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
    private PreparedStatement pstm = null;

    private String pwmquery =
    "INSERT INTO player_world_model ( player_id , positionX , positionY , positionIsReliable , isGoalie , simpleBody , speedX , speedY , ballInterceptionTime , stamina , playerType , lastState , roleNumber , bestPosX , bestPosY , neckAngle , bodyAngle , ballPositionX , ballPositionY , ballSpeedX , ballSpeedY , ballInterceptX , ballInterceptY , oppNum , markAreaX , markAreaY , markAreaRad , shouldMark , oppInterceptX , oppInterceptY , offsideY , doneAPass , passToPlayer , holes , based_on_vi , stateInfos, id ) " +
    "VALUES (? , ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
    private PreparedStatement pwmstm = null;

    private String spquery =
            "INSERT INTO simple_player ( id , pwm_id , type , positionX , positionY , positionIsReliable , isGoalie , simpleBody , speedX , speedY , ballInterceptionTime , stamina , playerType , lastState , roleNumber , bestPosX , bestPosY ) " +
            "VALUES (NULL , ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
    private PreparedStatement spstm = null;

    // IDs werden nicht automatisch erhht sondern manuell gezhlt
    private static int gameID = -1;
    private static int clientSceneID = -1;
    private static int playerID = -1;
    private static int pwmID = -1;

    public static SceneSet sceneSet = null;
    private static int currentRecordedScene = 0; // der Zeitpunkt, in dem die Aufnahme gestartet wird
    private static int sceneCounter = 0;
    final private int SCENE_DELAY = 15; // delay bis zur Aufnahme, damit PlayerWorldModel bekommen wird
    final private int CPU_SAVING_DELAY = 100;


    private static RobocupDatabase rcDatabase = null;



    /**
     * Ein neues Spiel wird erzeugt
     * @return boolean
     */
    public static boolean saveGame()
    {
        try {
            String query = "insert into game (date) values (?)";
            PreparedStatement stm2 = connection.prepareStatement(query);
            stm2.setTimestamp(1, new Timestamp(System.currentTimeMillis()));
            stm2.executeUpdate();
            connection.commit();

            int autoIncKeyFromApi = -1;

            ResultSet rs = stm2.getGeneratedKeys();

            if (rs.next()) {
                autoIncKeyFromApi = rs.getInt(1);
                gameID = autoIncKeyFromApi;
                System.out.format("[RobocupDatabase] New game %d started!\n", gameID);
                sceneCounter = 0;
            } else
                return false;
            rs.close();

        }
        catch (SQLException ex) {
            ex.printStackTrace();
        }

        return true;
    }

    /**
     * Bestimmt die ID fr den nchsten einzufgenden Eintrag
     * @param tableName String Name der Tabelle
     * @return int ID
     */
    private static int getNextID(String tableName)
    {
        int id = -1;

        try {
            Statement stm = connection.createStatement();
            ResultSet rs = stm.executeQuery("SELECT max( id ) FROM " + tableName);
            if (rs.next())
                id = rs.getInt(1) + 1;

            rs.close();
        } catch (SQLException ex) {
            ex.printStackTrace();
        }
        return id;
    }

    /**
     *
     * @param scene ClientScene
     * @return boolean
     * @deprecated Zu langsam
     */
    public boolean saveScene(ClientScene scene)
    {
        if (!recorded)
            return false;

        long tstart = System.currentTimeMillis();
        try {
            String query = "insert into client_scene (game_id, play_mode, ball_pos_x, ball_pos_y, ball_vel_x, ball_vel_y, time, ball_mode, ball_bestPosX, ball_bestPosY, ball_accX, ball_accY, id) values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
            PreparedStatement stm = connection.prepareStatement(query);
            stm.setInt(1, gameID);
            stm.setInt(2, scene.pmode.pmode);
            stm.setDouble(3, scene.ball.pos.getX());
            stm.setDouble(4, scene.ball.pos.getY());
            stm.setDouble(5, scene.ball.vel.getX());
            stm.setDouble(6, scene.ball.vel.getY());
            stm.setInt(7, scene.time);
            stm.setInt(8, scene.ball.mode);
            if (scene.ball.bestPos != null)
            {
                stm.setDouble(9, scene.ball.bestPos.getX());
                stm.setDouble(10, scene.ball.bestPos.getY());
            }
            else
            {
                stm.setNull(9, Types.FLOAT);
                stm.setNull(10, Types.FLOAT);
            }

            stm.setDouble(11, scene.ball.acc.getX());
            stm.setDouble(12, scene.ball.acc.getY());
            stm.setInt(13, clientSceneID);

            stm.executeUpdate();
            saveTeam(clientSceneID, scene.left);
            saveTeam(clientSceneID, scene.right);

            for (Player player: scene.player)
                savePlayer(player);

            System.out.format("[RobocupDatabase] saveScene %d in %f seconds!\n", clientSceneID++, (float)(System.currentTimeMillis() - tstart) / 1000.0);
/*
            long sceneID = -1;

            ResultSet rs = stm.getGeneratedKeys();

            if (rs.next()) {
                sceneID = rs.getLong(1);
                saveTeam(sceneID, scene.left);
                saveTeam(sceneID, scene.right);

                for (Player player: scene.player)
                    savePlayer(sceneID, player);

                System.out.format("[RobocupDatabase] saveScene %d in %f seconds!\n", sceneID, (float)(System.currentTimeMillis() - tstart) / 1000.0);
            } else
                return false;

            rs.close();
*/

        }

        catch (SQLException ex) {
            ex.printStackTrace();
        }

        return true;

    }

    /**
     *
     * @param clientsceneID long
     * @param team Team
     * @return boolean
     * @deprecated Zu langsam
     */
    public boolean saveTeam(long clientsceneID, Team team)
    {
        try {
            String query = "insert into team (scene_id, name, score, offside_line, offside) values (?, ?, ?, ?, ?)";
            PreparedStatement stm = connection.prepareStatement(query);
            stm.setLong(1, clientsceneID);
            stm.setString(2, team.name);
            stm.setInt(3, team.score);
            stm.setDouble(4, team.offsideline);
            stm.setBoolean(5, team.offside);

            stm.executeUpdate();
        }

        catch (SQLException ex) {
            ex.printStackTrace();
        }
        return true;

    }

    /**
     *
     * @param clientsceneID long
     * @param player Player
     * @return boolean
     * @deprecated Zu langsam
     */
    public boolean savePlayer(Player player)
    {
        try {
            String query = //"insert into player (scene_id, number, mode) values (?, ?, ?)";
            "INSERT INTO player (scene_id, unum, pos_x, pos_y, vel_x, vel_y, acc_x, acc_y, side, stamina, angle, angle_neck, angle_visible, view_quality, say_message, catch_dir, offside, mode, type, player_type, interception_time, role_number, kick_count, dash_count, turn_count, say_count, turn_neck_count, catch_count, move_count, change_view_count, hear_message, state, reliable, id) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
            PreparedStatement stm = connection.prepareStatement(query);
            stm.setLong(1, clientSceneID);
            stm.setInt(2, player.unum);
            stm.setDouble(3, player.pos.getX());
            stm.setDouble(4, player.pos.getY());
            stm.setDouble(5, player.vel.getX());
            stm.setDouble(6, player.vel.getY());
            stm.setDouble(7, player.acc.getX());
            stm.setDouble(8, player.acc.getY());
            stm.setInt(9, player.side);
            stm.setInt(10, player.stamina);
            stm.setFloat(11, player.angle);
            stm.setFloat(12, player.angleNeck);
            stm.setFloat(13, player.angleVisible);
            stm.setInt(14, player.viewQuality);
            stm.setString(15, player.sayMessage);
            stm.setDouble(16, player.catchDir);
            stm.setBoolean(17, player.offside);
            stm.setInt(18, player.mode);
            stm.setInt(19, player.type);
            stm.setInt(20, player.playerType);
            stm.setInt(21, player.interceptionTime);
            stm.setInt(22, player.roleNumber);
            stm.setInt(23, player.kickCount);
            stm.setInt(24, player.dashCount);
            stm.setInt(25, player.turnCount);
            stm.setInt(26, player.sayCount);
            stm.setInt(27, player.turnNeckCount);
            stm.setInt(28, player.catchCount);
            stm.setInt(29, player.moveCount);
            stm.setInt(30, player.changeViewCount);
            stm.setString(31, player.hearMessage.toString()); // testen
            stm.setString(32, player.state);
            stm.setBoolean(33, player.reliable);
            stm.setInt(34, playerID);

            stm.executeUpdate();
            savePlayerWorldModel(player.getWorldModel());
            playerID++;
/*
            long playerID = -1;

            ResultSet rs = stm.getGeneratedKeys();

            if (rs.next()) {
                playerID = rs.getLong(1);
//                System.out.format("[RobocupDatabase] savePlayer %d!\n", playerID);
                savePlayerWorldModel(playerID, player.getWorldModel());
            } else
                return false;

            rs.close();
*/


        }

        catch (SQLException ex) {
            ex.printStackTrace();
        }

        return true;

    }


    /**
     *
     * @param pwm PlayerWorldModel
     * @return boolean
     * @deprecated Zu langsam
     */
    public boolean savePlayerWorldModel(PlayerWorldModel pwm)
    {
        if (pwm == null)
            return false;
        try {
            String query = //"insert into player_world_model (player_id, neckAngle, bodyAngle, ballPositionX, ballPositionY, ballSpeedX) values (?, ?, ?, ?, ?, ?)";
            "INSERT INTO player_world_model ( player_id , positionX , positionY , positionIsReliable , isGoalie , simpleBody , speedX , speedY , ballInterceptionTime , stamina , playerType , lastState , roleNumber , bestPosX , bestPosY , neckAngle , bodyAngle , ballPositionX , ballPositionY , ballSpeedX , ballSpeedY , ballInterceptX , ballInterceptY , oppNum , markAreaX , markAreaY , markAreaRad , shouldMark , oppInterceptX , oppInterceptY , offsideY , doneAPass , passToPlayer , holes , based_on_vi , stateInfos, id ) " +
            "VALUES (? , ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";


            PreparedStatement stm = connection.prepareStatement(query);
            stm.setLong(1, playerID);
            stm.setDouble(2, pwm.positionX);
            stm.setDouble(3, pwm.positionY);
            stm.setBoolean(4, pwm.positionIsReliable);
            stm.setBoolean(5, pwm.isGoalie);
            stm.setDouble(6, pwm.simpleBody);
            stm.setDouble(7, pwm.speedX);
            stm.setDouble(8, pwm.speedY);
            stm.setInt(9, pwm.ballInterceptionTime);
            stm.setInt(10, pwm.stamina);
            stm.setDouble(11, pwm.playerType);
            stm.setInt(12, pwm.lastState.ordinal());
            stm.setDouble(13, pwm.roleNumber);
            if (pwm.bestPos != null)
            {
                stm.setDouble(14, pwm.bestPos.getX());
                stm.setDouble(15, pwm.bestPos.getY());
            }
            else
            {
                stm.setNull(14, Types.FLOAT);
                stm.setNull(15, Types.FLOAT);
            }

            stm.setDouble(16, pwm.neckAngle);
            stm.setDouble(17, pwm.bodyAngle);
            stm.setDouble(18, pwm.ballPositionX);
            stm.setDouble(19, pwm.ballPositionY);
            stm.setDouble(20, pwm.ballSpeedX);
            stm.setDouble(21, pwm.ballSpeedY);
            stm.setDouble(22, pwm.ballInterceptX);
            stm.setDouble(23, pwm.ballInterceptY);
            stm.setInt(24, pwm.oppNum);
            stm.setDouble(25, pwm.markAreaX);
            stm.setDouble(26, pwm.markAreaY);
            stm.setDouble(27, pwm.markAreaRad);
            stm.setBoolean(28, pwm.shouldMark);
            stm.setDouble(29, pwm.oppInterceptX);
            stm.setDouble(30, pwm.oppInterceptY);
            stm.setDouble(31, pwm.offsideY);
            stm.setBoolean(32, pwm.doneAPass);
            stm.setDouble(33, pwm.passToPlayer);
            stm.setInt(34, pwm.holes);
            stm.setBoolean(35, pwm.based_on_vi);
            stm.setString(36, pwm.stateInfos.toString()); // testen
            stm.setInt(37, pwmID); // testen

            stm.executeUpdate();

            saveSimplePlayers(pwm.teammates, "team_mate");
            saveSimplePlayers(pwm.nextTeammates, "next_team_mate_mate");
            saveSimplePlayers(pwm.opponents, "opponent");
            saveSimplePlayers(pwm.nextOpponents, "next_opponent");
            pwmID++;
/*
            long pwmID = -1;

            ResultSet rs = stm.getGeneratedKeys();

            if (rs.next()) {
                pwmID = rs.getLong(1);
//                System.out.format("[RobocupDatabase] savePlayerWorldModel %d!\n", pwmID);
                saveSimplePlayers(pwmID, pwm.teammates, "team_mate");
                saveSimplePlayers(pwmID, pwm.nextTeammates, "next_team_mate_mate");
                saveSimplePlayers(pwmID, pwm.opponents, "opponent");
                saveSimplePlayers(pwmID, pwm.nextOpponents, "next_opponent");

            } else
                return false;

            rs.close();
*/
      }

        catch (SQLException ex) {
            ex.printStackTrace();
        }
        return true;

    }


   public boolean saveSimplePlayers(SimplePlayer[] sps,
                                    String type) {
       try {
           String query =
                   "INSERT INTO simple_player ( id , pwm_id , type , positionX , positionY , positionIsReliable , isGoalie , simpleBody , speedX , speedY , ballInterceptionTime , stamina , playerType , lastState , roleNumber , bestPosX , bestPosY ) " +
                   "VALUES (NULL , ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";

           connection.setAutoCommit(false);
           PreparedStatement stm = connection.prepareStatement(query);

           for (SimplePlayer sp : sps) {
               if (sp == null)
                   break;
               stm.setLong(1, pwmID);
               stm.setString(2, type);
               stm.setDouble(3, sp.positionX);
               stm.setDouble(4, sp.positionY);
               stm.setBoolean(5, sp.positionIsReliable);
               stm.setBoolean(6, sp.isGoalie);
               stm.setDouble(7, sp.simpleBody);
               stm.setDouble(8, sp.speedX);
               stm.setDouble(9, sp.speedY);
               stm.setDouble(10, sp.ballInterceptionTime);
               stm.setInt(11, sp.stamina);
               stm.setInt(12, sp.playerType);
               stm.setInt(13, sp.lastState.ordinal());
               stm.setInt(14, sp.roleNumber);

               if (sp.bestPos != null) {
                   stm.setDouble(15, sp.bestPos.getX());
                   stm.setDouble(16, sp.bestPos.getY());
               } else {
                   stm.setNull(15, Types.FLOAT);
                   stm.setNull(16, Types.FLOAT);
               }

               stm.addBatch();

           }
           stm.executeBatch();
           connection.commit();
           connection.setAutoCommit(true);
//           int updateCount = stm.executebUpdate();
       }

       catch (SQLException ex) {
           ex.printStackTrace();
       }
       return true;

   }

   /**
    *
    * @param pwmID long
    * @param sp SimplePlayer
    * @param type String
    * @return boolean
    * @deprecated Zu langsam
    */
   public boolean saveSimplePlayer(long pwmID, SimplePlayer sp, String type)
    {
        if (sp == null)
            return false;
        try {
            String query =
            "INSERT INTO simple_player ( id , pwm_id , type , positionX , positionY , positionIsReliable , isGoalie , simpleBody , speedX , speedY , ballInterceptionTime , stamina , playerType , lastState , roleNumber , bestPosX , bestPosY ) " +
            "VALUES (NULL , ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";

            PreparedStatement stm = connection.prepareStatement(query);
            stm.setLong(1, pwmID);
            stm.setString(2, type);
            stm.setDouble(3, sp.positionX);
            stm.setDouble(4, sp.positionY);
            stm.setBoolean(5, sp.positionIsReliable);
            stm.setBoolean(6, sp.isGoalie);
            stm.setDouble(7, sp.simpleBody);
            stm.setDouble(8, sp.speedX);
            stm.setDouble(9, sp.speedY);
            stm.setDouble(10, sp.ballInterceptionTime);
            stm.setInt(11, sp.stamina);
            stm.setInt(12, sp.playerType);
            stm.setInt(13, sp.lastState.ordinal());
            stm.setInt(14, sp.roleNumber);
            stm.setDouble(15, sp.bestPos != null? sp.bestPos.getX() : -1);
            stm.setDouble(16, sp.bestPos != null? sp.bestPos.getY() : -1);

            stm.executeUpdate();
        }

        catch (SQLException ex) {
            ex.printStackTrace();
        }
        return true;

    }

    public void setSqlHost(String sqlHost) {
        this.sqlHost = sqlHost;
    }

    public void setSqlDB(String sqlDB) {
        this.sqlDB = sqlDB;
    }

    public void setSqlUser(String sqlUser) {
        this.sqlUser = sqlUser;
    }

    public void setSqlPassword(String sqlPassword) {
        this.sqlPassword = sqlPassword;
    }

    public String getSqlHost() {
        return sqlHost;
    }

    public String getSqlDB() {
        return sqlDB;
    }

    public String getSqlUser() {
        return sqlUser;
    }

    public String getSqlPassword() {
        return sqlPassword;
    }

    public Connection getConnection() {
        return connection;
    }



    /**
     * RobocupDatabase
     *
     * @param host String
     * @param db String
     * @param user String
     * @param password String
     */
    public RobocupDatabase(String host, String db, String user, String password) {
        sqlHost = host;
        sqlDB = db;
        sqlUser = user;
        sqlPassword = password;

        try {
            Class.forName("com.mysql.jdbc.Driver").newInstance();
            connection = DriverManager.getConnection("jdbc:mysql://" + sqlHost + "/" + sqlDB, sqlUser, sqlPassword);
            connection.setAutoCommit(false);


            // vorbereitet die Anfragen fr Batch-Arbeit
            sstm = connection.prepareStatement(squery);
            tstm = connection.prepareStatement(tquery);
            pstm = connection.prepareStatement(pquery);
            pwmstm = connection.prepareStatement(pwmquery);
            spstm = connection.prepareStatement(spquery);

            // Holt IDs fr nchste Records
            clientSceneID = getNextID("client_scene");
            playerID = getNextID("player");
            pwmID = getNextID("player_world_model");

            System.out.println("[RobocupDatabase] Database connected");
            saveGame();
        } catch (ClassNotFoundException ex) {
            ex.printStackTrace();
        } catch (IllegalAccessException ex) {
            ex.printStackTrace();
        } catch (InstantiationException ex) {
            ex.printStackTrace();
        } catch (SQLException ex) {
            ex.printStackTrace();
        }

    }

    public RobocupDatabase(SceneSet sceneSet) {
        try {
            DatabaseConfig dbConfig = DatabaseConfig.getInstance();
            dbConfig.load();
            sqlHost = dbConfig.getSqlHost();
            sqlDB = dbConfig.getSqlDB();
            sqlUser = dbConfig.getSqlUser();
            sqlPassword = dbConfig.getSqlPassword();

            Class.forName("com.mysql.jdbc.Driver").newInstance();
            connection = DriverManager.getConnection("jdbc:mysql://" + sqlHost + "/" + sqlDB, sqlUser, sqlPassword);
            connection.setAutoCommit(false);

            DatabaseReader.sceneSet = sceneSet;

            // vorbereitet die Anfragen fr Batch-Arbeit
            sstm = connection.prepareStatement(squery);
            tstm = connection.prepareStatement(tquery);
            pstm = connection.prepareStatement(pquery);
            pwmstm = connection.prepareStatement(pwmquery);
            spstm = connection.prepareStatement(spquery);

            // Holt IDs fr nchste Records
            clientSceneID = getNextID("client_scene");
            playerID = getNextID("player");
            pwmID = getNextID("player_world_model");

            System.out.println("[RobocupDatabase] Database connected");
        } catch (ClassNotFoundException ex) {
            ex.printStackTrace();
        } catch (IllegalAccessException ex) {
            ex.printStackTrace();
        } catch (InstantiationException ex) {
            ex.printStackTrace();
        } catch (SQLException ex) {
            ex.printStackTrace();
        }
    }

    public static synchronized RobocupDatabase getInstance(SceneSet sceneSet) {
        if (null == rcDatabase)
            rcDatabase = new RobocupDatabase(sceneSet);

        if (connection == null)
            rcDatabase = null;

        return rcDatabase;
    }

    public static synchronized RobocupDatabase getInstance() {
        return rcDatabase;
    }

    /**
     * Initialisiert die Instanz nochmal, nachdem Datenbankinfo geaendert wurde
     */
    public static synchronized void restartInstance()
    {
        try {
            connection.close();
        } catch (SQLException ex) {
            ex.printStackTrace();
        }

        rcDatabase = null;
        if (sceneSet == null) {
        	sceneSet = WorldModel.getSceneSet();
        }
        	
        rcDatabase = new RobocupDatabase(sceneSet);
    }


    public static boolean recorded = false;

    public static void recordEnable()
    {
        recorded = true;
        if (sceneSet == null) {
        	sceneSet = WorldModel.getSceneSet();
        }
        currentRecordedScene = sceneSet.size();
    }

    public static void recordDisable()
    {
        recorded = false;
    }

/******************************************************************************/
    /**
     * Speichert Szenen solange das Attribut recorded true ist
     * Es wird Szene-versetzt (SCENE_DELAY) gespeichert
     * @return boolean true falls erfolgreich
     */
    public synchronized boolean saveSceneBatch()
    {
        if (!recorded)
            return false;

        if (sceneSet == null) {
        	sceneSet = WorldModel.getSceneSet();
        }
        
        if (currentRecordedScene + SCENE_DELAY > sceneSet.size())
        {
            System.out.format("[RobocupDatabase] %d scenes until recording!\n", currentRecordedScene + SCENE_DELAY - sceneSet.size());
            return false;
        }

        ClientScene scene = sceneSet.elementAt(currentRecordedScene++);

        long tstart = System.currentTimeMillis();
        try {
            sstm.setInt(1, gameID);
            sstm.setInt(2, scene.pmode.pmode);
            sstm.setDouble(3, scene.ball.pos.getX());
            sstm.setDouble(4, scene.ball.pos.getY());
            sstm.setDouble(5, scene.ball.vel.getX());
            sstm.setDouble(6, scene.ball.vel.getY());
            sstm.setInt(7, scene.time);
            sstm.setInt(8, scene.ball.mode);
            if (scene.ball.bestPos != null) // bestPos wird nicht benutzt?
            {
                sstm.setDouble(9, scene.ball.bestPos.getX());
                sstm.setDouble(10, scene.ball.bestPos.getY());
            }
            else
            {
                sstm.setNull(9, Types.FLOAT);
                sstm.setNull(10, Types.FLOAT);
            }

            sstm.setDouble(11, scene.ball.acc.getX());
            sstm.setDouble(12, scene.ball.acc.getY());
            sstm.setInt(13, clientSceneID++);

            sstm.executeUpdate();
            saveTeamBatch(scene.left);
            saveTeamBatch(scene.right);

            for (Player player: scene.player)
                savePlayerBatch(player);

            tstm.executeBatch();
            pstm.executeBatch();
            pwmstm.executeBatch();
            spstm.executeBatch();

            connection.commit();

            System.out.format("[RobocupDatabase] saveScene %d in %.2f seconds!\n", sceneCounter++, (float)(System.currentTimeMillis() - tstart) / 1000.0);

        }

        catch (SQLException ex) {
            ex.printStackTrace();
        }

        return true;

    }

    /**
     * speichert ein Team
     * @param team Team
     * @return boolean
     */
    private boolean saveTeamBatch(Team team)
    {
        try {
            tstm.setLong(1, clientSceneID);
            tstm.setString(2, team.name);
            tstm.setInt(3, team.score);
            tstm.setDouble(4, team.offsideline);
            tstm.setBoolean(5, team.offside);

            tstm.addBatch();
        }
        catch (SQLException ex) {
            ex.printStackTrace();
        }
        return true;
    }

    /**
     * Speichert einen Spieler
     * @param player Player
     * @return boolean
     */
    private boolean savePlayerBatch(Player player)
    {
        try {
            pstm.setLong(1, clientSceneID);
            pstm.setInt(2, player.unum);
            pstm.setDouble(3, player.pos.getX());
            pstm.setDouble(4, player.pos.getY());
            pstm.setDouble(5, player.vel.getX());
            pstm.setDouble(6, player.vel.getY());
            pstm.setDouble(7, player.acc.getX());
            pstm.setDouble(8, player.acc.getY());
            pstm.setInt(9, player.side);
            pstm.setInt(10, player.stamina);
            pstm.setFloat(11, player.angle);
            pstm.setInt(12, player.angleNeck);
            pstm.setInt(13, player.angleVisible);
            pstm.setInt(14, player.viewQuality);
            pstm.setString(15, player.sayMessage);
            pstm.setDouble(16, player.catchDir);
            pstm.setBoolean(17, player.offside);
            pstm.setInt(18, player.mode);
            pstm.setInt(19, player.type);
            pstm.setInt(20, player.playerType);
            pstm.setInt(21, player.interceptionTime);
            pstm.setInt(22, player.roleNumber);
            pstm.setInt(23, player.kickCount);
            pstm.setInt(24, player.dashCount);
            pstm.setInt(25, player.turnCount);
            pstm.setInt(26, player.sayCount);
            pstm.setInt(27, player.turnNeckCount);
            pstm.setInt(28, player.catchCount);
            pstm.setInt(29, player.moveCount);
            pstm.setInt(30, player.changeViewCount);
            pstm.setString(31, player.hearMessage.toString()); // testen
            pstm.setString(32, player.state);
            pstm.setBoolean(33, player.reliable);
            pstm.setInt(34, playerID);

            pstm.addBatch();
            savePlayerWorldModelBatch(player.getWorldModel());
            playerID++;
        }
        catch (SQLException ex) {
            ex.printStackTrace();
        }

        return true;
    }

    /**
     * Speichert ein Weltmodel
     * @param pwm PlayerWorldModel
     * @return boolean
     */
    private boolean savePlayerWorldModelBatch(PlayerWorldModel pwm) {
        if (pwm == null)
        {
            pwmID++; // auch wenn der Spieler kein Weltmodel hat (ein Gegner!) wird die ID erhht um diese "ID-Lcken" zu erkennen
            return false;
        }
        try {
            pwmstm.setLong(1, playerID);
            pwmstm.setDouble(2, pwm.positionX);
            pwmstm.setDouble(3, pwm.positionY);
            pwmstm.setBoolean(4, pwm.positionIsReliable);
            pwmstm.setBoolean(5, pwm.isGoalie);
            pwmstm.setDouble(6, pwm.simpleBody);
            pwmstm.setDouble(7, pwm.speedX);
            pwmstm.setDouble(8, pwm.speedY);
            pwmstm.setInt(9, pwm.ballInterceptionTime);
            pwmstm.setInt(10, pwm.stamina);
            pwmstm.setDouble(11, pwm.playerType);
            pwmstm.setInt(12, pwm.lastState.ordinal());
            pwmstm.setDouble(13, pwm.roleNumber);
            if (pwm.bestPos != null) {
                pwmstm.setDouble(14, pwm.bestPos.getX());
                pwmstm.setDouble(15, pwm.bestPos.getY());
            } else {
                pwmstm.setNull(14, Types.FLOAT);
                pwmstm.setNull(15, Types.FLOAT);
            }

            pwmstm.setDouble(16, pwm.neckAngle);
            pwmstm.setDouble(17, pwm.bodyAngle);
            pwmstm.setDouble(18, pwm.ballPositionX);
            pwmstm.setDouble(19, pwm.ballPositionY);
            pwmstm.setDouble(20, pwm.ballSpeedX);
            pwmstm.setDouble(21, pwm.ballSpeedY);
            pwmstm.setDouble(22, pwm.ballInterceptX);
            pwmstm.setDouble(23, pwm.ballInterceptY);
            pwmstm.setInt(24, pwm.oppNum);
            pwmstm.setDouble(25, pwm.markAreaX);
            pwmstm.setDouble(26, pwm.markAreaY);
            pwmstm.setDouble(27, pwm.markAreaRad);
            pwmstm.setBoolean(28, pwm.shouldMark);
            pwmstm.setDouble(29, pwm.oppInterceptX);
            pwmstm.setDouble(30, pwm.oppInterceptY);
            pwmstm.setDouble(31, pwm.offsideY);
            pwmstm.setBoolean(32, pwm.doneAPass);
            pwmstm.setDouble(33, pwm.passToPlayer);
            pwmstm.setInt(34, pwm.holes);
            pwmstm.setBoolean(35, pwm.based_on_vi);
            pwmstm.setString(36, pwm.stateInfos.toString()); // testen
            pwmstm.setInt(37, pwmID); // testen

            pwmstm.addBatch();

            saveSimplePlayersBatch(pwm.teammates, "team_mate");
            saveSimplePlayersBatch(pwm.nextTeammates, "next_team_mate_mate");
            saveSimplePlayersBatch(pwm.opponents, "opponent");
            saveSimplePlayersBatch(pwm.nextOpponents, "next_opponent");
            pwmID++;
        }

        catch (SQLException ex) {
            ex.printStackTrace();
        }
        return true;
    }

    /**
     * SimplePlayers werden gespeichert
     * @param sps SimplePlayer[]
     * @param type String
     * @return boolean
     */
    private boolean saveSimplePlayersBatch(SimplePlayer[] sps, String type) {
        try {
            for (SimplePlayer sp : sps) {
                if (sp == null)
                    break;
                spstm.setLong(1, pwmID);
                spstm.setString(2, type);
                spstm.setDouble(3, sp.positionX);
                spstm.setDouble(4, sp.positionY);
                spstm.setBoolean(5, sp.positionIsReliable);
                spstm.setBoolean(6, sp.isGoalie);
                spstm.setDouble(7, sp.simpleBody);
                spstm.setDouble(8, sp.speedX);
                spstm.setDouble(9, sp.speedY);
                spstm.setDouble(10, sp.ballInterceptionTime);
                spstm.setInt(11, sp.stamina);
                spstm.setInt(12, sp.playerType);
                spstm.setInt(13, sp.lastState.ordinal());
                spstm.setInt(14, sp.roleNumber);

                if (sp.bestPos != null) {
                    spstm.setDouble(15, sp.bestPos.getX());
                    spstm.setDouble(16, sp.bestPos.getY());
                } else {
                    spstm.setNull(15, Types.FLOAT);
                    spstm.setNull(16, Types.FLOAT);
                }

                spstm.addBatch();
            }
        }

        catch (SQLException ex) {
            ex.printStackTrace();
        }
        return true;
   }

   public void finalizeGame() {
	   if (sceneSet == null) {
       	sceneSet = WorldModel.getSceneSet();
       }
	   
       ClientScene lastScene = sceneSet.lastScene();
       String query = String.format(
               "UPDATE game SET team_left = '%s', team_right = '%s', score_left = %d, score_right = %d, scene_number = %d WHERE id = %d",
               lastScene.left.name, lastScene.right.name, lastScene.left.score, lastScene.right.score, sceneCounter,
               gameID);

       try {
           Statement stm = connection.createStatement();
           stm.executeUpdate(query);
           stm.close();
           connection.commit();
       } catch (SQLException ex) {
           ex.printStackTrace();
       }
   }

   public void run() {
       try {
           setName("DAI-NAMITE SoccerScope RobocupDatabase");
       } catch (SecurityException e) {
           System.err.println("[RobocupDatabase] Could not set the thread name");
       }

       while (true) {
           if (recorded)
               saveSceneBatch();
           try {
               Thread.sleep(CPU_SAVING_DELAY); // macht eine Pause, damit der Thread nicht den ganzen Prozessor frisst :)
           } catch (InterruptedException ex) {
               ex.printStackTrace();
           }
       }
 /*
while (recorded) {
    saveSceneBatch();
    try {
        Thread.sleep(CPU_SAVING_DELAY); // macht eine Pause, damit der Thread nicht den ganzen Prozessor frisst :)
    } catch (InterruptedException ex) {
        ex.printStackTrace();
    }
}
*/
   }



   /**
    * Schliet alle Verbindungen mit der Datenbank
    */
   public void dispose()
   {
        try {
            sstm.close();
            tstm.close();

            pstm.close();

            pwmstm.close();

            spstm.close();

            connection.close();

            connection = null;
            rcDatabase = null;
        } catch (SQLException ex) {
            ex.printStackTrace();
        }

        System.out.println("[RobocupDatabase] Database disconnected");

   }

}
