/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope.model.agentplan;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.ImageIcon;

import soccerscope.model.ClientScene;
import soccerscope.util.geom.Point2f;
import soccerscope.view.layer.Layer;

public class AgentPlanCommand {

	private int level; //

	private int index; // player number index

	private int type;

	public AgentPlanCommand(int level, int selfUnum) {
		this.level = level;
		this.index = selfUnum - 1;
	}

	public int getLevel() {
		return this.level;
	}

	public int getType() {
		return this.type;
	}

	public String getComment() {
		return this.comment;
	}

	public final static int LINE = 1;

	public final static int RECT = 2;

	public final static int ELLIPSE = 3;

	public final static int POLYLINE = 4;

	public final static int STATE = 5;

	public final static int TARGET = 6;

	public final static int COMMENT = 7;

	public final static int ARROW = 8;

	public final static int FILLRECT = 9;

	public final static int FILLELLIPSE = 10;

	private Point2f start = null;

	private Point2f end = null;

	private Point2f topleft = null;

	private Point2f size;

	private ArrayList<Point2f> points = null;

	private int rotate;

	private Point2f targetPos = null;

	private final static Image targetImage = new ImageIcon(ClassLoader
			.getSystemResource("soccerscope/image/BlueDown16.gif")).getImage();

	private int offsetx;

	private int offsety;

	private Point2f commentPos = null;

	private String comment = null;

	private int state;

	private Point2f statePos = null;

	private int stateUnum;

	private final static Image stateImage1 = new ImageIcon(ClassLoader
			.getSystemResource("soccerscope/image/comeon.gif")).getImage();

	private final static Image stateImage2 = new ImageIcon(ClassLoader
			.getSystemResource("soccerscope/image/stop.gif")).getImage();

	private final static Image stateImage3 = new ImageIcon(ClassLoader
			.getSystemResource("soccerscope/image/kaminari.gif")).getImage();

	// private final static Image stateImage4 = new ImageIcon(ClassLoader
	// .getSystemResource("soccerscope/image/light.gif")).getImage();
	//
	// private final static Image stateImage5 = new ImageIcon(ClassLoader
	// .getSystemResource("soccerscope/image/star.gif")).getImage();
	//
	// private final static Image stateImage6 = new ImageIcon(ClassLoader
	// .getSystemResource("soccerscope/image/heart.gif")).getImage();

	private Color color = Color.white;

	public void parse(Context context, ClientScene clientScene)
			throws ParseException {
		// <Line-Command>:="line" <Point> <Point> [<Color>] [<Comment>]
		if (context.currentToken().equals("line")) {
			context.nextToken();
			this.start = context.currentPoint();
			context.nextToken();
			this.end = context.currentPoint();

			context.nextToken();
			if (context.isColorToken())
				this.color = context.currentColor();
			this.comment = context.currentComment();

			this.commentPos = this.end;
			this.type = LINE;
		} else if (context.currentToken().equals("arrow")) {
			context.nextToken();
			this.start = context.currentPoint();
			context.nextToken();
			this.end = context.currentPoint();

			context.nextToken();
			if (context.isColorToken())
				this.color = context.currentColor();
			this.comment = context.currentComment();

			this.commentPos = this.end;
			this.type = ARROW;
		}
		// <Rect-Command>:="rect" <Point> <Width> <Height> [<Rotate>] [<Color>]
		// [<Comment>]
		else if (context.currentToken().equals("rect")) {
			context.nextToken();
			this.topleft = context.currentPoint();
			context.nextToken();
			this.size = new Point2f();
			this.size.x = context.currentFloat();
			context.nextToken();
			this.size.y = context.currentFloat();

			context.nextToken();
			if (context.isColorToken())
				this.color = context.currentColor();
			this.comment = context.currentComment();

			this.commentPos = this.topleft;
			this.type = RECT;

		}
		// <FillRect-Command>:="fillrect" <Point> <Width> <Height> [<Rotate>]
		// [<Color>] [<Comment>]
		else if (context.currentToken().equals("fillrect")) {
			context.nextToken();
			this.topleft = context.currentPoint();
			context.nextToken();
			this.size = new Point2f();
			this.size.x = context.currentFloat();
			context.nextToken();
			this.size.y = context.currentFloat();

			context.nextToken();
			if (context.isColorToken())
				this.color = context.currentColor();
			this.comment = context.currentComment();

			this.commentPos = this.topleft;
			this.type = FILLRECT;

		}
		// <Ellipse-Command>:="ellipse" <Point> <Width> <Height> [<Rotate>]
		// [<Color>] [<Comment>]
		else if (context.currentToken().equals("ellipse")) {
			context.nextToken();
			this.topleft = context.currentPoint();
			context.nextToken();
			this.size = new Point2f();
			this.size.x = context.currentFloat();
			context.nextToken();
			this.size.y = context.currentFloat();

			context.nextToken();
			if (context.isColorToken())
				this.color = context.currentColor();
			this.comment = context.currentComment();

			this.commentPos = this.topleft;
			this.type = ELLIPSE;

		}
		// <FillEllipse-Command>:="fillellipse" <Point> <Width> <Height>
		// [<Rotate>] [<Color>] [<Comment>]
		else if (context.currentToken().equals("fillellipse")) {
			context.nextToken();
			this.topleft = context.currentPoint();
			context.nextToken();
			this.size = new Point2f();
			this.size.x = context.currentFloat();
			context.nextToken();
			this.size.y = context.currentFloat();

			context.nextToken();
			if (context.isColorToken())
				this.color = context.currentColor();
			this.comment = context.currentComment();

			this.commentPos = this.topleft;
			this.type = FILLELLIPSE;

		}
		// <Polyline-Command>:="polyline" <Point> <Point> <Point>* [<Color>]
		// [<Comment>]
		else if (context.currentToken().equals("polyline")) {
			context.nextToken();
			this.points = new ArrayList<Point2f>();
			this.points.add(context.currentPoint());
			context.nextToken();
			Point2f last;
			do {
				last = context.currentPoint();
				this.points.add(last);
				context.nextToken();
			} while (context.isPointToken());

			if (context.isColorToken())
				this.color = context.currentColor();
			this.comment = context.currentComment();

			this.commentPos = last;
			this.type = POLYLINE;

		}
		// <rLine-Command>:="rline" <Vetor> [<Color>] [<Comment>]
		else if (context.currentToken().equals("rline")) {
			context.nextToken();
			this.start = clientScene.player[this.index].pos;
			this.end = context.currentPoint();
			this.end.add(this.start);

			context.nextToken();
			if (context.isColorToken())
				this.color = context.currentColor();
			this.comment = context.currentComment();

			this.commentPos = this.end;
			this.type = LINE;

		}
		// <rRect-Command>:="rrect" <Vector> <Width> <Height> [<Rotate>]
		// [<Color>] [<Comment>]
		else if (context.currentToken().equals("rrect")) {
			context.nextToken();
			this.topleft = new Point2f(clientScene.player[this.index].pos);
			this.size = new Point2f();
			this.size.x = context.currentFloat();
			context.nextToken();
			this.size.y = context.currentFloat();
			this.topleft.sub(this.size);
			this.size.scale(2);

			context.nextToken();
			if (context.isColorToken())
				this.color = context.currentColor();
			this.comment = context.currentComment();

			this.commentPos = this.topleft;
			this.type = RECT;

		}
		// <rEllipse-Command>:="rellipse" <Vector> <Width> <Height> [<Rotate>]
		// [<Comment>]
		else if (context.currentToken().equals("rellipse")) {
			context.nextToken();
			this.topleft = new Point2f(clientScene.player[this.index].pos);
			this.size = new Point2f();
			this.size.x = context.currentFloat();
			context.nextToken();
			this.size.y = context.currentFloat();
			this.topleft.sub(this.size);
			this.size.scale(2);

			context.nextToken();
			if (context.isColorToken())
				this.color = context.currentColor();
			this.comment = context.currentComment();

			this.commentPos = this.topleft;
			this.type = ELLIPSE;

		}
		// <rPolyline-Command>:="rpolyline" <Vector> <Vector>* [<Color>]
		// [<Comment>]
		else if (context.currentToken().equals("rpolyline")) {
			context.nextToken();
			this.points = new ArrayList<Point2f>();
			this.points.add(clientScene.player[this.index].pos);
			Point2f last = new Point2f(clientScene.player[this.index].pos);
			do {
				Point2f end = context.currentPoint();
				end.add(last);
				this.points.add(end);
				last.set(end);
				context.nextToken();
			} while (context.isPointToken());

			if (context.isColorToken())
				this.color = context.currentColor();
			this.comment = context.currentComment();

			this.commentPos = last;
			this.type = POLYLINE;

		}
		// <sLine-Command>:="sline" <Point> [<Color>] [<Comment>]
		else if (context.currentToken().equals("sline")) {
			context.nextToken();
			this.start = clientScene.player[this.index].pos;
			this.end = context.currentPoint();

			context.nextToken();
			if (context.isColorToken())
				this.color = context.currentColor();
			this.comment = context.currentComment();

			this.commentPos = this.end;
			this.type = LINE;

		}
		// <sPolyline-Command>:="spolyline" <Point> <Point>* [<Color>]
		// [<Comment>]
		else if (context.currentToken().equals("spolyline")) {
			context.nextToken();
			this.points = new ArrayList<Point2f>();
			this.points.add(clientScene.player[this.index].pos);
			context.nextToken();
			Point2f last;
			do {
				last = context.currentPoint();
				this.points.add(last);
				context.nextToken();
			} while (context.isPointToken());

			if (context.isColorToken())
				this.color = context.currentColor();
			this.comment = context.currentComment();

			this.commentPos = last;
			this.type = POLYLINE;

		}
		// <State-Command>:="state" <Number> [<Unum>]
		else if (context.currentToken().equals("state")) {
			context.nextToken();
			this.state = context.currentNumber();
			context.nextToken();
			if (context.currentToken() != null) {
				this.stateUnum = context.currentNumber();
			} else {
				// unumʤϼʬ
				this.stateUnum = this.index + 1;
			}
			this.statePos = new Point2f(
					clientScene.player[this.stateUnum - 1].pos);
			this.type = STATE;

		}
		// <Target-Command>:="target" <Unum>| "target" <Point>
		else if (context.currentToken().equals("target")) {
			context.nextToken();
			if (context.isPointToken()) {
				this.targetPos = context.currentPoint();
				this.offsetx = -targetImage.getWidth(null) / 2;
				this.offsety = -targetImage.getHeight(null);
			} else {
				this.targetPos = clientScene.player[context.currentNumber() - 1].pos;
				this.offsetx = -targetImage.getWidth(null) / 2;
				this.offsety = -targetImage.getHeight(null);
			}
			this.type = TARGET;

		}
		// <AgentAction-Command>:="(dash" <Number>")"
		else if (context.currentToken().equals("(dash")) {
			// ޥɰν
			String arg1 = context.nextToken();
			arg1 = arg1.replace(')', ' ');
			arg1 = arg1.trim();
			try {
				clientScene.player[this.index].acc = clientScene.player[this.index]
						.getDashAccelerate(Float.parseFloat(arg1));
			} catch (NumberFormatException ne) {
				return;
			}

		}
		// <AgentAction-Command>:="(kick" <Number> <Number>")"
		else if (context.currentToken().equals("(kick")) {
			// ޥɰν
			String arg1 = context.nextToken();
			String arg2 = context.nextToken();
			if (arg2 == null) {
				throw new ParseException("kick command: no direction");
			}
			arg1 = arg1.replace(')', ' ');
			arg1 = arg1.trim();
			arg2 = arg2.replace(')', ' ');
			arg2 = arg2.trim();

			// åˤ®٤η׻
			if (clientScene.player[this.index].isKickable(clientScene.ball.pos)) {
				try {
					clientScene.ball.acc = clientScene.player[this.index]
							.getKickAccelerate(clientScene.ball.pos, Float
									.parseFloat(arg1), Float.parseFloat(arg2));
				} catch (NumberFormatException ne) {
					return;
				}
			}

		}
		// <AgentAction-Command>:="(catch" <Number> ")"
		else if (context.currentToken().equals("(catch")) {
			// ޥɰν
			String arg1 = context.nextToken();
			arg1 = arg1.replace(')', ' ');
			arg1 = arg1.trim();
			try {
				clientScene.player[this.index].catchDir = Float
						.parseFloat(arg1);
			} catch (NumberFormatException ne) {
				return;
			}
		}

		// <AgentAction-Command>:="(say" <Comment>")"
		else if (context.currentToken().equals("(say")) {
			clientScene.player[this.index].sayMessage = context.nextToken("\0");

		}
		// <AgentAction-Command>:="(turn" <Number>")"
		else if (context.currentToken().equals("(turn")) {

		}
		// <AgentAction-Command>:="(turn_neck" <Number>")"
		else if (context.currentToken().equals("(turn_neck")) {

		}
		// <Comment>
		else {
			this.comment = context.currentComment();
			this.commentPos = null;
			// commentPos = clientScene.player[index].pos;
			this.type = COMMENT;
		}
	}

	public void execute(Graphics g, Layer layer) {
		g.setColor(this.color);

		if (this.type == LINE) {
			layer.drawLine(g, this.start, this.end);
		} else if (this.type == ARROW) {
			layer.drawArrow(g, this.start, this.end);
		} else if (this.type == RECT) {
			if (this.rotate == 0)
				layer.drawRect(g, this.topleft, this.size);
			else
				layer.drawRect(g, this.topleft, this.size, this.rotate);
		} else if (this.type == FILLRECT) {
			if (this.rotate == 0)
				layer.fillRect(g, this.topleft, this.size);
			else
				layer.fillRect(g, this.topleft, this.size, this.rotate);
		} else if (this.type == ELLIPSE) {
			if (this.rotate == 0)
				layer.drawOval(g, this.topleft, this.size);
			else
				layer.drawOval(g, this.topleft, this.size, this.rotate);
		} else if (this.type == FILLELLIPSE) {
			layer.fillOval(g, this.topleft, this.size);
		} else if (this.type == POLYLINE) {
			Iterator it = this.points.iterator();
			Point2f start = new Point2f((Point2f) it.next());
			while (it.hasNext()) {
				Point2f end = (Point2f) it.next();
				layer.drawLine(g, start, end);
				start.set(end);
			}
		} else if (this.type == STATE) {
			switch (this.state) {
			case 1:
				layer.drawImage(g, stateImage1, this.statePos, -stateImage1
						.getWidth(null) / 2, -stateImage1.getHeight(null) / 2);
				break;
			case 2:
				layer.drawImage(g, stateImage2, this.statePos, -stateImage2
						.getWidth(null) / 2, -stateImage2.getHeight(null) / 2);
				break;
			case 3:
				layer.drawImage(g, stateImage3, this.statePos, -stateImage3
						.getWidth(null), -stateImage3.getHeight(null));
				break;
			default:
			}

		} else if (this.type == TARGET) {
			layer.drawImage(g, targetImage, this.targetPos, this.offsetx,
					this.offsety);
		}
		if (this.comment != null && this.commentPos != null)
			layer.drawString(g, this.comment, this.commentPos);
	}
}
