/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope.model.agentplan;

import java.awt.Color;
import java.util.StringTokenizer;

import soccerscope.util.geom.Point2f;

public class Context {

	private StringTokenizer tokenizer;

	private String currentToken;

	public Context(String text) {
		this.tokenizer = new StringTokenizer(text);
		nextToken();
	}

	public String nextToken() {
		if (this.tokenizer.hasMoreTokens()) {
			this.currentToken = this.tokenizer.nextToken();
		} else {
			this.currentToken = null;
		}
		return this.currentToken;
	}

	public String nextToken(String delim) {
		if (this.tokenizer.hasMoreTokens()) {
			this.currentToken = this.tokenizer.nextToken(delim);
		} else {
			this.currentToken = null;
		}
		return this.currentToken;
	}

	public String currentToken() {
		return this.currentToken;
	}

	public void skipToken(String token) throws ParseException {
		if (!token.equals(this.currentToken)) {
			throw new ParseException("Warning: " + token + " is expected, but "
					+ this.currentToken + " is found.");
		}
		nextToken();
	}

	public int currentNumber() throws ParseException {
		int number = 0;
		try {
			number = Integer.parseInt(this.currentToken);
		} catch (NumberFormatException e) {
			throw new ParseException("Warning: " + e);
		}
		return number;
	}

	public float currentFloat() throws ParseException {
		float number = 0;
		try {
			number = Float.parseFloat(this.currentToken);
		} catch (NumberFormatException e) {
			throw new ParseException("Warning: " + e);
		}
		return number;
	}

	public Point2f currentPoint() throws ParseException {
		float x, y;
		String point = this.currentToken.replace('(', ' ');
		point = point.replace(')', ' ');
		point = point.replace(',', ' ');
		point = point.trim();
		try {
			x = Float.parseFloat(point.substring(0, point.lastIndexOf(' ')));
			y = Float.parseFloat(point.substring(point.lastIndexOf(' ') + 1));
		} catch (NumberFormatException e) {
			throw new ParseException("Warning: " + e);
		}
		return new Point2f(x, y);
	}

	public boolean isPointToken() {
		if (this.currentToken != null
				&& this.currentToken.charAt(0) == '('
				&& this.currentToken.charAt(this.currentToken.length() - 1) == ')')
			return true;
		return false;
	}

	public boolean isColorToken() {
		if (currentToken() != null && this.currentToken.charAt(0) == '#')
			return true;
		return false;
	}

	public Color currentColor() throws ParseException {
		int color;
		try {
			color = Integer.parseInt(this.currentToken.substring(1), 16);
			nextToken();
		} catch (NumberFormatException e) {
			throw new ParseException("Warning: " + e);
		}
		return new Color(color);
	}

	public String currentComment() {
		String comment = currentToken();
		if (comment != null && nextToken("\0") != null)
			comment = comment.concat(currentToken());
		return comment;
	}
}
