/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope.util.analyze;

import soccerscope.model.ClientScene;
import soccerscope.model.GameEvent;
import soccerscope.model.Param;
import soccerscope.model.PlayMode;
import soccerscope.util.geom.Line2f;
import soccerscope.util.geom.Point2f;

public class GoalAnalyzer extends SceneAnalyzer {

	public static String NAME = "Goal";

	@Override
	public String getName() {
		return NAME;
	}

	@Override
	public GameEvent analyze(ClientScene clientScene, ClientScene prev) {
		GameEvent ge = null;
		/* ⡼ɤϻ郎ߤޤ뤿ᡢprev򸫤ơ⡼ɤˤʤäִ֤򡢼 */
		if (prev != null) {
			if (isPlayModeChanged(clientScene, prev, PlayMode.PM_AfterGoal_Left)) {
				countUpLeft(clientScene.time);
				ge = new GameEvent(clientScene.time, GameEvent.GOAL);
			}
			if (isPlayModeChanged(clientScene, prev,
					PlayMode.PM_AfterGoal_Right)) {
				countUpRight(clientScene.time);
				ge = new GameEvent(clientScene.time, GameEvent.GOAL);
			}
		}
		if (clientScene.pmode.pmode == PlayMode.PM_TimeOver) {
			Line2f btraj = new Line2f(prev.ball.pos, clientScene.ball.pos);
			Line2f rightGoalLine = new Line2f(Param.topRightCorner,
					Param.bottomRightCorner);
			Line2f leftGoalLine = new Line2f(Param.topLeftCorner,
					Param.bottomLeftCorner);
			if (btraj.intersect(rightGoalLine)) {
				Point2f p = btraj.intersection(rightGoalLine);
				if (Math.abs(p.y) <= Param.GOAL_WIDTH / 2
						&& ((prev.ball.pos.x <= p.x && p.x <= clientScene.ball.pos.x) || (clientScene.ball.pos.x <= p.x && p.x <= prev.ball.pos.x))) {
					countUpLeft(clientScene.time);
					ge = new GameEvent(clientScene.time, GameEvent.GOAL);
				}
			}
			if (btraj.intersect(leftGoalLine)) {
				Point2f p = btraj.intersection(leftGoalLine);
				if (Math.abs(p.y) <= Param.GOAL_WIDTH / 2
						&& ((prev.ball.pos.x <= p.x && p.x <= clientScene.ball.pos.x) || (clientScene.ball.pos.x <= p.x && p.x <= prev.ball.pos.x))) {
					countUpRight(clientScene.time);
					ge = new GameEvent(clientScene.time, GameEvent.GOAL);
				}
			}
		}

		return ge;
	}
}
