/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope.util.analyze;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;

import soccerscope.model.ClientScene;
import soccerscope.model.GameEvent;
import soccerscope.model.Param;
import soccerscope.model.PlayMode;
import soccerscope.model.Team;
import soccerscope.model.WorldModel;

public class PassAnalyzer extends SceneAnalyzer {

	public static String NAME = "Complete Pass";

	private static int pTable[] = new int[0];

	private static ArrayList<Pass> passList = new ArrayList<Pass>();

	private static ArrayList<Pass> passLeftList = new ArrayList<Pass>();

	private static ArrayList<Pass> passRightList = new ArrayList<Pass>();

	private Kicker leftKicker;

	private Kicker rightKicker;

	public final static int PLAY_OFF = 0;

	public final static int PLAY_ON = 1;

	public final static int LEFT_SIDE = 2;

	public final static int RIGHT_SIDE = 3;

	@Override
	public void init() {
		super.init();
		ClientScene clientScene = WorldModel.getInstance().getSceneSet()
				.lastScene();
		pTable = new int[clientScene.time + 1];
		passList.clear();
		passLeftList.clear();
		passRightList.clear();
		this.leftKicker = null;
		this.rightKicker = null;
	}

	@Override
	public String getName() {
		return NAME;
	}

	public class Kicker {
		public int time;

		public int unum;

		public Kicker() {
			this.time = 0;
			this.unum = 0;
		}

		public Kicker(int time, int unum) {
			this.time = time;
			this.unum = unum;
		}

		public void set(Kicker k) {
			this.time = k.time;
			this.unum = k.unum;
		}

		@Override
		public String toString() {
			return "kicker ::" + this.time + ": " + this.unum;
		}
	}

	public class Pass {
		public int side;

		public Kicker sender;

		public Kicker receiver;

		public Pass(int side, Kicker s, Kicker r) {
			this.side = side;
			this.sender = s;
			this.receiver = r;
		}

		@Override
		public String toString() {
			return this.side + " :: " + +this.sender.time + ": "
					+ this.sender.unum + " -> " + this.receiver.time + ": "
					+ this.receiver.unum;
		}
	}

	@Override
	public GameEvent analyze(ClientScene clientScene, ClientScene prev) {

		if (prev == null)
			return null;

		if (clientScene.time < pTable.length)
			pTable[clientScene.time] = PLAY_OFF;

		// ץ쥤ʳ, 
		if (clientScene.pmode.pmode != PlayMode.PM_PlayOn) {
			this.leftKicker = null;
			this.rightKicker = null;
			return null;
		}

		pTable[clientScene.time] = PLAY_ON;

		Kicker left = null;
		Kicker right = null;

		for (int i = 0; i < Param.MAX_PLAYER; i++) {
			if ((clientScene.player[i].isKicking() && prev.player[i]
					.isKickable(prev.ball.pos))
					|| (clientScene.player[i].isCatching() && prev.player[i]
							.isCatchable(prev.ball.pos))
					|| (clientScene.player[i].isTackling() && prev.player[i]
							.canTackle(prev.ball.pos))) {
				left = new Kicker(clientScene.time, clientScene.player[i].unum);
			}
		}
		for (int i = Param.MAX_PLAYER; i < Param.MAX_PLAYER * 2; i++) {
			if ((clientScene.player[i].isKicking() && prev.player[i]
					.isKickable(prev.ball.pos))
					|| (clientScene.player[i].isCatching() && prev.player[i]
							.isCatchable(prev.ball.pos))
					|| (clientScene.player[i].isTackling() && prev.player[i]
							.canTackle(prev.ball.pos))) {
				right = new Kicker(clientScene.time, clientScene.player[i].unum);
			}
		}

		GameEvent ge = null;
		if (left != null && right != null) {
			this.leftKicker = null;
			this.rightKicker = null;
			left = null;
			right = null;
		}
		if (left != null) {
			if (this.leftKicker != null && this.leftKicker.unum != left.unum) {
				countUpLeft(clientScene.time);
				passList.add(new Pass(Team.LEFT_SIDE, this.leftKicker, left));
				passLeftList
						.add(new Pass(Team.LEFT_SIDE, this.leftKicker, left));
				ge = new GameEvent(this.leftKicker.time, GameEvent.PASS);
			}
			if (this.leftKicker != null) {
				Arrays.fill(pTable, this.leftKicker.time, left.time + 1,
						LEFT_SIDE);
			}
			this.leftKicker = left;
			if (right == null)
				this.rightKicker = null;
		}
		if (right != null) {
			if (this.rightKicker != null && this.rightKicker.unum != right.unum) {
				countUpRight(clientScene.time);
				passList
						.add(new Pass(Team.RIGHT_SIDE, this.rightKicker, right));
				passRightList.add(new Pass(Team.RIGHT_SIDE, this.rightKicker,
						right));
				ge = new GameEvent(this.rightKicker.time, GameEvent.PASS);
			}
			if (this.rightKicker != null) {
				Arrays.fill(pTable, this.rightKicker.time, right.time + 1,
						RIGHT_SIDE);
			}
			this.rightKicker = right;
			if (left == null)
				this.leftKicker = null;
		}

		return ge;
	}

	public static int[] getPossessionTable() {
		return pTable;
	}

	public static int getPossessionTeam(int time) {
		if (time < pTable.length)
			return pTable[time];
		else
			return PLAY_OFF;
	}

	public static ArrayList getPassList() {
		return passList;
	}

	public static ArrayList getPassLeftList() {
		return passLeftList;
	}

	public static ArrayList getPassRightList() {
		return passRightList;
	}

	public static Pass getPass(int time) {
		Iterator it = passList.iterator();

		while (it.hasNext()) {
			Pass pass = (Pass) it.next();
			if (pass.sender.time <= time && time <= pass.receiver.time) {
				return pass;
			}
		}
		return null;
	}

	public static Pass getLeftPass(int time) {
		Iterator it = passLeftList.iterator();

		while (it.hasNext()) {
			Pass pass = (Pass) it.next();
			if (pass.sender.time <= time && time <= pass.receiver.time) {
				return pass;
			}
		}
		return null;
	}

	public static Pass getRightPass(int time) {
		Iterator it = passRightList.iterator();

		while (it.hasNext()) {
			Pass pass = (Pass) it.next();
			if (pass.sender.time <= time && time <= pass.receiver.time) {
				return pass;
			}
		}
		return null;
	}
}
