/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope.util.analyze;

import soccerscope.model.ClientScene;
import soccerscope.model.GameEvent;
import soccerscope.model.Param;
import soccerscope.model.PlayMode;
import soccerscope.util.geom.Point2f;
import soccerscope.util.geom.Vector2f;

public class PassMissAnalyzer extends SceneAnalyzer {

	public static String NAME = "Pass Miss";

	private Kicker leftKicker;

	private Kicker rightKicker;

	@Override
	public void init() {
		super.init();
		this.leftKicker = null;
		this.rightKicker = null;
	}

	@Override
	public String getName() {
		return NAME;
	}

	public class Kicker {
		public int time;

		public int unum;

		public Kicker() {
			this.time = 0;
			this.unum = 0;
		}

		public Kicker(int time, int unum) {
			this.time = time;
			this.unum = unum;
		}

		public void set(Kicker k) {
			this.time = k.time;
			this.unum = k.unum;
		}

		@Override
		public String toString() {
			return "kicker ::" + this.time + ": " + this.unum;
		}
	}

	@Override
	public GameEvent analyze(ClientScene clientScene, ClientScene prev) {

		if (prev == null)
			return null;

		Kicker left = null;
		Kicker right = null;

		// ץ쥤ʳ, 
		if (isPlayModeChanged(clientScene, prev, PlayMode.PM_FreeKick_Left)
				|| isPlayModeChanged(clientScene, prev, PlayMode.PM_KickIn_Left)
				|| isPlayModeChanged(clientScene, prev,
						PlayMode.PM_GoalKick_Left)
				|| isPlayModeChanged(clientScene, prev,
						PlayMode.PM_CornerKick_Left)
				|| isPlayModeChanged(clientScene, prev,
						PlayMode.PM_OffSide_Right)
				|| isPlayModeChanged(clientScene, prev,
						PlayMode.PM_Back_Pass_Right)
				|| isPlayModeChanged(clientScene, prev,
						PlayMode.PM_Free_Kick_Fault_Right)) {
			left = new Kicker(clientScene.time, 0);
		} else if (isPlayModeChanged(clientScene, prev,
				PlayMode.PM_FreeKick_Right)
				|| isPlayModeChanged(clientScene, prev,
						PlayMode.PM_KickIn_Right)
				|| isPlayModeChanged(clientScene, prev,
						PlayMode.PM_GoalKick_Right)
				|| isPlayModeChanged(clientScene, prev,
						PlayMode.PM_CornerKick_Right)
				|| isPlayModeChanged(clientScene, prev,
						PlayMode.PM_OffSide_Left)
				|| isPlayModeChanged(clientScene, prev,
						PlayMode.PM_Back_Pass_Left)
				|| isPlayModeChanged(clientScene, prev,
						PlayMode.PM_Free_Kick_Fault_Left)) {
			right = new Kicker(clientScene.time, 0);
		} else if (clientScene.pmode.pmode != PlayMode.PM_PlayOn) {
			this.leftKicker = null;
			this.rightKicker = null;
			return null;
		}

		boolean leftKickable = false;
		boolean rightKickable = false;
		for (int i = 0; i < Param.MAX_PLAYER; i++) {
			if (clientScene.player[i].isKickable(clientScene.ball.pos))
				leftKickable = true;
		}
		for (int i = Param.MAX_PLAYER; i < Param.MAX_PLAYER * 2; i++) {
			if (clientScene.player[i].isKickable(clientScene.ball.pos))
				rightKickable = true;
		}
		for (int i = 0; i < Param.MAX_PLAYER; i++) {
			if (!rightKickable
					&& (clientScene.player[i].isKicking() && prev.player[i]
							.isKickable(prev.ball.pos))
					|| (clientScene.player[i].isCatching() && prev.player[i]
							.isCatchable(prev.ball.pos))
					|| (clientScene.player[i].isTackling() && prev.player[i]
							.canTackle(prev.ball.pos))) {
				left = new Kicker(clientScene.time, clientScene.player[i].unum);
			}
		}
		for (int i = Param.MAX_PLAYER; i < Param.MAX_PLAYER * 2; i++) {
			if (!leftKickable
					&& (clientScene.player[i].isKicking() && prev.player[i]
							.isKickable(prev.ball.pos))
					|| (clientScene.player[i].isCatching() && prev.player[i]
							.isCatchable(prev.ball.pos))
					|| (clientScene.player[i].isTackling() && prev.player[i]
							.canTackle(prev.ball.pos))) {
				right = new Kicker(clientScene.time, clientScene.player[i].unum);
			}
		}

		if (left != null && right != null) {
			// leftKicker = null;
			// rightKicker = null;
			left = null;
			right = null;
		}
		if (left != null) {
			if (this.rightKicker != null) {
				this.rightKicker = null;
				countUpRight(clientScene.time);
			}
			if (isPassKick(clientScene, LEFT, left.unum)) {
				this.leftKicker = left;
			} else {
				this.leftKicker = null;
			}
		}
		if (right != null) {
			if (this.leftKicker != null) {
				this.leftKicker = null;
				countUpLeft(clientScene.time);
			}
			if (isPassKick(clientScene, RIGHT, right.unum)) {
				this.rightKicker = right;
			} else {
				this.rightKicker = null;
			}
		}

		return null;
	}

	public boolean isPassKick(ClientScene clientScene, int team, int unum) {
		if (unum == 0)
			return false;
		int start = 0;
		if (team == RIGHT)
			start = Param.MAX_PLAYER;

		Point2f tmpp = new Point2f(clientScene.ball.pos);
		Vector2f tmpv = new Vector2f(clientScene.ball.vel);
		for (int n = 0; n < 100; n++) {
			for (int i = 0; i < Param.MAX_PLAYER; i++) {
				float dist = clientScene.player[i + start].pos.dist(tmpp)
						- Param.KICKABLE_R;
				if (clientScene.player[i + start].estimateMinTimeByDistance(
						dist, 0.6f) < n) {
					if (unum == clientScene.player[i + start].unum)
						return false;
					else
						return true;
				}
			}
			tmpp.add(tmpv);
			tmpv.scale(0.94f);
			if (!inField(tmpp))
				return false;
		}

		return false;
	}

	public boolean inField(Point2f pos) {
		return -52.5f <= pos.x && pos.x <= 52.5f && -34.0f <= pos.y
				&& pos.y <= 34.0;
	}
}
