/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope.util.analyze;

import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;

import soccerscope.model.ClientScene;
import soccerscope.model.GameEvent;
import soccerscope.model.Player;
import soccerscope.model.WorldModel;

public abstract class SceneAnalyzer {

	public final static int ROW_NAME = 0;

	public final static int LEFT = 1;

	public final static int RIGHT = 2;

	public final static int COL_MAX = 3;

	protected int lcount;

	protected int rcount;

	protected int lastTime;

	private ArrayList<Integer> lcountList = new ArrayList<Integer>();

	private ArrayList<Integer> rcountList = new ArrayList<Integer>();

	public abstract GameEvent analyze(ClientScene clientScene, ClientScene prev);

	public abstract String getName();

	public void init() {
		this.lcountList = new ArrayList<Integer>();
		this.rcountList = new ArrayList<Integer>();
		this.lcount = 0;
		this.rcount = 0;
		this.lastTime = WorldModel.getInstance().getSceneSet().lastScene().time;
	}

	public void countUpLeft(int time) {
		this.lcountList.add(new Integer(time));
		this.lcount++;
	}

	public void countUpRight(int time) {
		this.rcountList.add(new Integer(time));
		this.rcount++;
	}

	public Object getValueAt(int col) {
		return getValueAt(col, 0, this.lastTime);
	}

	public Object getLeftValueAt(int fromTime, int toTime) {
		return getValueAt(LEFT, fromTime, toTime);
	}

	public Object getRightValueAt(int fromTime, int toTime) {
		return getValueAt(RIGHT, fromTime, toTime);
	}

	public Object getValueAt(int col, int fromTime, int toTime) {
		count(fromTime, toTime);
		switch (col) {
		case ROW_NAME:
			return getName();
		case LEFT:
			if (this.lcount == -1)
				return "--";
			return new Integer(this.lcount);
		case RIGHT:
			if (this.rcount == -1)
				return "--";
			return new Integer(this.rcount);
		default:
			return " ";
		}
	}

	public void count(int fromTime, int toTime) {
		this.lcount = 0;
		Iterator it = this.lcountList.iterator();
		while (it.hasNext()) {
			int time = ((Integer) it.next()).intValue();
			if (fromTime <= time && time <= toTime) {
				this.lcount++;
			}
		}
		this.rcount = 0;
		it = this.rcountList.iterator();
		while (it.hasNext()) {
			int time = ((Integer) it.next()).intValue();
			if (fromTime <= time && time <= toTime) {
				this.rcount++;
			}
		}
	}

	public TableModel getTableModel() {
		count(0, this.lastTime);
		TableModel dataModel = new AbstractTableModel() {
			private static final long serialVersionUID = 3379121955229725633L;

			@Override
			public String getColumnName(int col) {
				switch (col) {
				case ROW_NAME:
					return " ";
				case LEFT:
					return "Left";
				case RIGHT:
					return "Right";
				default:
					return " ";
				}
			}

			public int getColumnCount() {
				return COL_MAX;
			}

			public int getRowCount() {
				return 1;
			}

			public Object getValueAt(int row, int col) {
				switch (col) {
				case ROW_NAME:
					return getName();
				case LEFT:
					if (SceneAnalyzer.this.lcount == -1)
						return "--";
					return new Integer(SceneAnalyzer.this.lcount);
				case RIGHT:
					if (SceneAnalyzer.this.rcount == -1)
						return "--";
					return new Integer(SceneAnalyzer.this.rcount);
				default:
					return " ";
				}
			}
		};

		return dataModel;
	}

	public boolean isPlayModeChanged(ClientScene clientScene, ClientScene prev,
			int pmode) {
		return clientScene.pmode.pmode == pmode && prev.pmode.pmode != pmode;
	}

	public boolean doKicked(Player player, Player prev) {
		return player.kickCount > prev.kickCount;
	}

	public boolean doDashed(Player player, Player prev) {
		return player.dashCount > prev.dashCount;
	}

	public boolean doTurned(Player player, Player prev) {
		return player.turnCount > prev.turnCount;
	}

	// v0vãޤǤ˰ưΥ
	protected float getFinalDist(float v0, float d, float v) {
		return getTotalDist(v0, d, getTimeByVel(v0, d, v));
	}

	// v0tΰưΥ
	protected float getTotalDist(float v0, float d, int t) {
		if (t == 0)
			return 0;
		float b, e;
		b = 1 - d;
		e = 1 - (float) Math.pow(d, t);
		return v0 * (e / b);
	}

	// v0vãޤǤˤ
	protected int getTimeByVel(float v0, float d, float v) {
		return (int) Math.ceil((Math.log(v / (v0 / d))) / (Math.log(d)) - 1);
	}

	// [-180,180] -> [0,360]
	protected float normalize(float angle) {
		while (angle < 0)
			angle += 2 * Math.toDegrees(Math.PI);
		return angle;
	}
}
