/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope.util.geom;

public class Circle2f {

	public final static float EPS = 1.0e-3f;

	/* (x - a)^2 + (y - b)^2 = r^2 */
	private float a;

	/**
	 * 
	 */
	private float b;

	/**
	 * 
	 */
	private float r;

	// public static void main(String args[]) {
	// Point2f o = new Point2f();
	// Point2f a = new Point2f(1,1);
	// Point2f b = new Point2f(1,0);
	// Point2f c = new Point2f(0,1);
	// Point2f d = new Point2f(-1,1);
	// Line2f l1 = new Line2f(o, a);
	// Line2f l2 = new Line2f(o, d);
	// Line2f l3 = new Line2f(o, b);
	// Line2f l4 = new Line2f(o, c);
	// Circle2f c1 = new Circle2f(o, 1);
	// Circle2f c2 = new Circle2f(new Point2f(0, 2), 1);
	// Point2f p[] = c2.intersection(l4);
	// if (p.length >= 1)
	// System.out.println(p[0]);
	// if (p.length >= 2)
	// System.out.println(p[1]);
	// Line2f l[] = c1.tangentLine(a);
	// if (l.length >= 1)
	// System.out.println(l[0]);
	// if (l.length >= 2)
	// System.out.println(l[1]);
	// Circle2f c3 = new Circle2f(a, 1);
	// Line2f l5 = new Line2f(new Point2f(1, -1), 90.1f);
	// System.out.println(l5);
	// Point2f p2[] = c3.intersection(l5);
	// if (p2.length == 0)
	// System.out.println(l5.projectpoint(a));
	// if (p2.length >= 1)
	// System.out.println(p2[0]);
	// if (p2.length >= 2)
	// System.out.println(p2[1]);
	// }

	/**
	 * 
	 */
	public Circle2f() {
		this.a = 0;
		this.b = 0;
		this.r = 0;
	}

	/**
	 * 
	 * 
	 * @param c
	 * @param radius
	 */
	public Circle2f(Point2f c, float radius) {
		this.a = c.x;
		this.b = c.y;
		this.r = radius;
	}

	/**
	 * 
	 * 
	 * @return
	 */
	public Point2f getCenterPoint() {
		return new Point2f(this.a, this.b);
	}

	/**
	 * 
	 */
	public float getRadius() {
		return this.r;
	}

	/**
	 * 
	 * 
	 * @param c
	 */
	public void setCenterPoint(Point2f c) {
		this.a = c.x;
		this.b = c.y;
	}

	/**
	 * 
	 * 
	 * @param r
	 */
	public void setRadius(float r) {
		this.r = r;
	}

	/**
	 * 
	 * 
	 * @param p
	 * @return
	 */
	public boolean online(Point2f p) {
		float pxa = (p.x - this.a);
		float pyb = (p.y - this.b);
		return (Math.abs(pxa * pxa + pyb * pyb - this.r * this.r) <= EPS);
	}

	/**
	 * 
	 * 
	 * @param p
	 * @return
	 */
	public boolean contains(Point2f p) {
		float pxa = (p.x - this.a);
		float pyb = (p.y - this.b);
		return (pxa * pxa + pyb * pyb - this.r * this.r < 0);
	}

	/**
	 * 
	 * 
	 * @param p
	 * @return
	 */
	public Line2f[] tangentLine(Point2f p) {
		Line2f l[];
		// ξ硢Ϥʤ
		if (contains(p))
			return l = new Line2f[0];
		// ߼ξ硢1
		if (online(p)) {
			l = new Line2f[1];
			l[0] = new Line2f();
			l[0].A = p.x - this.a;
			l[0].B = p.y - this.b;
			l[0].C = -l[0].A * this.a - l[0].B * this.b - this.r * this.r;
			return l;
		}
		// ߳ξ硢2
		float A, B, C, D, E, F;
		A = p.x - this.a;
		B = p.y - this.b;
		C = -p.x * this.a - p.y * this.b + this.a * this.a + this.b * this.b
				- this.r * this.r;
		D = -A / B;
		E = -C / B;
		F = E - this.b;

		Point2f p1 = new Point2f();
		Point2f p2 = new Point2f();
		if (B == 0 || Math.abs(B) <= EPS) {
			p1.x = -C / A;
			p1.y = this.b
					+ (float) Math.sqrt(this.r * this.r - (p1.x - this.a)
							* (p1.x - this.a));
			p2.x = -C / A;
			p2.y = this.b
					- (float) Math.sqrt(this.r * this.r - (p2.x - this.a)
							* (p2.x - this.a));
		} else {
			double d = (D * F - this.a) * (D * F - this.a) - (1 + D * D)
					* (F * F + this.a * this.a - this.r * this.r);
			if (d < 0)
				return l = new Line2f[0];
			p1.x = (this.a - D * F + (float) Math.sqrt(d)) / (1 + D * D);
			p1.y = D * p1.x + E;
			p2.x = (this.a - D * F - (float) Math.sqrt(d)) / (1 + D * D);
			p2.y = D * p2.x + E;
		}

		l = new Line2f[2];
		l[0] = new Line2f();
		l[1] = new Line2f();
		l[0].A = p1.x - this.a;
		l[0].B = p1.y - this.b;
		l[0].C = -l[0].A * this.a - l[0].B * this.b - this.r * this.r;
		l[1].A = p2.x - this.a;
		l[1].B = p2.y - this.b;
		l[1].C = -l[0].A * this.a - l[0].B * this.b - this.r * this.r;
		return l;
	}

	/**
	 * 
	 * 
	 * @param p
	 * @return
	 */
	public Line2f normalLine(Point2f p) {
		return new Line2f(new Point2f(this.a, this.b), p);
	}

	/**
	 * 
	 * 
	 * @param l
	 * @return
	 */
	public boolean intersect(Line2f l) {
		return l.dist(getCenterPoint()) <= this.r;
	}

	/**
	 * 
	 * 
	 * @param l
	 * @return
	 */
	public Point2f[] intersection(Line2f l) {
		Point2f p[];
		if (l.dist(getCenterPoint()) > this.r)
			return p = new Point2f[0];

		float D, E, F;
		D = -l.A / l.B;
		E = -l.C / l.B;
		F = E - this.b;

		if (l.B == 0 || Math.abs(l.B) <= EPS) {
			p = new Point2f[2];
			p[0] = new Point2f();
			p[1] = new Point2f();
			p[0].x = -l.C / l.A;
			p[0].y = this.b
					+ (float) Math.sqrt(this.r * this.r - (p[0].x - this.a)
							* (p[0].x - this.a));
			p[1].x = -l.C / l.A;
			p[1].y = this.b
					- (float) Math.sqrt(this.r * this.r - (p[1].x - this.a)
							* (p[1].x - this.a));
			return p;
		}
		float d = (D * F - this.a) * (D * F - this.a) - (1 + D * D)
				* (F * F + this.a * this.a - this.r * this.r);
		if (Math.abs(d) < EPS) {
			p = new Point2f[1];
			p[0] = new Point2f();
			p[0].x = (this.a - D * F) / (1 + D * D);
			p[0].y = D * p[0].x + E;
			return p;
		}
		if (d < 0)
			return new Point2f[0];
		p = new Point2f[2];
		p[0] = new Point2f();
		p[1] = new Point2f();
		p[0].x = (this.a - D * F + (float) Math.sqrt(d)) / (1 + D * D);
		p[0].y = D * p[0].x + E;
		p[1].x = (this.a - D * F - (float) Math.sqrt(d)) / (1 + D * D);
		p[1].y = D * p[1].x + E;
		return p;
	}

	/**
	 * 
	 * 
	 * @param l
	 * 
	 * @return
	 */
	public boolean tangentTo(Line2f l) {
		float tmp = (l.A * this.a + l.B * this.b + l.C);
		return Math.abs(tmp * tmp - this.r * this.r * (l.A * l.A + l.B * l.B)) <= EPS;
	}

	/**
	 * 
	 * 
	 * @return
	 */
	@Override
	public String toString() {
		return "(x-" + this.a + ")^2 + (y-" + this.b + ")^2 = " + this.r + "^2";
	}

}
