/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope.util.geom;

import java.awt.geom.Point2D;
import java.io.Serializable;

/**
 * 
 */
public class Point2f extends Tuple2f implements Serializable {

	private static final long serialVersionUID = 1L;

	/**
	 * x=0, y=0
	 */
	public Point2f() {
		super();
	}

	/**
	 * x,y
	 * 
	 * @param x
	 *            x
	 * @param y
	 *            y
	 */
	public Point2f(float x, float y) {
		super(x, y);
	}

	/**
	 * ꤷPoint2f֥pƱͤPoint2f.
	 * 
	 * @param p
	 *            Point2f֥
	 */
	public Point2f(Point2f p) {
		super(p);
	}

	/**
	 * ꤷTuple2f֥tƱͤPoint2f.
	 * 
	 * @param t
	 *            Tuple2f֥
	 */
	public Point2f(Tuple2f t) {
		super(t);
	}

	/**
	 * pȤδ֤εΥ֤.
	 * 
	 * @param p
	 *            Point2f֥
	 * @return ֤εΥ
	 */
	public float distance(Point2f p) {
		return dist(p);
	}

	/**
	 * pȤδ֤εΥ֤.
	 * 
	 * @param p
	 *            Point2f֥
	 * @return ֤εΥ
	 */
	public float dist(Point2f p) {
		return (float) Math.sqrt(distanceSquared(p));
	}

	/**
	 * pȤδ֤εΥʿ֤.
	 * 
	 * @param p
	 *            Point2f֥
	 * @return ֤εΥʿ
	 */
	public float distanceSquared(Point2f p) {
		float dx = p.x - this.x;
		float dy = p.y - this.y;
		return dx * dx + dy * dy;
	}

	/**
	 * 
	 * @param p
	 *            Point2f֥
	 * @return ֤ΥޥϥåΥ
	 */
	public float distanceL1(Point2f p) {
		return (Math.abs(this.x - p.x) + Math.abs(this.y - p.y));
	}

	/**
	 * 
	 * @param p
	 *            Point2f֥
	 * @return ֤Υ׵Υ
	 */
	public float distanceLinf(Point2f p) {
		return Math.max(Math.abs(this.x - p.x), Math.abs(this.y - p.y));
	}

	/**
	 * 
	 * @param p
	 *            Point2f֥
	 * @return pؤ ()
	 */
	public float direction(Point2f p) {
		return dir(p);
	}

	/**
	 * 
	 * @param p
	 *            Point2f֥
	 * @return pؤ ()
	 */
	public float dir(Point2f p) {
		return (float) Math.toDegrees(Math.atan2(p.y - this.y, p.x - this.x));
	}

	/**
	 * 
	 * @param p1
	 *            Point2f֥
	 * @param p2
	 *            Point2f֥
	 * @return ֤γ ()
	 */
	public float angle(Point2f p1, Point2f p2) {
		return Geometry.normalizeAngle(dir(p2) - dir(p1));
	}

	/**
	 * 
	 * @param p
	 *            ž濴
	 * @param th
	 *            ž
	 * @return org濴thٲž
	 */
	public Point2f rotate(Point2f org, float th) {
		Vector2f tmp = new Vector2f(org, this);
		tmp = tmp.rotate(th);

		return new Point2f(tmp.x + org.x, tmp.y + org.y);
	}

	/**
	 * 
	 * @param p
	 *            Point2f֥
	 * @return pˤtrue
	 */
	public boolean inFrontOf(Point2f p) {
		return this.x > p.x;
	}

	/**
	 * 
	 * @param p
	 *            Point2f֥
	 * @return pˤtrue
	 */
	public boolean isBehind(Point2f p) {
		return this.x < p.x;
	}

	/**
	 * ˺ɸʸ֤
	 * 
	 * @return ˺ɸʸ
	 */
	public String toPolar() {
		Point2f p = new Point2f();
		return "(" + p.dist(this) + ", " + p.dir(this) + ")";
	}

	/**
	 * 
	 * @param r
	 *            ˺ɸȾʬ
	 * @param th
	 *            ˺ɸγʬ
	 * @return Point2f֥
	 */
	public static Point2f polar2point(float r, float th) {
		double d = Math.toRadians(th);
		return new Point2f((float) (r * Math.cos(d)), (float) (r * Math.sin(d)));
	}

	/**
	 */
	public Point2D.Float toPoint2DF() {
		return new Point2D.Float(this.x, this.y);
	}

	/**
	 * Point2fVector2f֤
	 */
	public Vector2f toVector2f() {
		return new Vector2f(this.x, this.y);
	}
}
