/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope.view;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTabbedPane;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import soccerscope.model.ClientScene;
import soccerscope.util.geom.Point2f;
import soccerscope.view.layer.BallLayer;
import soccerscope.view.layer.Layer;
import soccerscope.view.layer.PlayerLayer;

public class OptionWindow extends JFrame implements ScopeWindow,
		ActionListener, MouseListener, MouseMotionListener, ChangeListener {

	private static final long serialVersionUID = 3803181894480660011L;

	public final static String title = "option";

	private ScopePane scopePane;

	private FieldPane fieldPane;

	private ClientScene clientScene;

	// private Container container = null;

	private JTabbedPane tabbedPane;

	private JButton zoomin;

	private JButton zoomout;

	private JSlider magnifySlider;

	private JLabel magnifyLabel;

	private JSlider playerSlider;

	private JLabel playerLabel;

	private JSlider ballSlider;

	private JLabel ballLabel;

	public OptionWindow(ScopePane scopePane) {
		// frame title
		super("option");

		// 
		this.scopePane = scopePane;
		FieldPane fp = scopePane.getFieldPane();
		this.tabbedPane = new JTabbedPane(SwingConstants.TOP);
		getContentPane().add(this.tabbedPane);

		// ߡʥ
		this.clientScene = ClientScene.createScene();

		/* Fieldѥͥ */
		this.fieldPane = new FieldPane(this.clientScene);
		this.fieldPane.addForegroundLayer(new PlayerLayer(this.fieldPane));
		this.fieldPane.addForegroundLayer(new BallLayer(this.fieldPane));
		this.fieldPane.setMagnify(1.5f, 10.0f, 6.0f);
		this.fieldPane.addMouseListener(this);
		this.fieldPane.addMouseMotionListener(this);
		JPanel jp1 = new JPanel(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridwidth = 2;
		jp1.add(this.fieldPane, gbc);
		this.zoomin = new JButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/general/ZoomIn24.gif")));
		this.zoomin.addActionListener(this);
		this.zoomout = new JButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/general/ZoomOut24.gif")));
		this.zoomout.addActionListener(this);
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.gridy = 1;
		gbc.gridwidth = 1;
		gbc.weightx = 0.5;
		jp1.add(this.zoomin, gbc);
		gbc.gridx = 1;
		jp1.add(this.zoomout, gbc);
		this.tabbedPane.addTab("Field", jp1);

		/* Ψѥͥ */
		JPanel jp2 = new JPanel(new GridBagLayout());
		GridBagConstraints gbc2 = new GridBagConstraints();
		this.magnifyLabel = new JLabel("Field " + (int) fp.getMagnify());
		this.magnifySlider = new JSlider(SwingConstants.HORIZONTAL, 1, 500,
				(int) (fp.getMagnify() * 10));
		this.magnifySlider.addChangeListener(this);
		this.ballLabel = new JLabel("Ball " + (int) fp.getBallMagnify());
		this.ballSlider = new JSlider(SwingConstants.HORIZONTAL, 1, 200,
				(int) (fp.getBallMagnify() * 10));
		this.ballSlider.addChangeListener(this);
		this.playerLabel = new JLabel("Player " + (int) fp.getPlayerMagnify());
		this.playerSlider = new JSlider(SwingConstants.HORIZONTAL, 1, 200,
				(int) (fp.getPlayerMagnify() * 10));
		this.playerSlider.addChangeListener(this);
		jp2.add(this.magnifyLabel, gbc2);
		gbc2.gridy = 1;
		jp2.add(this.magnifySlider, gbc2);
		gbc2.gridy = 2;
		jp2.add(this.ballLabel, gbc2);
		gbc2.gridy = 3;
		jp2.add(this.ballSlider, gbc2);
		gbc2.gridy = 4;
		jp2.add(this.playerLabel, gbc2);
		gbc2.gridy = 5;
		jp2.add(this.playerSlider, gbc2);
		this.tabbedPane.addTab("Magnify", jp2);

		/* Layerѥͥ */
		ArrayList layers = new ArrayList();
		layers.addAll(scopePane.getShowLayer());
		layers.addAll(scopePane.getDrawLayer());
		Iterator it = layers.iterator();
		while (it.hasNext()) {
			Layer layer = (Layer) it.next();
			JPanel panel = layer.getConfigPanel();
			if (panel != null)
				this.tabbedPane.addTab(layer.getLayerName(), panel);
		}

		pack();
	}

	// ScopeWindow
	public void setScene(ClientScene clientScene) {
		this.clientScene = clientScene;
		this.fieldPane.setScene(clientScene);
		FieldPane fp = this.scopePane.getFieldPane();
		Point2f watchPoint = fp.getWatchPoint();
		Dimension drawSize = fp.getDrawableSize();
		Point2f topleft = this.fieldPane
				.fieldToScreen(new Point2f(
						watchPoint.x - drawSize.width / 2.0f, watchPoint.y
								- drawSize.height / 2.0f));

		Rectangle r = new Rectangle((int) topleft.x, (int) topleft.y,
				(int) (drawSize.width * this.fieldPane.getMagnify()),
				(int) (drawSize.height * this.fieldPane.getMagnify()));
		ArrayList list = new ArrayList();
		list.add(r);
		this.fieldPane.clearShapeList();
		this.fieldPane.setShapeList(list);
		repaint();
	}

	// ActionListener
	public void actionPerformed(ActionEvent ae) {
		Object o = ae.getSource();
		if (o == this.zoomin) {
			this.scopePane.setMagnify(this.scopePane.getMagnify() + 5);
			setScene(this.clientScene);
		} else if (o == this.zoomout) {
			this.scopePane.setMagnify(this.scopePane.getMagnify() - 5);
			setScene(this.clientScene);
		}
	}

	// MouseListener
	public void mouseClicked(MouseEvent me) {
	}

	public void mouseEntered(MouseEvent me) {
	}

	public void mouseExited(MouseEvent me) {
	}

	public void mousePressed(MouseEvent me) {
		this.scopePane.setWatchPoint(this.fieldPane
				.screenToField(me.getPoint()));
		setScene(this.clientScene);
	}

	public void mouseReleased(MouseEvent me) {
	}

	// MouseMotionListener
	public void mouseDragged(MouseEvent me) {
		this.scopePane.setWatchPoint(this.fieldPane
				.screenToField(me.getPoint()));
		setScene(this.clientScene);
	}

	public void mouseMoved(MouseEvent me) {
	}

	// ChangeListener
	public void stateChanged(ChangeEvent e) {
		JSlider o = (JSlider) e.getSource();
		FieldPane fp = this.scopePane.getFieldPane();

		if (o == this.magnifySlider) {
			this.scopePane.setMagnify(o.getValue() / 10.0f);
			this.magnifyLabel.setText("Field " + o.getValue() / 10.0f);
		} else if (o == this.playerSlider) {
			fp.setPlayerMagnify(o.getValue() / 10.0f);
			this.playerLabel.setText("Player " + o.getValue() / 10.0f);
		} else if (o == this.ballSlider) {
			fp.setBallMagnify(o.getValue() / 10.0f);
			this.ballLabel.setText("Ball " + o.getValue() / 10.0f);
		}
	}
}
