/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope.view;

import java.awt.Event;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.Iterator;
import java.util.Vector;
import java.util.prefs.Preferences;

import javax.swing.AbstractAction;
import javax.swing.ActionMap;
import javax.swing.ButtonGroup;
import javax.swing.ComponentInputMap;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.Timer;

import soccerscope.model.ClientScene;
import soccerscope.model.GameEvent;
import soccerscope.model.SceneSet;
import soccerscope.model.SoccerScopePreferences;
import soccerscope.model.WorldModel;

public class ScenePlayer extends JToolBar implements ActionListener {

  private static final long serialVersionUID = 2797370871303390715L;

	private ScopePane scopePane;

	private StatusBar statusBar;

	private Vector<ScopeWindow> scopeWindowVector;

	private Timer timer;

	private int index; // scene index (not simulation time)

	private int orient;

	private int interval;

	private boolean enable[];

	private int offset[];

	private final static int PLAY = 0;

	private final static int STOP = 1;

	private final static int REVERSE = 2;

	private final static int SYNC = 3;

	private JToggleButton recordButton;

	private JButton prevEventButton;

	private JButton quickReverseButton;

	private JButton reverseButton;

	private JButton stepReverseButton;

	private JButton stopButton;

	private JButton stepPlayButton;

	private JButton playButton;

	private JButton quickPlayButton;

	private JButton nextEventButton;

	private JButton syncButton;

	private JButton speedUpButton;

	private JButton speedDownButton;
    
	private JToggleButton cycleJumper;
	
	private JToggleButton cycleListJumper;
	
	private JToggleButton eventListJumper;
	
	private ButtonGroup cycleButtonGroup;
	
	private CycleNumberJumper cycleNumberJumper;
	
	private ChronologicScenePlayModeJumper chronoScenePMJumper;
	
	private PlayModeSortedJumper pmSuppportedJumper;

	public ScenePlayer(ScopePane scopePane, StatusBar statusBar,
			boolean canPublish) {
		super();

		this.scopePane = scopePane;
		this.statusBar = statusBar;
		this.scopeWindowVector = new Vector<ScopeWindow>();

		this.index = 0;
		this.orient = STOP;
		this.interval = 100;
		this.timer = new Timer(this.interval, new TimerListener());
		this.timer.stop();
		this.enable = new boolean[GameEvent.MAX];
		this.offset = new int[GameEvent.MAX];

		Preferences pf = SoccerScopePreferences.getPreferences();
		for (int i = GameEvent.MIN; i < GameEvent.MAX; i++) {
			if (i == GameEvent.GOAL)
				this.enable[i] = pf.getBoolean("GameEvent"
						+ GameEvent.getEventName(i), true);
			this.enable[i] = pf.getBoolean("GameEvent" + GameEvent.getEventName(i),
					false);
			this.offset[i] = pf.getInt("GameEvent" + GameEvent.getEventName(i)
					+ "Offset", 0);
		}

		statusBar.setStatus("stop");

		this.recordButton = new JToggleButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/general/Stop24.gif")),
				false);
		this.prevEventButton = new JButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/SearchPrev24.gif")));
		this.quickReverseButton = new JButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/media/Rewind24.gif")));
		this.reverseButton = new JButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/media/PlayBack24.gif")));
		this.stepReverseButton = new JButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/media/StepBack24.gif")));
		this.stopButton = new JButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/media/Pause24.gif")));
		this.stepPlayButton = new JButton(
				new ImageIcon(
						ClassLoader
								.getSystemResource("soccerscope/image/media/StepForward24.gif")));
		this.playButton = new JButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/media/Play24.gif")));
		this.quickPlayButton = new JButton(
				new ImageIcon(
						ClassLoader
								.getSystemResource("soccerscope/image/media/FastForward24.gif")));
		this.nextEventButton = new JButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/SearchNext24.gif")));
		this.syncButton = new JButton("sync");
		System.out.println(ScenePlayer.class.getResource("/").toString());

		this.speedUpButton = new JButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/up.gif")));

		this.speedDownButton = new JButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/down.gif")));

		
		this.cycleJumper = new JToggleButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/clock.gif")), true);
		this.cycleListJumper = new JToggleButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/whistle.gif")), false);
		this.eventListJumper = new JToggleButton(new ImageIcon(ClassLoader
				.getSystemResource("soccerscope/image/whistleclock.gif")), false);
		this.cycleButtonGroup = new ButtonGroup();
		this.cycleButtonGroup.add(cycleJumper);
		this.cycleButtonGroup.add(cycleListJumper);
		this.cycleButtonGroup.add(eventListJumper);
		this.cycleNumberJumper = new CycleNumberJumper(this);
		this.chronoScenePMJumper = new ChronologicScenePlayModeJumper(this);
		this.chronoScenePMJumper.setVisible(false);
		this.pmSuppportedJumper  = new PlayModeSortedJumper(this);
		this.pmSuppportedJumper.setVisible(false);
		this.cycleJumper.addActionListener(this);
		this.cycleListJumper.addActionListener(this);
		this.eventListJumper.addActionListener(this);

		this.add(this.recordButton);
		this.add(this.prevEventButton);
		this.add(this.quickReverseButton);
		this.add(this.reverseButton);
		this.add(this.stepReverseButton);
		this.add(this.speedDownButton);
		this.add(this.stopButton);
		this.add(this.speedUpButton);
		this.add(this.stepPlayButton);
		this.add(this.playButton);
		this.add(this.quickPlayButton);
		this.add(this.nextEventButton);

		this.addSeparator();
		this.add(this.syncButton);
		this.addSeparator();
		
		this.add(this.cycleJumper);
		this.add(this.cycleListJumper);
		this.add(this.eventListJumper);
		this.add(cycleNumberJumper);
		this.add(chronoScenePMJumper);
		this.add(pmSuppportedJumper);
		
		this.recordButton.addActionListener(this);
		this.prevEventButton.addActionListener(this);
		this.quickReverseButton.addActionListener(this);
		this.reverseButton.addActionListener(this);
		this.stepReverseButton.addActionListener(this);
		this.stopButton.addActionListener(this);
		this.stepPlayButton.addActionListener(this);
		this.playButton.addActionListener(this);
		this.quickPlayButton.addActionListener(this);
		this.nextEventButton.addActionListener(this);
		this.syncButton.addActionListener(this);
		this.speedUpButton.addActionListener(this);
		this.speedDownButton.addActionListener(this);

		if (!canPublish)
			this.recordButton.setEnabled(false);

		ActionMap am = getActionMap();
		ComponentInputMap im = new ComponentInputMap(this);

		im.put(KeyStroke.getKeyStroke(KeyEvent.VK_F, Event.CTRL_MASK),
				"forward");
		am.put("forward", new ForwardAction());
		im.put(KeyStroke.getKeyStroke(KeyEvent.VK_B, Event.CTRL_MASK),
				"backward");
		am.put("backward", new BackwardAction());
		im.put(KeyStroke.getKeyStroke(KeyEvent.VK_N, Event.CTRL_MASK), "next");
		am.put("next", new NextAction());
		im.put(KeyStroke.getKeyStroke(KeyEvent.VK_P, Event.CTRL_MASK), "prev");
		am.put("prev", new PrevAction());

		setActionMap(am);
		setInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW, im);

		setVisible(true);
	}

	class ForwardAction extends AbstractAction {
    private static final long serialVersionUID = -4453214908894001169L;

    public void actionPerformed(ActionEvent ae) {
			stepPlay();
		}
	}

	class BackwardAction extends AbstractAction {
    private static final long serialVersionUID = 7458784403569442279L;

    public void actionPerformed(ActionEvent ae) {
			stepReverse();
		}
	}

	class NextAction extends AbstractAction {
    private static final long serialVersionUID = 6055888679143181344L;

    public void actionPerformed(ActionEvent ae) {
			nextEvent(getCurrentTime());
		}
	}

	class PrevAction extends AbstractAction {
    private static final long serialVersionUID = 180381705390441078L;

    public void actionPerformed(ActionEvent ae) {
			prevEvent(getCurrentTime());
		}
	}

	// SceneɥϿ
	public void addScopeWindow(ScopeWindow swindow) {
		this.scopeWindowVector.add(swindow);
		setScene();
	}

    /**
     * Sets the scene given by the number of the scene.
     * @param sceneNumber number of the scene
     */
    public void setScene(int sceneNumber) {
        if (orient == SYNC) {
            this.statusBar.setStatus("stop");
            this.orient = STOP;
            this.interval = 500;
            this.timer.stop();
        }
        index = sceneNumber;
        setScene();
    }
    
    /**
     * Sets the scene given by the attribut index.
     */
	private void setScene() {
		SceneSet sceneSet = WorldModel.getSceneSet();

		// Looks lika a workaround for some bug to me...
		// TODO: Check if needed, fix bug, remove workaround
		if (this.index < 0)
			this.index = 0;
		if (this.index >= sceneSet.size())
			this.index = sceneSet.size() - 1;

		// 
		ClientScene clientScene = sceneSet.elementAt(this.index);
		Iterator it = this.scopeWindowVector.iterator();
		while (it.hasNext()) {
			((ScopeWindow) it.next()).setScene(clientScene);
		}
	}

	public int getCurrentTime() {
		SceneSet sceneSet = WorldModel.getSceneSet();
		return sceneSet.elementAt(this.index).time;
	}

	public ClientScene getCurrentScene() {
		SceneSet sceneSet = WorldModel.getSceneSet();
		return sceneSet.elementAt(this.index);
	}

	public void quickReverse() {
		this.statusBar.setStatus("reverse");
		this.orient = REVERSE;
		this.interval = 10;
		this.timer.setDelay(this.interval);
		this.timer.restart();
	}

	public void reverse() {
		this.statusBar.setStatus("reverse");
		this.orient = REVERSE;
		this.interval = 100;
		this.timer.setDelay(this.interval);
		this.timer.restart();
	}

	public void stepReverse() {
		this.statusBar.setStatus("stop");
		this.orient = STOP;
		this.interval = 500;
		this.timer.stop();
		if (this.index > 0)
			this.index--;
		setScene();
	}

	public void stop() {
		this.statusBar.setStatus("stop");
		this.orient = STOP;
		this.interval = 500;
		this.timer.stop();
	}

	public void stepPlay() {
		this.statusBar.setStatus("stop");
		this.orient = STOP;
		this.interval = 500;
		this.timer.stop();
		SceneSet sceneSet = WorldModel.getSceneSet();
		if (this.index < sceneSet.size())
			this.index++;
		setScene();
	}

	public void play() {
		this.statusBar.setStatus("play");
		this.orient = PLAY;
		this.interval = 100;
		this.timer.setDelay(this.interval);
		this.timer.restart();
	}

	public void quickPlay() {
		this.statusBar.setStatus("play");
		this.orient = PLAY;
		this.interval = 10;
		this.timer.setDelay(this.interval);
		this.timer.restart();
	}

	public void skip(int time) {
		SceneSet sceneSet = WorldModel.getSceneSet();
		if (time >= sceneSet.getLimitTime())
			time = sceneSet.getLimitTime() - 1;
		else if (time == 3000) {
			if (sceneSet.hasScene(3001))
				time = 3001;
			else
				time = 2999;
		} else if (time <= 0)
			time = 1;

		if (sceneSet.hasScene(time))
			this.index = sceneSet.getIndex(time);
		setScene();
	}

	public void head() {
		this.index = 0;
		setScene();
	}

	public void tail() {
		SceneSet sceneSet = WorldModel.getSceneSet();
		this.index = sceneSet.size() - 1;
		setScene();
	}

	public void sync() {
		this.statusBar.setStatus("sync");
		this.orient = SYNC;
		this.interval = 100;
		this.timer.setDelay(this.interval);
		this.timer.restart();
	}

	public void prevEvent(int time) {
		SceneSet sceneSet = WorldModel.getSceneSet();
		GameEvent ge = sceneSet.getPrevEvent(time);
		if (ge == null)
			return;
		if (this.enable[ge.type]) {
			int skiptime = ge.time + this.offset[ge.type];
			if (time <= skiptime)
				prevEvent(ge.time);
			skip(ge.time + this.offset[ge.type]);
			this.statusBar.setStatus(ge.getEventName());
		} else
			prevEvent(ge.time);
	}

	public void nextEvent(int time) {
		SceneSet sceneSet = WorldModel.getSceneSet();
		GameEvent ge = sceneSet.getNextEvent(time);
		if (ge == null)
			return;
		if (this.enable[ge.type]) {
			int skiptime = ge.time + this.offset[ge.type];
			if (time >= skiptime)
				nextEvent(ge.time);
			skip(ge.time + this.offset[ge.type]);
			this.statusBar.setStatus(ge.getEventName());
		} else
			nextEvent(ge.time);
	}

	public void setEventEnabled(int type, boolean b) {
		this.enable[type] = b;
		Preferences pf = SoccerScopePreferences.getPreferences();
		pf.putBoolean("GameEvent" + GameEvent.getEventName(type), b);
	}

	public boolean getEventEnabled(int type) {
		return this.enable[type];
	}

	public void setEventOffset(int type, int n) {
		this.offset[type] = n;
		Preferences pf = SoccerScopePreferences.getPreferences();
		pf.putInt("GameEvent" + GameEvent.getEventName(type) + "Offset", n);
	}

	public int getEventOffset(int type) {
		return this.offset[type];
	}

	public void actionPerformed(ActionEvent ae) {
		Object o = ae.getSource();

		if (o == this.recordButton) {
			this.scopePane.setRecording(this.recordButton.isSelected());
		} else if (o == this.prevEventButton) {
			prevEvent(getCurrentTime());
		} else if (o == this.quickReverseButton) {
			quickReverse();
		} else if (o == this.reverseButton) {
			reverse();
		} else if (o == this.stepReverseButton) {
			stepReverse();
		} else if (o == this.stopButton) {
			stop();
		} else if (o == this.stepPlayButton) {
			stepPlay();
		} else if (o == this.playButton) {
			play();
		} else if (o == this.quickPlayButton) {
			quickPlay();
		} else if (o == this.nextEventButton) {
			nextEvent(getCurrentTime());
		} else if (o == this.syncButton) {
			sync();
		} else if (o == this.speedUpButton) {
			if (this.orient == REVERSE) {
				if (this.interval < 500) {
					if (this.interval < 100)
						this.interval += 10;
					else
						this.interval += 40;
					this.timer.setDelay(this.interval);
					this.timer.restart();
				} else {
					this.orient = STOP;
					this.timer.stop();
				}
			} else {
				if (this.interval > 10) {
					if (this.orient == STOP)
						this.orient = PLAY;
					else if (this.interval <= 100)
						this.interval -= 10;
					else
						this.interval -= 40;
					this.timer.setDelay(this.interval);
					this.timer.restart();
				}
			}
		} else if (o == this.speedDownButton) {
			if (this.orient == PLAY) {
				if (this.interval < 500) {
					if (this.interval < 100)
						this.interval += 10;
					else
						this.interval += 40;
					this.timer.setDelay(this.interval);
					this.timer.restart();
				} else {
					this.orient = STOP;
					this.timer.stop();
				}
			} else {
				if (this.interval > 10) {
					if (this.orient == STOP)
						this.orient = REVERSE;
					else if (this.interval < 100)
						this.interval -= 10;
					else
						this.interval -= 40;
					this.timer.setDelay(this.interval);
					this.timer.restart();
				}
			}
        } else if (o == this.cycleJumper ||
        		o == this.cycleListJumper ||
        		o == this.eventListJumper) {
        	
        	this.cycleNumberJumper.setVisible(cycleJumper.isSelected());
        	this.chronoScenePMJumper.setVisible(cycleListJumper.isSelected());
        	this.pmSuppportedJumper.setVisible(eventListJumper.isSelected());
        }
		
	}


    
    
	class TimerListener implements ActionListener {
		public void actionPerformed(ActionEvent ae) {
			SceneSet sceneSet = WorldModel.getSceneSet();

			if (ScenePlayer.this.orient == PLAY) {
				if (ScenePlayer.this.index < sceneSet.size() - 1)
					ScenePlayer.this.index++;
				else
					stop();
				setScene();
			} else if (ScenePlayer.this.orient == REVERSE) {
				if (ScenePlayer.this.index > 0)
					ScenePlayer.this.index--;
				else
					stop();
				setScene();
			} else if (ScenePlayer.this.orient == SYNC) {
				tail();
			}
		}
	}
}
