/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope.view;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.prefs.Preferences;

import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollBar;

import soccerscope.SoccerScope;
import soccerscope.dialog.TrainingSetupDialog;
import soccerscope.dialog.TrainingStatsDialog;
import soccerscope.model.ClientScene;
import soccerscope.model.Param;
import soccerscope.model.PlayerWorldModel;
import soccerscope.model.SceneSet;
import soccerscope.model.SoccerObjectID;
import soccerscope.model.SoccerScopePreferences;
import soccerscope.model.Team;
import soccerscope.model.WorldModel;
import soccerscope.training.TrainingModel;
import soccerscope.util.geom.Point2f;
import soccerscope.view.layer.AgentLogLayer;
import soccerscope.view.layer.AgentWorldModelLayer;
import soccerscope.view.layer.BallLayer;
import soccerscope.view.layer.BallPossessionLayer;
import soccerscope.view.layer.CommentaryLayer;
import soccerscope.view.layer.ControlRegionLayer;
import soccerscope.view.layer.DelaunayLayer;
import soccerscope.view.layer.DominantRegionLayer;
import soccerscope.view.layer.DynamicSpineLayer;
import soccerscope.view.layer.HearLayer;
import soccerscope.view.layer.KickableLayer;
import soccerscope.view.layer.Layer;
import soccerscope.view.layer.MarkOpponentLayer;
import soccerscope.view.layer.MotionLayer;
import soccerscope.view.layer.NeckLayer;
import soccerscope.view.layer.NoNoiseLayer;
import soccerscope.view.layer.OfflineMoveLayer;
import soccerscope.view.layer.OffsideLayer;
import soccerscope.view.layer.PassCourseLayer;
import soccerscope.view.layer.PassLayer;
import soccerscope.view.layer.PlayerLayer;
import soccerscope.view.layer.SayLayer;
import soccerscope.view.layer.StaminaLayer;
import soccerscope.view.layer.TrainingModelLayer;
import soccerscope.view.layer.TrajectoryLayer;
import soccerscope.view.layer.UnumLayer;
import soccerscope.view.layer.VelocityLayer;
import soccerscope.view.layer.VisibleLayer;
import soccerscope.view.layer.VoronoiLayer;

public class ScopePane extends JPanel implements SoccerObjectID, ScopeWindow,
		AdjustmentListener, SoccerObjectSelectObserver, SoccerObjectSelector {

	private static final long serialVersionUID = 3257570624233683249L;

	private SoccerScope soccerscope;

	private FieldPane fieldPane;

	private InfoBar infoBar;

	private ClientScene clientScene;

	// Сط
	private int resolution = 10; // Сβ

	private JScrollBar horizBar;

	private JScrollBar vertBar;

	private boolean changeMagnify;

	// ޥ⡼
	private int mouseMode;

	public final static int SELECT_MODE = 1;

	public final static int HAND_MODE = 2;

	public final static int MAGNIFY_MODE = 3;

	public final static int MOVE_MODE = 4;

	private ArrayList<Layer> showLayer;

	private ArrayList<Layer> drawLayer;

	// BMPեؤν񤭽Ф
	private int index;

	private boolean recording;

	private BufferedImage bimg;

	// SoccerObjectSelectObserverΥꥹ
	private ArrayList<Object> observerList;

	// OfflineSoccerObjectObserverΥꥹ
	// private ArrayList offlineList;

	// ϰɽ
	private ArrayList<Object> shapeList;

	// 구
	private ArrayList<Object> textList;

	private Ruler ruler;

	// ѹˤ褫
	private boolean isSceneChanged;

	// ե饤󥨡Ȥΰư褹쥤䡼
	private OfflineMoveLayer offlineMoveLayer;

	// ˤ륨ȥץlevelѹΤ
	private AgentLogLayer agentLogLayer;

	private AgentWorldModelLayer agentWorldModelLayer;

	private TrainingModelLayer trainingModelLayer;
	
	private MarkOpponentLayer markOpponentLayer;

	private TrainingSetupDialog tsd = null;

	private TrainingStatsDialog tStats = null;

	private boolean dogBall;

	public ScopePane(SoccerScope soccerscope) {
		this();
		this.soccerscope = soccerscope;
	}

	public ScopePane() {
		super(new GridBagLayout(), true);
		this.soccerscope = null;

		this.index = 0;
		this.recording = false;
		this.bimg = null;
		this.dogBall = false;
		this.observerList = new ArrayList<Object>();
		this.observerList.add(this);
		this.shapeList = new ArrayList<Object>();
		this.textList = new ArrayList<Object>();

		// ߡʥ
		this.clientScene = ClientScene.createScene();

		// ܡ
		this.infoBar = new InfoBar(this.clientScene);
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridwidth = 1;
		gbc.gridheight = 1;
		gbc.weightx = 1.0;
		gbc.weighty = 0.0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.anchor = GridBagConstraints.NORTH;
		this.add(this.infoBar, gbc);

		// ե
		this.fieldPane = new FieldPane(this.clientScene);
		DefaultMouseListener dml = new DefaultMouseListener();
		this.selectModeListener = new SelectModeMouseListener();
		this.handModeListener = new HandModeMouseListener();
		this.magnifyModeListener = new MagnifyModeMouseListener();
		this.moveModeListener = new MoveModeMouseListener();
		this.fieldPane.addMouseListener(dml);
		this.fieldPane.addMouseMotionListener(dml);

		setMouseMode(SELECT_MODE);

		// 쥤䡼Ͽ
		/*
		 * soccerscope2Ǥshowdrawΰ㤤 show : ɽ :
		 * ä˷׻ɬפʤ draw : 褹 : äȷ׻ or
		 * 褹뤿̤ʥǡ(̤˷׻)
		 */
		DominantRegionLayer dominantLayer = new DominantRegionLayer(this.fieldPane,
				false);
		VisibleLayer visibleLayer = new VisibleLayer(this.fieldPane, false);
		TrajectoryLayer trajectoryLayer = new TrajectoryLayer(this.fieldPane, false);
		VoronoiLayer voronoiLayer = new VoronoiLayer(this.fieldPane, false);
		DelaunayLayer delaunayLayer = new DelaunayLayer(this.fieldPane, false);
		CommentaryLayer commentaryLayer = new CommentaryLayer(this.fieldPane, false);
		NoNoiseLayer nonoiseLayer = new NoNoiseLayer(this.fieldPane, false);
		ControlRegionLayer controlLayer = new ControlRegionLayer(this.fieldPane,
				false);
		DynamicSpineLayer spineLayer = new DynamicSpineLayer(this.fieldPane, false);
		this.fieldPane.addBackgroundLayer(dominantLayer);
		this.fieldPane.addBackgroundLayer(visibleLayer);
		this.fieldPane.addBackgroundLayer(trajectoryLayer);
		this.fieldPane.addBackgroundLayer(voronoiLayer);
		this.fieldPane.addBackgroundLayer(delaunayLayer);
		this.fieldPane.addBackgroundLayer(commentaryLayer);
		this.fieldPane.addBackgroundLayer(nonoiseLayer);
		this.fieldPane.addBackgroundLayer(controlLayer);
		this.fieldPane.addBackgroundLayer(spineLayer);

		MotionLayer motionLayer = new MotionLayer(this.fieldPane, false);
		PassCourseLayer passLayer = new PassCourseLayer(this.fieldPane, false);
		PassLayer tpassLayer = new PassLayer(this.fieldPane, false);
		BallPossessionLayer ballposLayer = new BallPossessionLayer(this.fieldPane,
				false);
		NeckLayer neckLayer = new NeckLayer(this.fieldPane, false);
		KickableLayer kickableLayer = new KickableLayer(this.fieldPane, false);
		HearLayer hearLayer = new HearLayer(this.fieldPane, false);
		UnumLayer unumLayer = new UnumLayer(this.fieldPane);
		StaminaLayer staminaLayer = new StaminaLayer(this.fieldPane, false);
		PlayerLayer playerLayer = new PlayerLayer(this.fieldPane);
		SayLayer sayLayer = new SayLayer(this.fieldPane, false);
		OffsideLayer offsideLayer = new OffsideLayer(this.fieldPane, false);
		BallLayer ballLayer = new BallLayer(this.fieldPane);
		VelocityLayer velocityLayer = new VelocityLayer(this.fieldPane, false);
		this.agentWorldModelLayer = new AgentWorldModelLayer(this.fieldPane, false);
		this.trainingModelLayer = new TrainingModelLayer(this.fieldPane, true);
		this.markOpponentLayer = new MarkOpponentLayer(this.fieldPane, true);
		this.addSoccerObjectSelectObserver(this.agentWorldModelLayer);
		this.addSoccerObjectSelectObserver(this.trainingModelLayer);
		this.addSoccerObjectSelectObserver(this.markOpponentLayer);
		// clientWorldModelLayer = new ClientWorldModelLayer(fieldPane, false);
		this.agentLogLayer = new AgentLogLayer(this.fieldPane, false);
		this.offlineMoveLayer = new OfflineMoveLayer(this.fieldPane, false);
		this.fieldPane.addForegroundLayer(motionLayer);
		this.fieldPane.addForegroundLayer(passLayer);
		this.fieldPane.addForegroundLayer(tpassLayer);
		this.fieldPane.addForegroundLayer(ballposLayer);
		this.fieldPane.addForegroundLayer(neckLayer);
		this.fieldPane.addForegroundLayer(kickableLayer);
		this.fieldPane.addForegroundLayer(unumLayer);
		this.fieldPane.addForegroundLayer(staminaLayer);
		this.fieldPane.addForegroundLayer(playerLayer);
		this.fieldPane.addForegroundLayer(sayLayer);
		this.fieldPane.addForegroundLayer(hearLayer);
		this.fieldPane.addForegroundLayer(offsideLayer);
		this.fieldPane.addForegroundLayer(ballLayer);
		this.fieldPane.addForegroundLayer(velocityLayer);
		this.fieldPane.addForegroundLayer(this.agentWorldModelLayer);
		this.fieldPane.addForegroundLayer(this.trainingModelLayer);
		this.fieldPane.addForegroundLayer(this.markOpponentLayer);
		
		// todo fieldPane.addForegroundLayer(clientWorldModelLayer);
		this.fieldPane.addForegroundLayer(this.agentLogLayer);
		this.fieldPane.addForegroundLayer(this.offlineMoveLayer);

		// on/offshowϥ쥤䡼
		this.showLayer = new ArrayList<Layer>();
		this.showLayer.add(neckLayer);
		this.showLayer.add(visibleLayer);
		this.showLayer.add(kickableLayer);
		this.showLayer.add(staminaLayer);
		this.showLayer.add(velocityLayer);
		this.showLayer.add(motionLayer);
		this.showLayer.add(sayLayer);
		this.showLayer.add(hearLayer);
		this.showLayer.add(this.agentLogLayer);
		this.showLayer.add(this.agentWorldModelLayer);
		this.showLayer.add(this.trainingModelLayer);
		this.showLayer.add(this.markOpponentLayer);
		this.showLayer.add(playerLayer);
		// todo showLayer.add(clientWorldModelLayer);
		// on/offdrawϥ쥤䡼
		this.drawLayer = new ArrayList<Layer>();
		this.drawLayer.add(offsideLayer);
		this.drawLayer.add(passLayer);
		this.drawLayer.add(tpassLayer);
		this.drawLayer.add(ballposLayer);
		this.drawLayer.add(dominantLayer);
		this.drawLayer.add(trajectoryLayer);
		this.drawLayer.add(voronoiLayer);
		this.drawLayer.add(delaunayLayer);
		this.drawLayer.add(commentaryLayer);
		this.drawLayer.add(nonoiseLayer);
		this.drawLayer.add(controlLayer);
		this.drawLayer.add(spineLayer);

		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 1;
		gbc.gridwidth = 1;
		gbc.gridheight = 1;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.anchor = GridBagConstraints.NORTH;
		this.add(this.fieldPane, gbc);

		// ľС
		this.vertBar = new JScrollBar(JScrollBar.VERTICAL);
		this.vertBar
				.setMinimum((int) (-(Param.PITCH_WIDTH + 2 * Param.PITCH_MARGIN) / 2 * this.resolution));
		this.vertBar
				.setMaximum((int) ((Param.PITCH_WIDTH + 2 * Param.PITCH_MARGIN) / 2 * this.resolution));
		this.vertBar
				.setValue((int) (-(Param.PITCH_WIDTH + 2 * Param.PITCH_MARGIN) / 2 * this.resolution));
		this.vertBar
				.setVisibleAmount((int) ((Param.PITCH_WIDTH + 2 * Param.PITCH_MARGIN) * this.resolution));
		this.vertBar.addAdjustmentListener(this);
		gbc = new GridBagConstraints();
		gbc.gridx = 1;
		gbc.gridy = 1;
		gbc.gridwidth = 1;
		gbc.gridheight = 1;
		gbc.weightx = 0.0;
		gbc.weighty = 1.0;
		gbc.fill = GridBagConstraints.VERTICAL;
		gbc.anchor = GridBagConstraints.NORTH;
		this.add(this.vertBar, gbc);

		// ʿС
		this.horizBar = new JScrollBar(JScrollBar.HORIZONTAL);
		this.horizBar
				.setMinimum((int) (-(Param.PITCH_LENGTH + 2 * Param.PITCH_MARGIN) / 2 * this.resolution));
		this.horizBar
				.setMaximum((int) ((Param.PITCH_LENGTH + 2 * Param.PITCH_MARGIN) / 2 * this.resolution));
		this.horizBar
				.setValue((int) (-(Param.PITCH_LENGTH + 2 * Param.PITCH_MARGIN) / 2 * this.resolution));
		this.horizBar
				.setVisibleAmount((int) ((Param.PITCH_LENGTH + 2 * Param.PITCH_MARGIN) * this.resolution));
		this.horizBar.addAdjustmentListener(this);
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 2;
		gbc.gridwidth = 1;
		gbc.gridheight = 1;
		gbc.weightx = 1.0;
		gbc.weighty = 0.0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.anchor = GridBagConstraints.NORTH;
		this.add(this.horizBar, gbc);

		this.addMouseWheelListener(new MouseWheelListener() {
			public void mouseWheelMoved(MouseWheelEvent e) {
				if (e.getWheelRotation() > 0)
					ScopePane.this.soccerscope.getScenePlayer().stepPlay();
				else if (e.getWheelRotation() < 0)
					ScopePane.this.soccerscope.getScenePlayer().stepReverse();
			}
		});
	}

	// showϥ쥤䡼μ
	public ArrayList getShowLayer() {
		return this.showLayer;
	}

	// drawϥ쥤䡼μ
	public ArrayList getDrawLayer() {
		return this.drawLayer;
	}

	// AgentLogLayerμ
	public AgentLogLayer getAgentLogLayer() {
		return this.agentLogLayer;
	}

	// AgentWorldModelLayerμ
	public AgentWorldModelLayer getAgentWorldModelLayer() {
		return this.agentWorldModelLayer;
	}

	// observerϿ
	public void addSoccerObjectSelectObserver(SoccerObjectSelectObserver soso) {
		this.observerList.add(soso);
		broadCastStateChanged();
	}

	// ѲobserverΤ
	private void deliverStateChanged(int id, boolean flag) {
		Iterator it = this.observerList.iterator();
		while (it.hasNext()) {
			Object o = it.next();
			if (o instanceof SoccerObjectSelectObserver)
				((SoccerObjectSelectObserver) o).selectSoccerObject(id, flag);
		}
	}

	// Ѳ
	public void selectSoccerObject(int id, boolean flag) {
		this.fieldPane.setSelected(id, flag);
	}

	public boolean isSelected(int id) {
		return this.fieldPane.isSelected(id);
	}

	public void selectOfflineSoccerObject(int id, boolean flag) {
		this.fieldPane.setOfflineSelected(id, flag);
	}

	public void refreshSelectOfflineSoccerObject() {
		for (int id = 0; id < MAX_ID; id++)
			this.fieldPane.setOfflineSelected(id, false);
	}

	// ߤξ֤Ʊ󤹤
	public void broadCastStateChanged() {
		for (int i = 0; i < MAX_ID; i++)
			deliverStateChanged(i, this.fieldPane.isSelected(i));
	}

	public void setScenePlayer(ScenePlayer scenePlayer) {
		scenePlayer.addScopeWindow(this.infoBar);
		scenePlayer.addScopeWindow(this.fieldPane);
		scenePlayer.addScopeWindow(this);
	}

	public FieldPane getFieldPane() {
		return this.fieldPane;
	}

	public InfoBar getInfoBar() {
		return this.infoBar;
	}

	public ClientScene getScene() {
		return this.clientScene;
	}

	// ɽ
	public void setScene(ClientScene clientScene) {
		// System.out.println("TEST");
		this.clientScene = clientScene;
		WorldModel wm = WorldModel.getInstance();
		SceneSet sceneSet = wm.getSceneSet();
		this.index = sceneSet.indexOf(clientScene);
		this.isSceneChanged = true;
		if (this.dogBall)
			this.fieldPane.setWatchPoint(clientScene.ball.pos);
		repaint();
	}

	// Ͽ⡼ɤɤμ
	public boolean isRecording() {
		return this.recording;
	}

	// Ͽ⡼ɤ
	public void setRecording(boolean flag) {
		this.recording = flag;
	}

	// ܡɤä
	public void setBallDogMode(boolean flag) {
		this.dogBall = flag;
	}

	@Override
  public void paint(Graphics g) {
		Dimension d = this.getSize();

		// ҥݡͥȤκ褬
		// ҥݡͥȤbimg
		if (this.bimg == null || this.bimg.getWidth() != d.width
				|| this.bimg.getHeight() != d.height) {
			this.bimg = (BufferedImage) createImage(d.width, d.height);
			Graphics g2 = this.bimg.getGraphics();
			g2.setClip(g.getClip());
			paintChildren(g2);
			this.isSceneChanged = false;
		}
		// ҥݡͥȤ
		if (!this.isSceneChanged && this.dragged
				&& (this.mouseMode == SELECT_MODE || this.mouseMode == MAGNIFY_MODE)) {
			g.drawImage(this.bimg, 0, 0, this);
		} else {
			Graphics g2 = this.bimg.getGraphics();
			g2.setClip(g.getClip());
			paintChildren(g2);
			this.isSceneChanged = false;
			g.drawImage(this.bimg, 0, 0, this);
		}

		// shapeListǤʤʤ,褹
		// gľ
		if (!this.shapeList.isEmpty()) {
			Graphics2D g2 = (Graphics2D) g;
			g2.setColor(Color.lightGray);
			g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
					0.5f));
			int sls = this.shapeList.size();
			for (int i = 0; i < sls; i++)
				g2.draw((Shape) this.shapeList.get(i));
			g2.setComposite(AlphaComposite
					.getInstance(AlphaComposite.SRC, 1.0f));
		}

		if (!this.textList.isEmpty()) {
			g.setColor(Color.white);
			int tls = this.textList.size();
			for (int i = 0; i < tls; i++) {
				Text text = (Text) this.textList.get(i);
				g.drawString(text.str, text.x, text.y);
			}
		}

		if (this.ruler != null) {
			this.ruler.draw(g);
		}

		// Ͽ褹
		if (this.recording) {
			try {
				d.width -= this.vertBar.getWidth();
				d.height -= this.horizBar.getHeight();

				DecimalFormat df = new DecimalFormat("0000");
				Preferences pf = SoccerScopePreferences.getPreferences();
				String type = pf.get("PublishType", "png");
				File f = new File("bmps/" + this.soccerscope.getDataSourceName()
						+ df.format(this.index) + "." + type);
				javax.imageio.ImageIO.write(this.bimg.getSubimage(0, 0, d.width,
						d.height), type, f);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	// ߤγΨ򥹥С˥եɥХå
	private void updateMagnify() {
		Dimension dSize = this.fieldPane.getDrawableSize();
		Point2f watchPoint = this.fieldPane.getWatchPoint();
		this.changeMagnify = true;
		this.vertBar.setVisibleAmount(dSize.height * this.resolution);
		this.horizBar.setVisibleAmount(dSize.width * this.resolution);
		this.vertBar.setValue((int) (watchPoint.y * this.resolution - dSize.height
				* this.resolution / 2));
		this.horizBar.setValue((int) (watchPoint.x * this.resolution - dSize.width
				* this.resolution / 2));
		this.vertBar.setVisibleAmount(dSize.height * this.resolution);
		this.horizBar.setVisibleAmount(dSize.width * this.resolution);
		this.changeMagnify = false;
	}

	// Сθ
	public void setWatchPoint(Point2f pos) {
		this.fieldPane.setWatchPoint(pos);
		updateMagnify();
	}

	// СθΨ
	public void setMagnify(float magnify) {
		this.fieldPane.setMagnify(magnify);
		updateMagnify();
	}

	// СθΨ
	public void setMagnify(float magnify, float ballMagnify, float playerMagnify) {
		this.fieldPane.setMagnify(magnify, ballMagnify, playerMagnify);
		updateMagnify();
	}

	// СθΨ
	public void fitMagnify() {
		this.fieldPane.setWatchPoint(0, 0);
		this.fieldPane.fitMagnify();
		updateMagnify();
	}

	// Ψμ
	public float getMagnify() {
		return this.fieldPane.getMagnify();
	}

	// AdjustmentListener
	public void adjustmentValueChanged(AdjustmentEvent ae) {
		JScrollBar sBar = (JScrollBar) ae.getSource();

		// magnifyѹܥˤ륤٥Ȥ̵뤹
		if (this.changeMagnify)
			return;

		if (sBar == this.vertBar) {
			int y = sBar.getValue() + sBar.getVisibleAmount() / 2;

			Point2f watchPoint = this.fieldPane.getWatchPoint();
			watchPoint.y = y / this.resolution;
			this.fieldPane.setWatchPoint(watchPoint);

		} else if (sBar == this.horizBar) {
			int x = sBar.getValue() + sBar.getVisibleAmount() / 2;

			Point2f watchPoint = this.fieldPane.getWatchPoint();
			watchPoint.x = x / this.resolution;
			this.fieldPane.setWatchPoint(watchPoint);

		}
	}

	// ޥ⡼ɤꤹ
	public void setMouseMode(int mode) {
		this.mouseMode = mode;
		// 򤵤줿֥Ȥ򤹤٤ƽ֤᤹
		refreshSelectOfflineSoccerObject();
		// ⡼ɤˤäƥޥѤ
		if (this.mouseMode == SELECT_MODE) {
			setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			this.listener = this.selectModeListener;
		} else if (this.mouseMode == HAND_MODE) {
			setCursor(new Cursor(Cursor.HAND_CURSOR));
			this.listener = this.handModeListener;
		} else if (this.mouseMode == MAGNIFY_MODE) {
			setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
			this.listener = this.magnifyModeListener;
		} else if (this.mouseMode == MOVE_MODE) {
			setCursor(new Cursor(Cursor.HAND_CURSOR));
			this.listener = this.moveModeListener;
		}
	}

	// ޥ⡼ɤ
	public int getMouseMode() {
		return this.mouseMode;
	}

	class Text {
		String str;

		int x;

		int y;
	}

	private int getSelectedSoccerObjectID(Point2f press) {
		float ballSize = Param.BALL_SIZE * this.fieldPane.getBallMagnify();
		if (press.distanceLinf(this.clientScene.ball.pos) <= ballSize) {
			return BALL;
		}

		int MAX_N = Param.MAX_PLAYER * 2;
		for (int i = 0; i < MAX_N; i++) {
			float playerSize = this.clientScene.player[i].getSize()
					* this.fieldPane.getPlayerMagnify();
			if (press.distanceLinf(this.clientScene.player[i].pos) <= playerSize) {
				return i;
			}
		}

		return MAX_ID;
	}

	interface MouseEventListener extends MouseListener, MouseMotionListener {
	}

	private MouseEventListener listener;

	private SelectModeMouseListener selectModeListener;

	private HandModeMouseListener handModeListener;

	private MagnifyModeMouseListener magnifyModeListener;

	private MoveModeMouseListener moveModeListener;

	private Point pressPoint;

	private Point releasePoint;

	private Point dragPoint;

	private Point2f currentPoint;

	private Point2f press;

	private int selectID;

	private boolean moving = false;

	private boolean dragged = false; // ޥɥå

	class DefaultMouseListener implements MouseEventListener {
		public void mouseClicked(MouseEvent me) {
			final MouseEvent mef = me;
			// å
			if (me.getModifiers() == InputEvent.BUTTON1_MASK) {
				ScopePane.this.listener.mouseClicked(me);
			}

			// 楯å
			else if (me.getModifiers() == InputEvent.BUTTON2_MASK) {
				openTrainScenarioDialog();
			}

			// å
			else if (me.getModifiers() == InputEvent.BUTTON3_MASK) {
				// ݥåץåץ˥塼Ф
				Point2f press = ScopePane.this.fieldPane.screenToField(me.getPoint());
				JPopupMenu jpop = new JPopupMenu();
				Iterator showit = ScopePane.this.showLayer.iterator();
				while (showit.hasNext()) {
					jpop.add(((Layer) showit.next()).createJCheckBoxMenuItem());
				}
				jpop.addSeparator();
				// ץ쥤䡼򥯥å
				int ID = getSelectedSoccerObjectID(press);
				if (ID != MAX_ID) {
					JMenu jm = new JMenu("Property");
					if (ID == BALL) {
						jm.add(new JMenuItem("pos:" + ScopePane.this.clientScene.ball.pos));
						jm.add(new JMenuItem("vel:" + ScopePane.this.clientScene.ball.vel));
					} else {
						jm.add(new JMenuItem("pos:"
								+ ScopePane.this.clientScene.player[ID].pos));
						jm.add(new JMenuItem("vel:"
								+ ScopePane.this.clientScene.player[ID].vel));
						jm.add(new JMenuItem("angle:"
								+ ScopePane.this.clientScene.player[ID].angle));
						jm.add(new JMenuItem("neck:"
								+ ScopePane.this.clientScene.player[ID].angleNeck));
						jm.add(new JMenuItem("stam:"
								+ ScopePane.this.clientScene.player[ID].stamina));

						if (ScopePane.this.clientScene.player[ID].sayMessage != null
								&& ScopePane.this.clientScene.player[ID].sayMessage.length() > 0) {
							jm.add(new JMenuItem("say: "
									+ ScopePane.this.clientScene.player[ID].sayMessage));
						}
						for (int i = 0; i < ScopePane.this.clientScene.player[ID].hearMessage
								.size(); i++)
							jm
									.add(new JMenuItem(
											"heared: "
													+ ScopePane.this.clientScene.player[ID].hearMessage
															.get(i)));
					}
					jpop.add(jm);
					jpop.addSeparator();
					// display all states an values:
					if (ID != BALL) {
						JMenu stateMenue = new JMenu("States");
						int cycle = ScopePane.this.clientScene.time;
						int playerNumber = ID;
                        PlayerWorldModel pwm =clientScene.player[playerNumber].getWorldModel();
                        if (pwm != null) {
    						LinkedList<String> list = pwm.getStateInfos();
    						for (String str : list) {
    							stateMenue.add(new JMenuItem(str));
    						}
    						jpop.add(stateMenue);
    						jpop.addSeparator();
                        }
                    }
				}

				if (ScopePane.this.soccerscope != null) {
					soccerscope.net.SoccerServerConnection ssc = ScopePane.this.soccerscope
							.getSoccerServerConnection();
					if (ssc != null) {
						JMenuItem xPos = new JMenuItem("x: "
								+ ScopePane.this.fieldPane.screenToField(mef.getPoint()).x);
						JMenuItem yPos = new JMenuItem("y: "
								+ ScopePane.this.fieldPane.screenToField(mef.getPoint()).y);
						jpop.add(xPos);
						jpop.add(yPos);
						JMenuItem dropBall = new JMenuItem("Drop Ball");
						dropBall.addActionListener(new ActionListener() {
							public void actionPerformed(ActionEvent event) {
								float xf = ScopePane.this.fieldPane.screenToField(mef
										.getPoint()).x;
								float yf = ScopePane.this.fieldPane.screenToField(mef
										.getPoint()).y;

								// float xf = mef.getPoint().x /
								// fieldPane.getMagnify();
								// float yf = mef.getPoint().y /
								// fieldPane.getMagnify();
								try {
									ScopePane.this.soccerscope.getSoccerServerConnection()
											.dispfoul(xf, yf, Team.NEUTRAL);
								} catch (Exception ex) {
								}
							}
						});
						JMenuItem foulLeft = new JMenuItem("Foul Left");
						foulLeft.addActionListener(new ActionListener() {
							public void actionPerformed(ActionEvent event) {
								float xf = ScopePane.this.fieldPane.screenToField(mef
										.getPoint()).x;
								float yf = ScopePane.this.fieldPane.screenToField(mef
										.getPoint()).y;

								// float xf = mef.getPoint().x /
								// fieldPane.getMagnify();
								// float yf = mef.getPoint().y /
								// fieldPane.getMagnify();
								try {
									ScopePane.this.soccerscope.getSoccerServerConnection()
											.dispfoul(xf, yf, Team.LEFT_SIDE);
								} catch (Exception ex) {
								}
							}
						});
						JMenuItem foulRight = new JMenuItem("Foul Right");
						foulRight.addActionListener(new ActionListener() {
							public void actionPerformed(ActionEvent event) {
								float xf = ScopePane.this.fieldPane.screenToField(mef
										.getPoint()).x;
								float yf = ScopePane.this.fieldPane.screenToField(mef
										.getPoint()).y;

								// float xf = mef.getPoint().x /
								// fieldPane.getMagnify();
								// float yf = mef.getPoint().y /
								// fieldPane.getMagnify();
								try {
									ScopePane.this.soccerscope.getSoccerServerConnection()
											.dispfoul(xf, yf, Team.RIGHT_SIDE);
								} catch (Exception ex) {
								}
							}
						});
						jpop.add(dropBall);
						jpop.add(foulLeft);
						jpop.add(foulRight);

						// jpop.add(new JMenuItem("Drop Ball"));

					}

					/*
					 * try { int xMeter = me.getX(); if (xMeter > 68 | xMeter <
					 * -68) xMeter = 0; int yMeter = me.getY(); if (yMeter > 39 |
					 * yMeter < -39) yMeter = 0;
					 * 
					 * ClientScene clientScene =
					 * soccerscope.getScenePlayer().getCurrentScene();
					 * 
					 * soccerscope.getSoccerServerConnection().dispfoul(xMeter,yMeter,
					 * Team.NEUTRAL);
					 *  } catch (IOException ie) { System.err.println(ie); }
					 */

				}

				jpop.pack();
				jpop.show(ScopePane.this.fieldPane, me.getX(), me.getY());
			}
		}

		public void mousePressed(MouseEvent me) {

			ScopePane.this.pressPoint = me.getPoint();
			ScopePane.this.currentPoint = ScopePane.this.fieldPane.getWatchPoint();

			if (me.getModifiers() == InputEvent.BUTTON1_MASK) {
				ScopePane.this.listener.mousePressed(me);
			}
		}

		public void mouseReleased(MouseEvent me) {

			if (!ScopePane.this.shapeList.isEmpty()) {
				ScopePane.this.shapeList.clear();
				repaint();
			}
			if (!ScopePane.this.textList.isEmpty()) {
				ScopePane.this.textList.clear();
				repaint();
			}
			if (ScopePane.this.ruler != null) {
				ScopePane.this.ruler = null;
				repaint();
			}

			ScopePane.this.releasePoint = me.getPoint();

			// ޥɥåƤʤФʤˤ⤷ʤ
			if (!ScopePane.this.dragged)
				return;

			ScopePane.this.dragged = false;

			if (me.getModifiers() == InputEvent.BUTTON1_MASK) {
				ScopePane.this.listener.mouseReleased(me);
			}
		}

		public void mouseEntered(MouseEvent me) {
		}

		public void mouseExited(MouseEvent me) {
		}

		public void mouseDragged(MouseEvent me) {

			ScopePane.this.dragPoint = me.getPoint();
			ScopePane.this.dragged = true;

			// ɥå
			if (me.getModifiers() == InputEvent.BUTTON1_MASK) {
				ScopePane.this.listener.mouseDragged(me);
			}

			// ɥå
			else if (me.getModifiers() == InputEvent.BUTTON2_MASK) {
				// Ф
				ScopePane.this.ruler = new Ruler(ScopePane.this.pressPoint.x, ScopePane.this.pressPoint.y
						+ ScopePane.this.infoBar.getHeight(), ScopePane.this.dragPoint.x, ScopePane.this.dragPoint.y
						+ ScopePane.this.infoBar.getHeight(), ScopePane.this.fieldPane, ScopePane.this.infoBar.getHeight());
				repaint();
			}
		}

		public void mouseMoved(MouseEvent me) {
		}
	}

	// SELECT_MODE Υ٥ȥꥹʡ
	class SelectModeMouseListener implements MouseEventListener {
		public void mouseClicked(MouseEvent me) {
			Point2f press = ScopePane.this.fieldPane.screenToField(me.getPoint());
			int ID = getSelectedSoccerObjectID(press);
			if (ID != MAX_ID)
				deliverStateChanged(ID, !ScopePane.this.fieldPane.isSelected(ID));
		}

		public void mousePressed(MouseEvent me) {

		}

		public void mouseReleased(MouseEvent me) {
			// ϰϤη׻

			Dimension d = new Dimension(
					Math.abs(ScopePane.this.releasePoint.x - ScopePane.this.pressPoint.x), Math
							.abs(ScopePane.this.releasePoint.y - ScopePane.this.pressPoint.y));
			Point p = new Point(Math.min(ScopePane.this.pressPoint.x, ScopePane.this.releasePoint.x), Math
					.min(ScopePane.this.pressPoint.y, ScopePane.this.releasePoint.y));
			Rectangle r = new Rectangle(p, d);

			// ܡ뤬ϰϤ˴ޤޤ뤫
			float ballSize = Param.BALL_SIZE * ScopePane.this.fieldPane.getBallMagnify()
					* ScopePane.this.fieldPane.getMagnify();
			Point2f ballPoint = ScopePane.this.fieldPane.fieldToScreen(ScopePane.this.clientScene.ball.pos);
			Rectangle ballRect = new Rectangle((int) (ballPoint.x - ballSize),
					(int) (ballPoint.y - ballSize), (int) (ballSize * 2),
					(int) (ballSize * 2));
			if (r.contains(ballRect) || r.intersects(ballRect)) {
				deliverStateChanged(BALL, !ScopePane.this.fieldPane.isSelected(BALL));

			}

			// ץ쥤䤬ϰϤ˴ޤޤ뤫
			for (int i = 0; i < Param.MAX_PLAYER * 2; i++) {
				float playerSize = ScopePane.this.clientScene.player[i].getSize()
						* ScopePane.this.fieldPane.getPlayerMagnify() * ScopePane.this.fieldPane.getMagnify();
				Point2f pPoint = ScopePane.this.fieldPane
						.fieldToScreen(ScopePane.this.clientScene.player[i].pos);
				Rectangle pRect = new Rectangle((int) (pPoint.x - playerSize),
						(int) (pPoint.y - playerSize), (int) (playerSize * 2),
						(int) (playerSize * 2));
				if (r.contains(pRect) || r.intersects(pRect)) {
					deliverStateChanged(i, !ScopePane.this.fieldPane.isSelected(i));
				}
			}
		}

		public void mouseEntered(MouseEvent me) {
		}

		public void mouseExited(MouseEvent me) {
		}

		public void mouseDragged(MouseEvent me) {
			// ̤ϰϤ褹
			Dimension d = new Dimension(Math.abs(ScopePane.this.dragPoint.x - ScopePane.this.pressPoint.x),
					Math.abs(ScopePane.this.dragPoint.y - ScopePane.this.pressPoint.y));
			Point p = new Point(Math.min(ScopePane.this.pressPoint.x, ScopePane.this.dragPoint.x), Math.min(
					ScopePane.this.pressPoint.y, ScopePane.this.dragPoint.y)
					+ ScopePane.this.infoBar.getHeight());
			ScopePane.this.shapeList.clear();
			ScopePane.this.shapeList.add(new Rectangle(p, d));
			repaint();
		}

		public void mouseMoved(MouseEvent me) {
		}
	}

	// HAND_MODE Υ٥ȥꥹʡ
	class HandModeMouseListener implements MouseEventListener {
		public void mouseClicked(MouseEvent me) {

		}

		public void mousePressed(MouseEvent me) {

			press = fieldPane.screenToField(me.getPoint());
			selectID = getSelectedSoccerObjectID(press);
			if (selectID != MAX_ID) {
				selectOfflineSoccerObject(selectID, true);
				offlineMoveLayer.setEnable(true, clientScene, press);
			}
		}

		public void mouseReleased(MouseEvent me) {

			if (selectID != MAX_ID) {
				if (selectID == BALL) {
					clientScene.ball.pos = offlineMoveLayer
							.getMovePoint(selectID);
				} else {
					clientScene.player[selectID].pos = offlineMoveLayer
							.getMovePoint(selectID);
				}
			}
			offlineMoveLayer.setEnable(false);
			refreshSelectOfflineSoccerObject();
			isSceneChanged = true;
			repaint();
		}

		public void mouseEntered(MouseEvent me) {
		}

		public void mouseExited(MouseEvent me) {
		}

		public void mouseDragged(MouseEvent me) {

			if (selectID == MAX_ID) {
				// ư
				Point2f watchPoint = new Point2f(currentPoint);
				Point2f moveVector = new Point2f(dragPoint.x - pressPoint.x,
						dragPoint.y - pressPoint.y);
				moveVector.scale(1.0f / fieldPane.getMagnify());
				watchPoint.sub(moveVector);
				fieldPane.setWatchPoint(watchPoint);
				updateMagnify();
			} else {
				Point2f movePoint = fieldPane.screenToField(dragPoint);
				movePoint.x = (float) Math.rint(movePoint.x);
				movePoint.y = (float) Math.rint(movePoint.y);
				offlineMoveLayer.move(movePoint);
				isSceneChanged = true;
				repaint();
			}
		}

		public void mouseMoved(MouseEvent me) {
		}
	}

	// MOVE_MODEΥ٥ȥꥹʡ
	class MoveModeMouseListener implements MouseEventListener {
		public void mouseClicked(MouseEvent me) {
			Point2f press = fieldPane.screenToField(me.getPoint());
			int ID = getSelectedSoccerObjectID(press);
			if (ID == MAX_ID)
				refreshSelectOfflineSoccerObject();
		}

		public void mousePressed(MouseEvent me) {
			Point2f press = fieldPane.screenToField(me.getPoint());
			selectID = getSelectedSoccerObjectID(press);
			// ư쥤䡼賫&֥Ȥ򿿤
			if (selectID != MAX_ID) {
				selectOfflineSoccerObject(selectID, true);
				moving = true;
				offlineMoveLayer.setEnable(true, clientScene, press);
			}
		}

		public void mouseReleased(MouseEvent me) {

			if (!moving) {
				// ϰϤη׻
				Dimension d = new Dimension(Math.abs(releasePoint.x
						- pressPoint.x), Math
						.abs(releasePoint.y - pressPoint.y));
				Point p = new Point(Math.min(pressPoint.x, releasePoint.x),
						Math.min(pressPoint.y, releasePoint.y));
				Rectangle r = new Rectangle(p, d);

				// ܡ뤬ϰϤ˴ޤޤ뤫
				float ballSize = Param.BALL_SIZE * fieldPane.getBallMagnify()
						* fieldPane.getMagnify();
				Point2f ballPoint = fieldPane
						.fieldToScreen(clientScene.ball.pos);
				Rectangle ballRect = new Rectangle(
						(int) (ballPoint.x - ballSize),
						(int) (ballPoint.y - ballSize), (int) (ballSize * 2),
						(int) (ballSize * 2));
				if ((r.contains(ballRect) || r.intersects(ballRect)))
					selectOfflineSoccerObject(BALL, true);

				// ץ쥤䤬ϰϤ˴ޤޤ뤫
				int MAX_N = Param.MAX_PLAYER;
				for (int i = 0; i < MAX_N; i++) {
					float playerSize = clientScene.player[i].getSize()
							* fieldPane.getPlayerMagnify()
							* fieldPane.getMagnify();
					Point2f pPoint = fieldPane
							.fieldToScreen(clientScene.player[i].pos);
					Rectangle pRect = new Rectangle(
							(int) (pPoint.x - playerSize),
							(int) (pPoint.y - playerSize),
							(int) (playerSize * 2), (int) (playerSize * 2));
					if (r.contains(pRect) || r.intersects(pRect)) {
						selectOfflineSoccerObject(i, true);
					}
				}

				offlineMoveLayer.setTest(false);
			} else {
				TrainingModel tm = WorldModel.getInstance().trainModel;
				for (int id = 0; id < MAX_ID; id++) {
					if (id == BALL && fieldPane.isOfflineSelected(id)) {
						clientScene.ball.setPosition(offlineMoveLayer
								.getMovePoint(id));
						if (tm != null)
							tm.objectTrainingModel[id]
									.setPosition(offlineMoveLayer
											.getMovePoint(id));
					} else if (fieldPane.isOfflineSelected(id)) {
						clientScene.player[id].setPosition(offlineMoveLayer
								.getMovePoint(id));
						if (tm != null)
							tm.objectTrainingModel[id]
									.setPosition(offlineMoveLayer
											.getMovePoint(id));
					}
				}
				offlineMoveLayer.setEnable(false);
				refreshSelectOfflineSoccerObject();
				isSceneChanged = true;
				repaint();
				offlineMoveLayer.setTest(false);
				moving = false;
			}
		}

		public void mouseEntered(MouseEvent me) {
		}

		public void mouseExited(MouseEvent me) {
		}

		public void mouseDragged(MouseEvent me) {
			if (!moving) {
				// ̤ϰϤ褹
				Dimension d = new Dimension(Math
						.abs(dragPoint.x - pressPoint.x), Math.abs(dragPoint.y
						- pressPoint.y));
				Point p = new Point(Math.min(pressPoint.x, dragPoint.x), Math
						.min(pressPoint.y, dragPoint.y)
						+ infoBar.getHeight());
				shapeList.clear();
				shapeList.add(new Rectangle(p, d));
				repaint();
			} else {
				if (selectID != MAX_ID) {
					Point2f movePoint = fieldPane.screenToField(dragPoint);
					movePoint.x = (float) Math.rint(movePoint.x);
					movePoint.y = (float) Math.rint(movePoint.y);
					offlineMoveLayer.move(movePoint);

					isSceneChanged = true;
					repaint();
				}
			}
		}

		public void mouseMoved(MouseEvent me) {
		}
	}

	// MAGNIFY_MODE Υ٥ȥꥹʡ
	class MagnifyModeMouseListener implements MouseEventListener {
		public void mouseClicked(MouseEvent me) {
			System.out.println("8");
			// Ψ10ܤ
			Point2f watchPoint = fieldPane.screenToField(me.getPoint());
			fieldPane.setWatchPoint(watchPoint);
			float magnify = fieldPane.getMagnify() + 10;
			setMagnify(magnify);
		}

		public void mousePressed(MouseEvent me) {

		}

		public void mouseReleased(MouseEvent me) {
			// 
			Point p = new Point((pressPoint.x + releasePoint.x) / 2,
					(pressPoint.y + releasePoint.y) / 2);
			Point2f watchPoint = fieldPane.screenToField(p);
			fieldPane.setWatchPoint(watchPoint);

			// Ψ
			Dimension d = new Dimension(
					Math.abs(releasePoint.x - pressPoint.x), Math
							.abs(releasePoint.y - pressPoint.y));
			Dimension dSize = fieldPane.getSize();
			float magnify = Math.min((float) dSize.width / (float) d.width,
					(float) dSize.height / (float) d.height)
					* fieldPane.getMagnify();
			setMagnify(magnify);
		}

		public void mouseEntered(MouseEvent me) {
		}

		public void mouseExited(MouseEvent me) {
		}

		public void mouseDragged(MouseEvent me) {
			// ̤ϰϤ褹
			Dimension d = new Dimension(Math.abs(dragPoint.x - pressPoint.x),
					Math.abs(dragPoint.y - pressPoint.y));
			Point p = new Point(Math.min(pressPoint.x, dragPoint.x), Math.min(
					pressPoint.y, dragPoint.y)
					+ infoBar.getHeight());
			shapeList.clear();
			shapeList.add(new Rectangle(p, d));
			repaint();
		}

		public void mouseMoved(MouseEvent me) {
		}
	}

	public void printOut() {
		for (int i = 0; i < Param.MAX_PLAYER; i++) {
			System.out.println(clientScene.player[i].pos);
		}
		System.out.println();
	}

	/**
	 * This method opens the Train-Scenario Editor Dialog
	 */
	public void openTrainScenarioDialog() {
		ClientScene scene = (ClientScene) fieldPane.getScene();
		if (tsd == null) {
			tsd = new TrainingSetupDialog(soccerscope, scene);
			observerList.add(tsd);
			tsd.setVisible(true);
		} else
			tsd.setVisible(true);
	}

	public void openTrainStatsDialog() {
		ClientScene scene = (ClientScene) fieldPane.getScene();
		if (tStats == null) {
			tStats = new TrainingStatsDialog(soccerscope, scene);
			observerList.add(tStats);
			tStats.setVisible(true);
		} else {
			tStats.setVisible(true);
		}
	}
}
/**
 * $Log: ScopePane.java,v $
 * Revision 1.5.16.5  2006/11/23 09:41:04  endert
 * Added License Note at top of each *.java file.
 *
 * Revision 1.5.16.4  2006/11/13 16:50:20  endert
 * Removed CVS-Logs and author tags
 *
 * Revision 1.5.16.3  2006/10/25 14:33:37  endert
 * Latest SoccerScope Version plus Database ability
 *
 * Revision 1.5.4.2  2006/06/26 14:19:12  benjaboe
 * Player get his own PlayerWorldModel.
 * No NullPointerExceptions when Player has no PlayerWorldModel.
 *
 * Revision 1.5.4.1  2006/06/26 07:04:24  benjaboe
 * Datamodell umgebaut
 *
 * Revision 1.5  2006/05/17 14:08:06  endert
 * New Layer for MarkArea
 *
 * Revision 1.4  2006/04/18 22:46:31  wetzker
 * merged with situationEvaluation branch
 * new:
 * - central situation evaluation for states with ball control
 * - kickNRun situation
 * - ball interception possible when ball kickable
 * - role model for players 6-11(roles are shown in gui, roles not used much for acting yet)
 *
 * Revision 1.3  2006/04/18 00:28:00  endert
 * PointTo Ready, OpponentAnalyzer fixed and GUI updated!
 *
 * Revision 1.2  2006/03/27 23:25:53  wetzker
 * simplified stateEvaluation
 * - moved most of SimpleState into Abstract
 * - moved most of StateEvaluation into StateTactic
 * adapted GUIUpdate
 * - deleted state tables
 * - only 1 state update each cycle
 * - gui should now be much faster (less holes)
 *
 * Revision 1.1  2006/03/13 07:37:49  axle
 * moved from ROOT/src for Maven 2 Buildsystem and Team/Soccerscope separation
 *
 * Revision 1.15  2006/03/07 12:46:35  endert
 * Merge after refactoring:
 *  - New State-Interface: One Method for each action type
 *  - Action Bean removed - ActionFactorys for helper methods, States calculate
 *    main-Action themselves
 *  - formatted code
 *  - added State-List in Monitor (Context-Menu of Player)
 *  - other smaller stuff
 *
 * Revision 1.14.2.2  2006/03/06 23:17:51  endert
 * Removed warnings, added State-Overview to Context-Menu!
 *
 * Revision 1.14.2.1  2006/02/25 22:07:31  wetzker
 * formated
 *
 * Revision 1.14  2006/02/24 18:53:55  wetzker
 * used enums for Play_modes and Referee_messages
 * - still has to be improved after creation of new Play_mode-Model
 * - tagged old version
 * removed millions of warnings
 *
 * Revision 1.13  2006/02/22 16:04:47  endert
 * This is done:
 * - Rebound SoccerScope to Agents (can be added or removed in config-file)
 * - Situations now all extend abstract class Situation
 * - Refactorings: ActionBean _ removed from Attributes
 * - SoccerScope will automatically connect to host given in Start-Call
 * - Trainer uses the same host as soccerscope
 * - Removed unused stuff (old confs, bat, etc.)
 * - Code formatted (good luck when merging)
 * - organized imports
 * - all that i forgot to mention here
 * Revision 1.12 2006/02/06 17:10:04 endert *** empty log message ***
 * 
 * Revision 1.10.6.1 2005/12/27 19:35:52 endert New Branch created for my Dipl!
 * 
 * Revision 1.10 2005/12/02 14:27:40 endert Added Training-Statistics and
 * Tacke-Changes from Branch 'TrainingStatistics'
 * 
 * Revision 1.9.8.1 2005/11/09 22:50:11 gamel finished Statistic Window
 * 
 * Revision 1.9 2005/10/31 17:09:40 endert Added Button for the Scenario Editor
 * to Toolbar.
 * 
 * Revision 1.8 2005/10/25 20:51:38 endert Added Training-Editor to SoccerScope,
 * restructuring in some old classes and some experimental stuff (goals,
 * commitments, mental model)
 * 
 * Revision 1.7 2005/10/14 14:58:15 endert Now, say-messages are visualized
 * inside in SoccerScope. However, the information comes from the RobocupAgent
 * directly, using the SoccerScope.getInstance() method (in GUI-Mode only), so
 * that it is a work-around. The message itself can be seen inside the
 * "Property"-Menu of the Players Context-Menu, as well as any heared PSI,
 * whereas saying is visualized additionally on the layer (String removed, was
 * to big! Uncomment the drawStrings in SayLayer, to revisualize them).
 * 
 * Revision 1.6 2005/08/16 16:26:47 wetzker merged composite branch
 * 
 * Revision 1.4.4.1 2005/07/14 16:53:54 wetzker + speed in AgentWM is displayed
 * 
 * Revision 1.5 2005/07/14 16:11:53 thyrion added display of position within
 * popupmenu
 * 
 * Revision 1.4 2005/06/15 17:03:17 wetzker removed JIAC from Head + changes in
 * framework please report bugs or shortcomings some scenarios might not work
 * because of wrong imports
 * 
 * Revision 1.2.2.1 2005/06/15 16:45:00 wetzker final version
 * 
 * Revision 1.3 2005/06/12 18:52:16 thyrion fixed saving&loading of our
 * worldmodel (to load first open server log)
 * 
 * Revision 1.2 2005/05/04 07:09:27 thyrion added drop ball & foul left/right to
 * popupmenu
 * 
 * Revision 1.1 2005/04/20 10:39:05 endert Fresh Team Repository installation
 * from /home/e/endert/Robocup to here.
 * 
 * Revision 1.5 2005/03/20 21:39:36 andpaul removed some warnings
 * 
 * 
 * 
 */
