/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope.view;

import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.border.TitledBorder;

import soccerscope.model.ClientScene;
import soccerscope.model.Param;
import soccerscope.model.SoccerObjectID;

public class SelectWindow extends JFrame implements SoccerObjectID,
		ScopeWindow, ActionListener, ItemListener, MouseListener,
		SoccerObjectSelectObserver, SoccerObjectSelector {

	private static final long serialVersionUID = -1524019188307131423L;

	public final static String title = "select";

	private ClientScene clientScene;

	private Container container = null;

	private PlayerButton pb[];

	private JCheckBox cb[];

	private JCheckBox ballcb;

	// SoccerObjectSelectObserverΥꥹ
	private ArrayList observerList;

	private JPopupMenu popup;

	private JMenuItem selectleft;

	private JMenuItem selectright;

	private JMenuItem deselectleft;

	private JMenuItem deselectright;

	public SelectWindow() {
		// frame title
		super("select");

		// 
		this.observerList = new ArrayList();

		// ߡʥ
		this.clientScene = ClientScene.createScene();

		// get content pane
		this.container = this.getContentPane();
		this.container
				.setLayout(new BoxLayout(this.container, BoxLayout.Y_AXIS));
		this.container.addMouseListener(this);

		JPanel leftp = new JPanel();
		leftp.setLayout(new BoxLayout(leftp, BoxLayout.Y_AXIS));
		leftp.setBorder(new TitledBorder("Left"));
		JPanel rightp = new JPanel();
		rightp.setLayout(new BoxLayout(rightp, BoxLayout.Y_AXIS));
		rightp.setBorder(new TitledBorder("Right"));

		this.pb = new PlayerButton[Param.MAX_PLAYER * 2];
		this.cb = new JCheckBox[Param.MAX_PLAYER * 2];
		FlowLayout flowlayout = new FlowLayout(FlowLayout.LEFT, 1, 1);
		for (int i = 0; i < Param.MAX_PLAYER; i++) {
			this.cb[i] = new JCheckBox();
			this.cb[i].addItemListener(this);
			this.pb[i] = new PlayerButton(this.clientScene.player[i]);
			this.pb[i].addActionListener(this);
			JPanel p1 = new JPanel(flowlayout);
			p1.add(this.cb[i]);
			p1.add(this.pb[i]);
			leftp.add(p1);
			this.cb[i + Param.MAX_PLAYER] = new JCheckBox();
			this.cb[i + Param.MAX_PLAYER].addItemListener(this);
			this.pb[i + Param.MAX_PLAYER] = new PlayerButton(
					this.clientScene.player[i + Param.MAX_PLAYER]);
			this.pb[i + Param.MAX_PLAYER].addActionListener(this);
			JPanel p2 = new JPanel(flowlayout);
			p2.add(this.cb[i + Param.MAX_PLAYER]);
			p2.add(this.pb[i + Param.MAX_PLAYER]);
			rightp.add(p2);
		}

		JPanel teamp = new JPanel();
		teamp.setLayout(new BoxLayout(teamp, BoxLayout.X_AXIS));
		teamp.add(leftp);
		teamp.add(rightp);
		this.container.add(teamp);

		this.ballcb = new JCheckBox("Ball");
		this.ballcb.addItemListener(this);
		JPanel ballp = new JPanel(flowlayout);
		ballp.setBorder(new TitledBorder("Others"));
		ballp.add(this.ballcb);
		this.container.add(ballp);

		pack();

		this.popup = new JPopupMenu();
		this.selectleft = new JMenuItem("Select all Left");
		this.selectleft.addActionListener(this);
		this.selectright = new JMenuItem("Select all Right");
		this.selectright.addActionListener(this);
		this.deselectleft = new JMenuItem("Deselect all Left");
		this.deselectleft.addActionListener(this);
		this.deselectright = new JMenuItem("Deselect all Right");
		this.deselectright.addActionListener(this);
		this.popup.add(this.selectleft);
		this.popup.add(this.selectright);
		this.popup.add(this.deselectleft);
		this.popup.add(this.deselectright);
	}

	// ɽ
	public void setScene(ClientScene clientScene) {
		this.clientScene = clientScene;
		for (int i = 0; i < Param.TOTAL_PLAYER; i++) {
			this.pb[i].setPlayer(clientScene.player[i]);
		}
	}

	// observerϿ
	public void addSoccerObjectSelectObserver(SoccerObjectSelectObserver soso) {
		this.observerList.add(soso);
	}

	// ѲobserverΤ
	private void deliverStateChanged(int id, boolean flag) {
		Iterator it = this.observerList.iterator();
		while (it.hasNext())
			((SoccerObjectSelectObserver) it.next()).selectSoccerObject(id,
					flag);
	}

	// Ѳ
	public void selectSoccerObject(int id, boolean flag) {
		if (id == BALL)
			this.ballcb.setSelected(flag);
		else
			this.cb[id].setSelected(flag);
	}

	// ActionListener
	public void actionPerformed(ActionEvent ae) {
		Object o = ae.getSource();
		if (o == this.selectleft) {
			for (int i = LEFT_1; i <= LEFT_11; i++)
				this.cb[i].setSelected(true);
		} else if (o == this.selectright) {
			for (int i = RIGHT_1; i <= RIGHT_11; i++)
				this.cb[i].setSelected(true);
		} else if (o == this.deselectleft) {
			for (int i = LEFT_1; i <= LEFT_11; i++)
				this.cb[i].setSelected(false);
		} else if (o == this.deselectright) {
			for (int i = RIGHT_1; i <= RIGHT_11; i++)
				this.cb[i].setSelected(false);
		} else {
			for (int i = 0; i < Param.TOTAL_PLAYER; i++)
				if (o == this.pb[i])
					this.cb[i].setSelected(!this.cb[i].isSelected());
		}
	}

	// ItemListener
	public void itemStateChanged(ItemEvent ie) {
		Object o = ie.getSource();
		if (o == this.ballcb) {
			deliverStateChanged(BALL, this.ballcb.isSelected());
			return;
		} else {
			for (int i = 0; i < Param.TOTAL_PLAYER; i++)
				if (o == this.cb[i]) {
					deliverStateChanged(i, this.cb[i].isSelected());
					return;
				}
		}
	}

	// MouseListener
	public void mouseClicked(MouseEvent e) {
		// å
		if (e.getModifiers() == InputEvent.META_MASK) {
			this.popup.show(e.getComponent(), e.getX(), e.getY());
		}
	}

	public void mouseEntered(MouseEvent e) {
	}

	public void mouseExited(MouseEvent e) {
	}

	public void mousePressed(MouseEvent e) {
	}

	public void mouseReleased(MouseEvent e) {
	}
}
