/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope.view.layer;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JCheckBox;
import javax.swing.JPanel;

import soccerscope.model.ClientScene;
import soccerscope.model.Param;
import soccerscope.util.geom.Point2f;
import soccerscope.view.FieldPane;

public class DelaunayLayer extends Layer implements ItemListener {

	public DelaunayLayer(FieldPane fieldPane, boolean enable) {
		super(fieldPane, enable);
		this.drawLeft = true;
		this.drawRight = false;
	}

	@Override
	public String getLayerName() {
		return "Delaunay";
	}

	private JCheckBox leftBox;

	private JCheckBox rightBox;

	@Override
	public JPanel getConfigPanel() {
		this.leftBox = new JCheckBox("Left", this.drawLeft);
		this.leftBox.addItemListener(this);
		this.rightBox = new JCheckBox("Right", this.drawRight);
		this.rightBox.addItemListener(this);
		JPanel panel = new JPanel();
		panel.add(this.leftBox);
		panel.add(this.rightBox);

		return panel;
	}

	public void itemStateChanged(ItemEvent ie) {
		Object o = ie.getSource();
		if (o == this.leftBox || o == this.rightBox) {
			this.drawLeft = this.leftBox.isSelected();
			this.drawRight = this.rightBox.isSelected();
			this.fieldPane.repaint();
		}
	}

	private ClientScene clientScene;

	private boolean drawLeft;

	private boolean drawRight;

	@Override
	public void draw(Graphics g) {
		if (!this.enable)
			return;

		this.clientScene = this.fieldPane.getScene();
		float ballMagnify = this.fieldPane.getBallMagnify();
		float playerMagnify = this.fieldPane.getPlayerMagnify();

		if (this.drawLeft) {
			setDelaunay(1, 11);
			for (int i = 1; i < 11 - 1; i++) {
				if (!this.clientScene.player[i].isEnable())
					continue;

				Color drawColor = this.clientScene.player[i].getColor();
				g.setColor(drawColor);
				int n = i * STORAGEWIDTH + i + 1;
				for (int j = i + 1; j < 11; j++) {
					if (this.distance[n] >= 0) {
						drawLine(g, this.clientScene.player[i].pos,
								this.clientScene.player[j].pos);
					}
					n++;
				}
			}
		}
		if (this.drawRight) {
			setDelaunay(12, 22);
			for (int i = 12; i < 22 - 1; i++) {
				if (!this.clientScene.player[i].isEnable())
					continue;

				Color drawColor = this.clientScene.player[i].getColor();
				g.setColor(drawColor);
				int n = i * STORAGEWIDTH + i + 1;
				for (int j = i + 1; j < 22; j++) {
					if (this.distance[n] >= 0) {
						drawLine(g, this.clientScene.player[i].pos,
								this.clientScene.player[j].pos);
					}
					n++;
				}
			}
		}
	}

	private static final int STORAGEWIDTH = Param.MAX_PLAYER * 2 + 4;

	private double distance[] = new double[Param.MAX_PLAYER * 2 * STORAGEWIDTH];

	private void setDelaunay(int minNum, int maxNum) {
		int m, n;
		double a, b, c, x0, y0, x1, y1, x2, y2, x3, y3;
		for (int i = minNum; i < maxNum - 1; i++) {
			Point2f drawPoint0 = new Point2f(this.clientScene.player[i].pos);
			Point2f p0 = this.fieldPane.fieldToScreen(drawPoint0);
			x0 = p0.x;
			y0 = p0.y;
			n = i * STORAGEWIDTH + i + 1;
			for (int j = i + 1; j < maxNum; j++) {
				Point2f drawPoint1 = new Point2f(this.clientScene.player[j].pos);
				Point2f p1 = this.fieldPane.fieldToScreen(drawPoint1);
				x1 = p1.x;
				y1 = p1.y;
				a = b = Math
						.sqrt((x1 - x0) * (x1 - x0) + (y1 - y0) * (y1 - y0));
				loop: for (int k = minNum; k < i; k++) {
					Point2f drawPoint2 = new Point2f(
							this.clientScene.player[k].pos);
					Point2f p2 = this.fieldPane.fieldToScreen(drawPoint2);
					x2 = p2.x;
					y2 = p2.y;
					m = k * STORAGEWIDTH + k + 1;
					for (int l = k + 1; l < maxNum; l++) {
						c = this.distance[m];
						if (l != i && l != j && c >= 0) {
							Point2f drawPoint3 = new Point2f(
									this.clientScene.player[l].pos);
							Point2f p3 = this.fieldPane
									.fieldToScreen(drawPoint3);
							x3 = p3.x;
							y3 = p3.y;
							if (isCross(x0, y0, x1, y1, x2, y2, x3, y3)) {
								if (b > c) {
									a = -1;
									break loop;
								}
							}
						}
						m++;
					}
				}
				this.distance[n] = a;
				if (a >= 0) {
					for (int k = minNum; k < i; k++) {
						Point2f drawPoint2 = new Point2f(
								this.clientScene.player[k].pos);
						Point2f p2 = this.fieldPane.fieldToScreen(drawPoint2);
						x2 = p2.x;
						y2 = p2.y;
						m = k * STORAGEWIDTH + k + 1;
						for (int l = k + 1; l < maxNum; l++) {
							Point2f drawPoint3 = new Point2f(
									this.clientScene.player[l].pos);
							Point2f p3 = this.fieldPane
									.fieldToScreen(drawPoint3);
							x3 = p3.x;
							y3 = p3.y;
							if (l != i && l != j && this.distance[m] >= 0
									&& isCross(x0, y0, x1, y1, x2, y2, x3, y3))
								this.distance[m] = -1;
							m++;
						}
					}
				}
				n++;
			}
		}
	}

	private boolean isCross(double x0, double y0, double x1, double y1,
			double x2, double y2, double x3, double y3) {
		double a, b;

		if (x1 == x0)
			a = 10000;
		else
			a = (y1 - y0) / (x1 - x0);
		b = y0 - a * x0;
		if ((y2 - (a * x2 + b)) * (y3 - (a * x3 + b)) < 0) {
			if (x3 == x2)
				a = 10000;
			else
				a = (y3 - y2) / (x3 - x2);
			b = y2 - a * x2;
			if ((y0 - (a * x0 + b)) * (y1 - (a * x1 + b)) < 0)
				return true;
		}
		return false;
	}
}
