/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope.view.layer;

import java.awt.BasicStroke;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.JCheckBox;
import javax.swing.JPanel;

import soccerscope.model.ClientScene;
import soccerscope.model.Param;
import soccerscope.model.SceneSet;
import soccerscope.model.Team;
import soccerscope.model.WorldModel;
import soccerscope.util.Color2;
import soccerscope.util.analyze.PassAnalyzer;
import soccerscope.view.FieldPane;

public class DynamicSpineLayer extends Layer implements ItemListener {

	public DynamicSpineLayer(FieldPane fieldPane, boolean enable) {
		super(fieldPane, enable);
		this.drawLeft = true;
		this.drawRight = false;
	}

	@Override
	public String getLayerName() {
		return "Dynamic Spine";
	}

	private JCheckBox leftBox;

	private JCheckBox rightBox;

	@Override
	public JPanel getConfigPanel() {
		this.leftBox = new JCheckBox("Left", this.drawLeft);
		this.leftBox.addItemListener(this);
		this.rightBox = new JCheckBox("Right", this.drawRight);
		this.rightBox.addItemListener(this);
		JPanel panel = new JPanel();
		panel.add(this.leftBox);
		panel.add(this.rightBox);

		return panel;
	}

	public void itemStateChanged(ItemEvent ie) {
		Object o = ie.getSource();
		this.drawLeft = this.leftBox.isSelected();
		this.drawRight = this.rightBox.isSelected();
		this.fieldPane.repaint();
	}

	private boolean drawLeft;

	private boolean drawRight;

	private BasicStroke stroke1 = new BasicStroke(1);

	private BasicStroke stroke2 = new BasicStroke(2);

	private BasicStroke stroke3 = new BasicStroke(3);

	private BasicStroke stroke4 = new BasicStroke(4);

	private BasicStroke stroke5 = new BasicStroke(5);

	@Override
	public void draw(Graphics g) {
		if (!this.enable)
			return;

		ClientScene clientScene = this.fieldPane.getScene();
		SceneSet set = WorldModel.getInstance().getSceneSet();

		ArrayList list = PassAnalyzer.getPassList();
		Iterator it = list.iterator();
		int[][] left = new int[Param.MAX_PLAYER + 1][Param.MAX_PLAYER + 1];
		int[][] right = new int[Param.MAX_PLAYER + 1][Param.MAX_PLAYER + 1];

		while (it.hasNext()) {
			PassAnalyzer.Pass pass = (PassAnalyzer.Pass) it.next();
			if (pass.sender.time >= clientScene.time)
				continue;
			if (pass.side == Team.LEFT_SIDE && this.drawLeft) {
				left[pass.sender.unum][pass.receiver.unum]++;
			}
			if (pass.side == Team.RIGHT_SIDE && this.drawRight) {
				right[pass.sender.unum][pass.receiver.unum]++;
			}
		}

		Graphics2D g2 = (Graphics2D) g;
		if (this.drawLeft) {
			for (int i = 1; i < Param.MAX_PLAYER + 1; i++)
				for (int j = i + 1; j < Param.MAX_PLAYER + 1; j++) {
					int count = left[i][j] + left[j][i];
					if (count > 10) {
						g2.setColor(Color2.red);
						g2.setStroke(this.stroke5);
					} else if (count > 5) {
						g2.setColor(Color2.yellow);
						g2.setStroke(this.stroke3);
					} else if (count > 3) {
						g2.setColor(Color2.green);
						g2.setStroke(this.stroke1);
					} else if (count > 0) {
						g2.setColor(Color2.lightGray);
						g2.setStroke(this.stroke1);
					}
					if (count > 0) {
						drawLine(g2, clientScene.player[i - 1].pos,
								clientScene.player[j - 1].pos);
					}
				}
		}
		if (this.drawRight) {
			for (int i = 1; i < Param.MAX_PLAYER + 1; i++)
				for (int j = i + 1; j < Param.MAX_PLAYER + 1; j++) {
					int count = right[i][j] + right[j][i];
					if (count > 10) {
						g2.setColor(Color2.red);
						g2.setStroke(this.stroke5);
					} else if (count > 5) {
						g2.setColor(Color2.yellow);
						g2.setStroke(this.stroke3);
					} else if (count > 3) {
						g2.setColor(Color2.green);
						g2.setStroke(this.stroke1);
					} else if (count > 0) {
						g2.setColor(Color2.lightGray);
						g2.setStroke(this.stroke1);
					}
					if (count > 0) {
						drawLine(g2, clientScene.player[i - 1
								+ Param.MAX_PLAYER].pos, clientScene.player[j
								- 1 + Param.MAX_PLAYER].pos);
					}
				}
		}
		g2.setStroke(new BasicStroke(1));
	}
}
