/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope.view.layer;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.AffineTransform;
import java.awt.geom.Line2D;
import java.awt.geom.PathIterator;

import javax.swing.AbstractButton;
import javax.swing.JCheckBox;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JToggleButton;

import soccerscope.model.Param;
import soccerscope.util.ButtonManager;
import soccerscope.util.geom.Point2f;
import soccerscope.util.geom.Tuple2f;
import soccerscope.view.FieldPane;

public abstract class Layer implements Param {
	protected FieldPane fieldPane;

	protected boolean enable;

	public Layer(FieldPane fieldPane, boolean enable) {
		this.fieldPane = fieldPane;
		this.enable = enable;
	}

	public abstract void draw(Graphics g);

	public abstract String getLayerName();

	// Υ쥤˴ؤѥ֤ͥ
	public JPanel getConfigPanel() {
		return null;
	}

	public boolean isEnabled() {
		return this.enable;
	}

	public void setEnabled(boolean enable) {
		ButtonManager.setSelected(getLayerName(), enable);
		this.fieldPane.repaint();
	}

	public JCheckBox createJCheckBox() {
		JCheckBox checkBox = new JCheckBox(getLayerName(), this.enable);
		checkBox.addActionListener(new LayerAction());
		ButtonManager.addButton(getLayerName(), checkBox);
		return checkBox;
	}

	public JMenuItem createJCheckBoxMenuItem() {
		JCheckBoxMenuItem checkBoxMI = new JCheckBoxMenuItem(getLayerName(),
				this.enable);
		checkBoxMI.addActionListener(new LayerAction());
		ButtonManager.addButton(getLayerName(), checkBoxMI);
		return checkBoxMI;
	}

	public JToggleButton createJToggleButton() {
		JToggleButton button = new JToggleButton(getLayerName(), this.enable);
		button.addActionListener(new LayerAction());
		ButtonManager.addButton(getLayerName(), button);
		return button;
	}

	class LayerAction implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			Layer.this.enable = ((AbstractButton) e.getSource()).isSelected();
			setEnabled(Layer.this.enable);
		}
	}

	/**
	 * *** եɺɸϤˤȤ᥽åɷ ****
	 */
	public void drawString(Graphics g, String s, Point2f point) {
		g.drawString(s, this.fieldPane.F2SX(point.x), this.fieldPane.F2SY(point.y));
	}

	public void drawLine(Graphics g, Point2f p1, Point2f p2) {
		int x1 = this.fieldPane.F2SX(p1.x);
		int y1 = this.fieldPane.F2SY(p1.y);
		int x2 = this.fieldPane.F2SX(p2.x);
		int y2 = this.fieldPane.F2SY(p2.y);
		// if (x1 < 0 || y1 < 0 || x2 < 0 || y2 < 0) return ;
		g.drawLine(x1, y1, x2, y2);
	}

	public void drawArrow(Graphics g, Point2f p1, Point2f p2) {
		int x1 = this.fieldPane.F2SX(p1.x);
		int y1 = this.fieldPane.F2SY(p1.y);
		int x2 = this.fieldPane.F2SX(p2.x);
		int y2 = this.fieldPane.F2SY(p2.y);
		if (p1.dist(p2) > 2) {// ưΥ2mʾ˴ؤƤɽ
			int x3 = (10 * x2 + x1) / 11;
			int y3 = (10 * y2 + y1) / 11;
			Line2D line1 = new Line2D.Float(x2, y2, x3, y3);
			AffineTransform affine1 = new AffineTransform();
			affine1.rotate(Math.PI / 6, x2, y2);
			Line2D line2 = new Line2D.Float(x2, y2, x3, y3);
			AffineTransform affine2 = new AffineTransform();
			affine2.rotate(-Math.PI / 6, x2, y2);
			Graphics2D g2 = (Graphics2D) g;
			g2.draw(affine1.createTransformedShape(line1));
			g2.draw(affine2.createTransformedShape(line2));
		}
		// if (x1 < 0 || y1 < 0 || x2 < 0 || y2 < 0) return ;
		g.drawLine(x1, y1, x2, y2);
	}

	public void drawRect(Graphics g, Point2f topleft, Tuple2f size) {
		g.drawRect(this.fieldPane.F2SX(topleft.x), this.fieldPane.F2SY(topleft.y),
				this.fieldPane.F2SSX(size.x), this.fieldPane.F2SSY(size.y));
	}

	public void fillRect(Graphics g, Point2f topleft, Tuple2f size) {
		g.fillRect(this.fieldPane.F2SX(topleft.x), this.fieldPane.F2SY(topleft.y),
				this.fieldPane.F2SSX(size.x), this.fieldPane.F2SSY(size.y));
	}

	public void drawOval(Graphics g, Point2f topleft, Tuple2f size) {
		g.drawOval(this.fieldPane.F2SX(topleft.x), this.fieldPane.F2SY(topleft.y),
				this.fieldPane.F2SSX(size.x), this.fieldPane.F2SSY(size.y));
	}

	public void fillOval(Graphics g, Point2f topleft, Tuple2f size) {
		g.fillOval(this.fieldPane.F2SX(topleft.x), this.fieldPane.F2SY(topleft.y),
				this.fieldPane.F2SSX(size.x), this.fieldPane.F2SSY(size.y));
	}

	public void drawArc(Graphics g, Point2f topleft, Tuple2f size,
			int startAngle, int arcAngle) {
		g.drawArc(this.fieldPane.F2SX(topleft.x), this.fieldPane.F2SY(topleft.y),
				this.fieldPane.F2SSX(size.x), this.fieldPane.F2SSY(size.y), startAngle,
				arcAngle);
	}

	public void fillArc(Graphics g, Point2f topleft, Tuple2f size,
			int startAngle, int arcAngle) {
		g.fillArc(this.fieldPane.F2SX(topleft.x), this.fieldPane.F2SY(topleft.y),
				this.fieldPane.F2SSX(size.x), this.fieldPane.F2SSY(size.y), startAngle,
				arcAngle);
	}

	public void drawCircle(Graphics g, Point2f point, float r) {
		g.drawOval(this.fieldPane.F2SX(point.x - r), this.fieldPane.F2SY(point.y - r),
				this.fieldPane.F2SSX(r * 2), this.fieldPane.F2SSY(r * 2));
	}

	public void fillCircle(Graphics g, Point2f point, float r) {
		g.fillOval(this.fieldPane.F2SX(point.x - r), this.fieldPane.F2SY(point.y - r),
				this.fieldPane.F2SSX(r * 2), this.fieldPane.F2SSY(r * 2));
	}

	// 濴濴Ȥž
	public void drawRect(Graphics g, Point2f topleft, Tuple2f size, float rotate) {
		Graphics2D g2 = (Graphics2D) g;
		g2.translate(this.fieldPane.F2SX(topleft.x + size.x / 2), this.fieldPane
				.F2SY(topleft.y + size.y / 2));
		g2.rotate(Math.toRadians(rotate));
		g2.drawRect(this.fieldPane.F2SSX(-size.x / 2), this.fieldPane.F2SSY(-size.y / 2),
				this.fieldPane.F2SSX(size.x), this.fieldPane.F2SSY(size.y));
		g2.rotate(Math.toRadians(-rotate));
		g2.translate(-this.fieldPane.F2SX(topleft.x + size.x / 2), -this.fieldPane
				.F2SY(topleft.y + size.y / 2));
	}

	// 濴濴Ȥž
	public void fillRect(Graphics g, Point2f topleft, Tuple2f size, float rotate) {
		Graphics2D g2 = (Graphics2D) g;
		g2.translate(this.fieldPane.F2SX(topleft.x + size.x / 2), this.fieldPane
				.F2SY(topleft.y + size.y / 2));
		g2.rotate(Math.toRadians(rotate));
		g2.fillRect(this.fieldPane.F2SSX(-size.x / 2), this.fieldPane.F2SSY(-size.y / 2),
				this.fieldPane.F2SSX(size.x), this.fieldPane.F2SSY(size.y));
		g2.rotate(Math.toRadians(-rotate));
		g2.translate(-this.fieldPane.F2SX(topleft.x + size.x / 2), -this.fieldPane
				.F2SY(topleft.y + size.y / 2));
	}

	// center濴ȤƲž
	public void drawRect(Graphics g, Point2f topleft, Tuple2f size,
			float rotate, Point2f center) {
		Graphics2D g2 = (Graphics2D) g;
		g2.translate(this.fieldPane.F2SX(center.x), this.fieldPane.F2SY(center.y));
		g2.rotate(Math.toRadians(rotate));
		g2.drawRect(this.fieldPane.F2SSX(topleft.x - center.x), this.fieldPane
				.F2SSY(topleft.y - center.y), this.fieldPane.F2SSX(size.x),
				this.fieldPane.F2SSY(size.y));
		g2.rotate(Math.toRadians(-rotate));
		g2.translate(-this.fieldPane.F2SX(center.x), -this.fieldPane.F2SY(center.y));
	}

	// center濴ȤƲž
	public void fillRect(Graphics g, Point2f topleft, Tuple2f size,
			float rotate, Point2f center) {
		Graphics2D g2 = (Graphics2D) g;
		g2.translate(this.fieldPane.F2SX(center.x), this.fieldPane.F2SY(center.y));
		g2.rotate(Math.toRadians(rotate));
		g2.fillRect(this.fieldPane.F2SSX(topleft.x - center.x), this.fieldPane
				.F2SSY(topleft.y - center.y), this.fieldPane.F2SSX(size.x),
				this.fieldPane.F2SSY(size.y));
		g2.rotate(Math.toRadians(-rotate));
		g2.translate(-this.fieldPane.F2SX(center.x), -this.fieldPane.F2SY(center.y));
	}

	// ʱߤ濴濴Ȥž
	public void drawOval(Graphics g, Point2f topleft, Tuple2f size, float rotate) {
		Graphics2D g2 = (Graphics2D) g;
		g2.translate(this.fieldPane.F2SX(topleft.x + size.x / 2), this.fieldPane
				.F2SY(topleft.y + size.y / 2));
		g2.rotate(Math.toRadians(rotate));
		g2.drawOval(this.fieldPane.F2SSX(-size.x / 2), this.fieldPane.F2SSY(-size.y / 2),
				this.fieldPane.F2SSX(size.x), this.fieldPane.F2SSY(size.y));
		g2.rotate(Math.toRadians(-rotate));
		g2.translate(-this.fieldPane.F2SX(topleft.x + size.x / 2), -this.fieldPane
				.F2SY(topleft.y + size.y / 2));
	}

	private float coords[] = new float[6];

	private Point2f startPos = new Point2f();

	private Point2f endPos = new Point2f();

	public void drawPath(Graphics g, PathIterator pit) {
		int type;
		do {
			type = pit.currentSegment(this.coords);
			switch (type) {
			case PathIterator.SEG_MOVETO:
				this.startPos.set(this.coords[0], this.coords[1]);
				break;
			case PathIterator.SEG_LINETO:
				this.endPos.set(this.coords[0], this.coords[1]);
				drawLine(g, this.startPos, this.endPos);
				this.startPos.set(this.endPos);
				break;
			case PathIterator.SEG_QUADTO:
				break;
			case PathIterator.SEG_CUBICTO:
				break;
			case PathIterator.SEG_CLOSE:
				return;
			default:
				return;
			}
			pit.next();
		} while (!pit.isDone());
	}

	public void drawImage(Graphics g, Image img, Point2f point) {
		g.drawImage(img, this.fieldPane.F2SX(point.x), this.fieldPane.F2SY(point.y),
				this.fieldPane);
	}

	public void drawImage(Graphics g, Image img, Point2f point, int offsetx,
			int offsety) {
		g.drawImage(img, this.fieldPane.F2SX(point.x) + offsetx, this.fieldPane
				.F2SY(point.y)
				+ offsety, this.fieldPane);
	}

}
