/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope.view.layer;

import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import soccerscope.model.ClientScene;
import soccerscope.model.Param;
import soccerscope.model.SceneSet;
import soccerscope.model.SoccerObjectID;
import soccerscope.model.WorldModel;
import soccerscope.util.Color2;
import soccerscope.view.FieldPane;

// ʪΤΥ⡼褹쥤䡼

public class MotionLayer extends Layer implements ActionListener {

	public MotionLayer(FieldPane fieldPane, boolean enable) {
		super(fieldPane, enable);
	}

	@Override
	public String getLayerName() {
		return "Motion";
	}

	private JTextField upperText;

	private JTextField lowerText;

	@Override
	public JPanel getConfigPanel() {
		JLabel upperLabel = new JLabel("after");
		JLabel lowerLabel = new JLabel("before");
		this.upperText = new JTextField(String.valueOf(this.upper), 4);
		this.lowerText = new JTextField(String.valueOf(this.lower), 4);
		JButton apply = new JButton("Apply");
		this.upperText.addActionListener(this);
		this.lowerText.addActionListener(this);
		apply.addActionListener(this);

		JPanel p = new JPanel();
		p.setLayout(new GridBagLayout());
		GridBagConstraints gbc1 = new GridBagConstraints();
		GridBagConstraints gbc2 = new GridBagConstraints();
		gbc1.anchor = GridBagConstraints.NORTHWEST;
		gbc1.ipadx = 5;
		gbc1.ipady = 5;
		gbc1.gridx = 0;
		gbc1.weightx = 0.0;
		gbc2.anchor = GridBagConstraints.NORTHWEST;
		gbc2.gridx = 1;
		gbc2.weightx = 1.0;

		p.add(lowerLabel, gbc1);
		p.add(upperLabel, gbc1);

		p.add(this.lowerText, gbc2);
		p.add(this.upperText, gbc2);

		gbc1.weighty = 1.0;
		gbc1.gridwidth = 2;
		p.add(apply, gbc1);

		return p;
	}

	public void actionPerformed(ActionEvent ae) {
		try {
			this.upper = Integer.parseInt(this.upperText.getText());
			this.lower = Integer.parseInt(this.lowerText.getText());
			this.fieldPane.repaint();
		} catch (NumberFormatException nfe) {
		}
	}

	private int upper = 5;

	private int lower = 5;

	@Override
	public void draw(Graphics g) {
		if (!this.enable)
			return;

		ClientScene clientScene = this.fieldPane.getScene();
		float ballMagnify = this.fieldPane.getBallMagnify();
		float playerMagnify = this.fieldPane.getPlayerMagnify();

		float ballR = Param.BALL_SIZE * ballMagnify;
		float playerR = Param.PLAYER_SIZE * playerMagnify;

		WorldModel wm = WorldModel.getInstance();
		SceneSet sceneSet = wm.getSceneSet();

		for (int offset = this.upper; offset > 0; offset--) {
			if (sceneSet.hasScene(clientScene.time + offset)) {
				ClientScene tmpClientScene = (ClientScene) sceneSet
						.getScene(clientScene.time + offset);
				if (tmpClientScene.ball.isEnable()
						&& this.fieldPane.isSelected(SoccerObjectID.BALL)) {
					g.setColor(Color2.mix(tmpClientScene.ball.getColor(),
							Color2.forestGreen, 1, offset));
					fillCircle(g, tmpClientScene.ball.pos, ballR);
				}
				for (int i = 0; i < Param.MAX_PLAYER * 2; i++) {
					if (!tmpClientScene.player[i].isEnable()
							|| !this.fieldPane.isSelected(i))
						continue;
					g.setColor(Color2.mix(tmpClientScene.player[i].getColor(),
							Color2.forestGreen, 1, offset));
					fillCircle(g, tmpClientScene.player[i].pos,
							tmpClientScene.player[i].getSize() * playerMagnify);
				}
			}
		}
		for (int offset = this.lower; offset > 0; offset--) {
			if (sceneSet.hasScene(clientScene.time - offset)) {
				ClientScene tmpClientScene = (ClientScene) sceneSet
						.getScene(clientScene.time - offset);
				if (tmpClientScene.ball.isEnable()
						&& this.fieldPane.isSelected(SoccerObjectID.BALL)) {
					g.setColor(Color2.mix(tmpClientScene.ball.getColor(),
							Color2.forestGreen, 1, offset));
					fillCircle(g, tmpClientScene.ball.pos, ballR);
				}
				for (int i = 0; i < Param.MAX_PLAYER * 2; i++) {
					if (!tmpClientScene.player[i].isEnable()
							|| !this.fieldPane.isSelected(i))
						continue;
					g.setColor(Color2.mix(tmpClientScene.player[i].getColor(),
							Color2.forestGreen, 1, offset));
					fillCircle(g, tmpClientScene.player[i].pos,
							tmpClientScene.player[i].getSize() * playerMagnify);
				}
			}
		}
	}
}
