/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope.view.layer;

import java.awt.Color;
import java.awt.Graphics;

import soccerscope.model.ClientScene;
import soccerscope.model.Param;
import soccerscope.model.SoccerObjectID;
import soccerscope.util.geom.Point2f;
import soccerscope.view.FieldPane;

// ץ쥤䡼褹쥤䡼

public class OfflineMoveLayer extends Layer implements SoccerObjectID {

	Point2f point[];

	Point2f startPoint;

	Point2f movePoint;

	ClientScene clientScene;

	boolean is_test;

	public OfflineMoveLayer(FieldPane fieldPane, boolean enable) {
		super(fieldPane, enable);
		ClientScene.createScene();
		this.point = new Point2f[MAX_ID];
	}

	@Override
	public String getLayerName() {
		return "OfflineMove";
	}

	private Point2f drawPoint = new Point2f();

	private Point2f drawSize = new Point2f();

	public void setEnable(boolean enable) {
		this.enable = enable;
	}

	public void setTest(boolean is_test) {
		this.is_test = is_test;
	}

	public void setEnable(boolean enable, ClientScene clientScene, Point2f pos) {
		this.enable = enable;
		this.startPoint = new Point2f(pos);
		this.movePoint = new Point2f(pos);
		this.clientScene = clientScene;
		for (int i = 0; i < MAX_PLAYER * 2; i++)
			this.point[i] = new Point2f(clientScene.player[i].pos);
		this.point[BALL] = new Point2f(clientScene.ball.pos);
	}

	public void move(Point2f point) {
		this.movePoint = new Point2f(point);
	}

	public Point2f getMovePoint(int id) {
		this.drawPoint = new Point2f(this.point[id]);
		Point2f currentPoint = new Point2f(this.movePoint);
		currentPoint.x -= this.startPoint.x;
		currentPoint.y -= this.startPoint.y;
		this.drawPoint.x += currentPoint.x;
		this.drawPoint.y += currentPoint.y;
		return this.drawPoint;
	}

	public void setPosition(Point2f p[]) {
		for (int i = 0; i < Param.MAX_PLAYER; i++)
			this.point[i] = p[i];
	}

	@Override
	public void draw(Graphics g) {
		if (!this.enable)
			return;
		float ballMagnify = this.fieldPane.getBallMagnify();
		float playerMagnify = this.fieldPane.getPlayerMagnify();
		Point2f currentPoint = new Point2f(this.movePoint);
		currentPoint.x -= this.startPoint.x;
		currentPoint.y -= this.startPoint.y;

		// Ball
		if (this.fieldPane.isOfflineSelected(BALL)) {
			if (!this.clientScene.ball.isEnable())
				return;

			float ballR = Param.BALL_SIZE * ballMagnify;
			this.drawPoint = new Point2f(this.point[BALL]);
			this.drawPoint.x += currentPoint.x;
			this.drawPoint.y += currentPoint.y;
			g.setColor(this.clientScene.ball.getColor());
			drawCircle(g, this.drawPoint, ballR);
		}

		// Player
		// ե᡼ƥ
		if (this.is_test) {
			for (int i = 0; i < MAX_PLAYER; i++) {
				float playerR = this.clientScene.player[i].getSize()
						* playerMagnify;
				Color drawColor = this.clientScene.player[i].getColor();
				this.drawPoint = this.point[i];
				g.setColor(drawColor);
				drawCircle(g, this.drawPoint, playerR);
			}
		}
		int START = (is_test) ? MAX_PLAYER : 0;
		for (int i = START; i < MAX_PLAYER * 2; i++) {
			if (!this.clientScene.player[i].isEnable())
				continue;
			if (!this.fieldPane.isOfflineSelected(i))
				continue;
			float playerR = this.clientScene.player[i].getSize()
					* playerMagnify;
			Color drawColor = this.clientScene.player[i].getColor();

			this.drawPoint = new Point2f(this.point[i]);
			this.drawPoint.x += currentPoint.x;
			this.drawPoint.y += currentPoint.y;

			g.setColor(drawColor);
			drawCircle(g, this.drawPoint, playerR);
		}
	}
}
