/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope.view.layer;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;

import javax.swing.ImageIcon;

import soccerscope.model.ClientScene;
import soccerscope.model.Player;
import soccerscope.util.geom.Point2f;
import soccerscope.view.FieldPane;

// ץ쥤䡼褹쥤䡼

public class PlayerLayer extends Layer {

	public PlayerLayer(FieldPane fieldPane) {
		super(fieldPane, true);
	}

	@Override
	public String getLayerName() {
		return "Player";
	}

	private Point2f drawPoint = new Point2f();

	private Point2f drawSize = new Point2f();

	private Image selectImage = new ImageIcon(ClassLoader
			.getSystemResource("soccerscope/image/select.gif")).getImage();

	@Override
	public void draw(Graphics g) {
		if (!this.enable) {
			return;
		}
		ClientScene clientScene = this.fieldPane.getScene();
		float ballMagnify = this.fieldPane.getBallMagnify();
		float playerMagnify = this.fieldPane.getPlayerMagnify();

		for (int i = 0; i < MAX_PLAYER * 2; i++) {
			if (!clientScene.player[i].isEnable())
				continue;

			float playerR = clientScene.player[i].getSize() * playerMagnify;
			Color drawColor = clientScene.player[i].getColor();

			// draw mode
			g.setColor(Color.black);

			if (clientScene.player[i].isCollision()) {
				drawCircle(g, clientScene.player[i].pos, playerR + 0.1f);
			}

			if (clientScene.player[i].isKicking()
					|| clientScene.player[i].isTackling()
					|| clientScene.player[i].isCatching()) {
				g.setColor(drawColor.darker());
			} else if (clientScene.player[i].isKickFault()
					|| clientScene.player[i].isTackleFault()
					|| clientScene.player[i].isCatchFault()) {
				g.setColor(drawColor.darker().darker());
			}
			if (this.fieldPane.isOfflineSelected(i))
				g.setColor(drawColor.brighter());
			fillCircle(g, clientScene.player[i].pos, playerR);

			if (clientScene.player[i].isGoalie()
					&& clientScene.player[i].catchDir != Player.ERROR_DIR) {
				g.setColor(drawColor.brighter());
				drawRect(g, new Point2f(clientScene.player[i].pos.x,
						clientScene.player[i].pos.y
								- GOALIE_CATCHABLE_AREA_WIDTH / 2),
						new Point2f(GOALIE_CATCHABLE_AREA_LENGTH,
								GOALIE_CATCHABLE_AREA_WIDTH),
						(int) clientScene.player[i].angle
								+ clientScene.player[i].catchDir,
						clientScene.player[i].pos);
			}

			// draw body angle
			g.setColor(drawColor);
			this.drawSize.set(playerR, playerR);
			this.drawPoint.sub(clientScene.player[i].pos, this.drawSize);
			this.drawSize.scale(2);
			fillArc(g, this.drawPoint, this.drawSize,
					-((int) clientScene.player[i].angle + 90), 180);

			// draw body
			g.setColor(drawColor);
			drawCircle(g, clientScene.player[i].pos, playerR);

			if (this.fieldPane.isSelected(i)) {
				this.drawPoint.set(0, -playerR);
				this.drawPoint.add(clientScene.player[i].pos);
				drawImage(g, this.selectImage, this.drawPoint,
						-this.selectImage.getWidth(null) / 2, -this.selectImage
								.getHeight(null));
			}
		}
	}
}
