/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope.view.layer;

import java.awt.Color;
import java.awt.Graphics;

import soccerscope.model.ClientScene;
import soccerscope.model.Param;
import soccerscope.util.Color2;
import soccerscope.util.geom.Point2f;
import soccerscope.view.FieldPane;

public class VisibleLayer extends Layer {

	public VisibleLayer(FieldPane fieldPane, boolean enable) {
		super(fieldPane, enable);
	}

	@Override
	public String getLayerName() {
		return "Visible";
	}

	private Point2f drawPoint = new Point2f();

	@Override
	public void draw(Graphics g) {
		if (!this.enable)
			return;

		ClientScene clientScene = this.fieldPane.getScene();
		float ballMagnify = this.fieldPane.getBallMagnify();
		float playerMagnify = this.fieldPane.getPlayerMagnify();

		for (int i = 0; i < Param.MAX_PLAYER * 2; i++) {
			if (!clientScene.player[i].isEnable()
					|| !this.fieldPane.isSelected(i))
				continue;

			float playerR = clientScene.player[i].getSize() * playerMagnify;
			Color drawColor = clientScene.player[i].getColor();

			int a = (int) (-(clientScene.player[i].angle + clientScene.player[i].angleNeck) - clientScene.player[i].angleVisible / 2.0);

			// ֹỤ̃狼ʤꥢ
			this.drawPoint.sub(clientScene.player[i].pos, pitchSize2);
			g
					.setColor(Color2.mix(drawColor,
							Color2.forestGreen.darker(), 1, 2));
			fillArc(g, this.drawPoint, pitchSize4, a,
					clientScene.player[i].angleVisible);

			// Ụ̃狼ʤʤäƤ륨ꥢ
			this.drawPoint.sub(clientScene.player[i].pos, teamTooFarSize);
			g.setColor(Color2.mix(drawColor, Color2.forestGreen.darker(), 1,
					1.5));
			fillArc(g, this.drawPoint, teamTooFarSize2, a,
					clientScene.player[i].angleVisible);

			// ֹ椬狼ʤʤäƤ륨ꥢ
			this.drawPoint.sub(clientScene.player[i].pos, unumTooFarSize);
			g
					.setColor(Color2.mix(drawColor,
							Color2.forestGreen.darker(), 1, 1));
			fillArc(g, this.drawPoint, unumTooFarSize2, a,
					clientScene.player[i].angleVisible);

			// visible distance
			this.drawPoint.sub(clientScene.player[i].pos, visibleSize);
			fillOval(g, this.drawPoint, visibleSize2);
		}
	}
}
